<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;

use App\Models\Providers\Provider;
use App\Models\Customers\Customer;
use App\Models\Notifications\ScheduledNotification;
use App\Jobs\CustomerCommonMailJob;
use App\Notifications\Customers\Notification;

class ProviderFollowUpMessage extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'provider:follow_up_message';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Send follow up message to customer.';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        $date = date('Y-m-d H:i:s');

        $scheduledNotifications =  ScheduledNotification::where(['send_to'=>'customer','is_sent'=>'no'])->where('scheduled_date_time','<=',$date)->get();
       
        if($scheduledNotifications->count())
        {
            foreach($scheduledNotifications as $key => $scheduledNotification) 
            {
               $customer = Customer::where(['id'=>$scheduledNotification->receiver_id])->first(); 
               if($customer)
               {
                    /*  push notification   */
                        $deviceToken = $customer->device_token; 
                        $title = $scheduledNotification->title;
                        $message = $scheduledNotification->message;
                        sendPushNotification($deviceToken,$title,$message);
                    /* end push notification   */  

                    /*  database notification  */
                         $details = [
                             'title'    => $title,
                             'message'    => $message,
                         ];
                         $customer->notify(new Notification($details));
                    /*  end database */

                    /* mail  */
                    $greeting = 'Dear '.decryptString($customer->first_name).' '.decryptString($customer->last_name);
                    $body     =  $scheduledNotification->message;
                    $subject     =  $scheduledNotification->title;
                    
                    $sendMail = new CustomerCommonMailJob(decryptString($customer->email), $greeting, $body, $subject);
                    dispatch($sendMail);
                    /* end mail  */

                    ScheduledNotification::where(['id'=>$scheduledNotification->id])->update(['is_sent'=>'yes']);
               }    
            }
            echo "Follow up message cummand run successfully!";
        }
    }
}
