<?php
use Twilio\Rest\Client;
use App\Jobs\SendEmailJob;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Storage;
use App\Models\EmailTemplate\EmailTemplate;
use App\Models\LoginActivity\LoginActivity;
use App\Models\Settings\Setting;
use Carbon\Carbon;
use App\Models\Customers\Customer;

if (!function_exists('app_path')) {
    /**
     * Get the path to the application folder.
     *
     * @param  string $path
     * @return string
     */
    function app_path($path = '')
    {
        return app('path') . ($path ? DIRECTORY_SEPARATOR . $path : $path);
    }
}

if (!function_exists('public_path')) {
    /**
     * Return the path to public dir
     *
     * @param null $path
     *
     * @return string
     */
    function public_path($path = null)
    {
        return rtrim(app()->basePath('public/' . $path), '/');
    }
}


if (!function_exists('pre')) {
    
    function pre($array)
    {
        echo '<pre>';
        print_r($array);
        echo '</pre>';
        die;
    }
}


if (!function_exists('has_scope')) {
    
   /**
    * Validate user scopes and return user_id.
    * 
    * @param string $scope ( Optional )
    * @return int
    */
   function has_scope($scope)
   {
       try {

           $response = new OAuth2\Response();
           $oauth = new App\Http\Controllers\Auth\OauthServerController();

           // verify the JWT Access Token in the request
           if (!$oauth->server->verifyResourceRequest(OAuth2\Request::createFromGlobals(), $response, $scope)) {

               // if the scope required is different from what the token allows, this will send a "401 insufficient_scope" error
               $response->send();
               die;
           }
           
           return true;           
       }

       catch (\Exception $e) {

           echo $e->getMessage();
           die;
       }
   }
}


function image_to_base64($path)
{
    if (!file_exists($path) || !is_file($path)) {

        return null;
    }
    
    $type = pathinfo($path, PATHINFO_EXTENSION);
    $data = file_get_contents($path);
    $base64 = 'data:image/' . $type . ';base64,' . base64_encode($data);
    return $base64;
}

function return_json($response = array()) {
    header('Content-Type: application/json');
    exit(json_encode($response,JSON_UNESCAPED_SLASHES));
}



// geberate random password
function randomPassword($len = 8) {

    //enforce min length 8
    if($len < 8)
        $len = 8;

    //define character libraries - remove ambiguous characters like iIl|1 0oO
    $sets = array();
    $sets[] = 'ABCDEFGHJKLMNPQRSTUVWXYZ';
    $sets[] = 'abcdefghjkmnpqrstuvwxyz';
    $sets[] = '23456789';
    $sets[]  = '!@#$%&*';

    $password = '';
    
    //append a character from each set - gets first 4 characters
    foreach ($sets as $set) {
        $password .= $set[array_rand(str_split($set))];
    }

    //use all characters to fill up to $len
    while(strlen($password) < $len) {
        //get a random set
        $randomSet = $sets[array_rand($sets)];
        
        //add a random char from the random set
        $password .= $randomSet[array_rand(str_split($randomSet))]; 
    }
    
    //shuffle the password string before returning!
    return str_shuffle($password);
}

//push notification 
function sendPushNotification($token,$title,$message) 
{
    $ch = curl_init("https://fcm.googleapis.com/fcm/send");
   
    //The device token.
    $token = $token; //token here

    //Title of the Notification.
    $title = $title;

    //Body of the Notification.
    $body = $message;

    //Creating the notification array.
    $notification = array('title' => $title, 'body' => $body, 'sound' => 'default');

    //This array contains, the token and the notification. The 'to' attribute stores the token.
    $arrayToSend = array(
                        'to' => $token,
                        'notification' => $notification,
                        'data' => $notification,
                        'priority'=>'high'     
                    );

    //Generating JSON encoded string form the above array.
    $json = json_encode($arrayToSend);

    //Setup headers:
    $headers = array();
    $headers[] = 'Content-Type: application/json';
    $headers[] = 'Authorization: key= AAAABB6DaKc:APA91bFKX20iUuiS24V5x8QfzfGDuOzcRfC8k6SwLVO06iVCseRLKvsgMtr2TIjsrLCFa3Hehsoy34X0kpYJAVMKxkNwH1bJQIrAuBrupTXdo6xRP1HcYAXm6x1Z6zNGa2wrSgY9ZTyl'; // key here

    //Setup curl, add headers and post parameters.
    curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
    curl_setopt($ch, CURLOPT_POSTFIELDS, $json);
    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);


    //Send the request
    $response = curl_exec($ch);
    //Close request
    curl_close($ch);
    return $response;
}

function sendCustomerPushNotification($token,$title,$message) 
{
    $ch = curl_init("https://fcm.googleapis.com/fcm/send");
   
    //The device token.
    $token = $token; //token here

    //Title of the Notification.
    $title = $title;

    //Body of the Notification.
    $body = $message;

    //Creating the notification array.
    $notification = array('title' => $title, 'body' => $body, 'sound' => 'default');

    //This array contains, the token and the notification. The 'to' attribute stores the token.
    $arrayToSend = array(
                        'to' => $token,
                        'notification' => $notification,
                        'data' => $notification,
                        'priority'=>'high'     
                    );

    //Generating JSON encoded string form the above array.
    $json = json_encode($arrayToSend);

    //Setup headers:
    $headers = array();
    $headers[] = 'Content-Type: application/json';
    $headers[] = 'Authorization: key= AAAA_rEo1P4:APA91bFMn9SnuX4HtYwEH0m-VZr9918fW89FWiwqa2IYt7tIoqCdklh9YhCnkCCS-RS3oCIwxzh9Xorl84caIPx__KVj8heCuZz0LHqwSt1X0imSmdIz69tGzqYTBVM--OGOFWa50-SA'; // key here

    //Setup curl, add headers and post parameters.
    curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
    curl_setopt($ch, CURLOPT_POSTFIELDS, $json);
    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);


    //Send the request
    $response = curl_exec($ch);
    //Close request
    curl_close($ch);
    return $response;
}



    //group push notification 
function pushGroupNotification($token,$message) {
    $ch = curl_init("https://fcm.googleapis.com/fcm/send");
   
    //The device token.
    $token = $token; //token here

    //Title of the Notification.
    $title = $message;

    //Body of the Notification.
    $body = $message;

    //Creating the notification array.
    $notification = array('title' => $title, 'body' => $body, 'sound' => 'default');

    //This array contains, the token and the notification. The 'to' attribute stores the token.
    $arrayToSend = array(
        'registration_ids' => $token,
        'notification' => [
            'title' => $title,
            'body' => $body,
            'sound' => 'default',
        ],
        'data' => [
            'click_action' => "FLUTTER_NOTIFICATION_CLICK",
        ]
    );

    //Generating JSON encoded string form the above array.
    $json = json_encode($arrayToSend);

    //Setup headers:
    $headers = array();
    $headers[] = 'Content-Type: application/json';
    $headers[] = 'Authorization: key= AAAADYMwkaU:APA91bEX4B1Wq_lzYov4cfQJPi3N-cBfLFwr8taiTxZhSim_cGo4iD6WyV5G-c-2SdigioJDjZZUPj-lsEmbHzR7DKpWmThuFUvuy8KMRaCWgB-qY6VHnuIgYJCK-JbLXsAQB_ES_TqL'; // key here

    //Setup curl, add headers and post parameters.
    curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
    curl_setopt($ch, CURLOPT_POSTFIELDS, $json);
    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);


    //Send the request
    $response = curl_exec($ch);
    //Close request
    curl_close($ch);
    return $response;
} 

//sms
function sendMessage($message, $recipients){
        $account_sid   = env("TWILIO_SID");
        $auth_token    = env("TWILIO_AUTH_TOKEN");
        $twilio_number = env("TWILIO_NUMBER");
        $client  = new Client($account_sid, $auth_token);
        $response = $client->messages->create($recipients,
            ['from' => $twilio_number, 'body' => $message]
        );
        return $response->sid;
}


function _k(){
    $slt = _slt(32); 
    $str = _str(48); 
    return base64_encode(md5(sha1($slt.$str)));
}

function _slt($l=32){
    $r = '';
    $i = 0;
    while($i <= $l){
        $r .= _s($i); if(strlen($r) == $l) break;
        $r .= _a($i); if(strlen($r) == $l) break;
        $r .= _n($i); if(strlen($r) == $l) break;
        $i++;
    }
    return $r;
}

function _str($l=32){
    $r = '';
    $i = 0;
    while($i <= $l){
        $r .= _a($i); if(strlen($r) == $l) break;
        $r .= _s($i); if(strlen($r) == $l) break;
        $r .= _a($i); if(strlen($r) == $l) break;
        $r .= _n($i); if(strlen($r) == $l) break;
        $i++;
    }
    return $r;
}

function _a($i=0){
    $a = range('a', 'l');
    return !empty($a[$i]) ? $a[$i] : $a[0];
}

function _s($i=0){
    $s = '!@#$%^&*()+-';
    return !empty($s[$i]) ? $s[$i] : $s[0];
}

function _n($i=0){
    $n = range(0, 11);
    return !empty($n[$i]) ? $n[$i] : $n[0];
}


function encryptString($value)
{
    $cipher = env('CIPHER');
    $encryption_key = _k();
    $encryption_key = base64_decode($encryption_key);
    $iv = env('IV');
    return openssl_encrypt($value, $cipher, $encryption_key, 0, $iv);
    //return $value;
}

function decryptString($value)
{
    $cipher = env('CIPHER');
    $encryption_key = _k();
    $encryption_key = base64_decode($encryption_key);
    $iv = env('IV');
    return openssl_decrypt($value, $cipher, $encryption_key, 0, $iv);
    //return $value;
}    

function sendEmail($mailData,$templateName)
{
    $templateData = EmailTemplate::where('name',$templateName)->first();
    if($templateData)
    {
        dispatch(new SendEmailJob($mailData,$templateData));
    }
}

function uploadFile($file,$directoryName,$image)
{
    /* start s3 */
    $client = Storage::disk('s3')->getDriver()->getAdapter()->getClient();
    $bucket = Config::get('filesystems.disks.s3.bucket');
    $info = $client->doesObjectExist($bucket, $directoryName);
    if (!$info)
    {
       Storage::makeDirectory($directoryName);
       $destinationPath = $directoryName.'/'.$image;
       Storage::disk('s3')->put($destinationPath, file_get_contents($file));
    }
    /*  end s3 */
}

function remove_image_s3_bucket($directoryName,$file){
    $destinationPath = $directoryName.'/'.$file;
    Storage::disk('s3')->delete($destinationPath);
}

function presignedUrl($directory,$doc)
{
    // $client = Storage::disk('s3')->getDriver()->getAdapter()->getClient();
    // $bucket = Config::get('filesystems.disks.s3.bucket');
    // $response = $client->doesObjectExist($bucket, $directory.'/'.$doc);
    // if($response){
    //     $command = $client->getCommand('GetObject', [
    //         'Bucket' => $bucket,
    //         'Key' => $directory.'/'.$doc
    //     ]);
    //     $request = $client->createPresignedRequest($command, '+10 minutes');
    //     return $presignedUrl = (string)$request->getUri();
    // }else{
    //     return '';
    // }
    return 'https://cdn.pixabay.com/photo/2016/08/08/09/17/avatar-1577909_1280.png';
}

function siteConfig($name)
{
    //return "enable";
    return Setting::where('name',$name)->value('value');
}

function loginLogs($userFk,$email,$status,$description)
{
    $loginActivity = new LoginActivity();
    $loginActivity->user_id = $userFk;
    $loginActivity->email = $email;
    $loginActivity->login_status = $status;
    $loginActivity->description = $description;
    return $loginActivity->save();
}


function hashPassword($password) 
{
        return sha1($password);
}

function rejectionDateTimeFormat($datetime)
{
    return date('m/d/Y h:i A',strtotime($datetime));
}

function showDateFormat($date) {
    return date('m/d/Y', strtotime($date));
}

function currentDate(){
    return date('m/d/Y');
}

function dateFormat($date) {
    return date('Y-m-d', strtotime($date));
}

function nextAvailableslotDateTimeLocalToUtc($value,$timezone){
    $date = Carbon::createFromFormat("Y-m-d H:i", $value, $timezone);
    $dateTime = $date->setTimezone('UTC');
    return date("Y-m-d H:i", strtotime($dateTime)); // Save time in 24 hours format ("H:i")
}

function nextAvailableslotTimeLocalToUtc($value,$timezone){
    $date = Carbon::createFromFormat("H:i", $value, $timezone);
    $dateTime = $date->setTimezone('UTC');
    return date("H:i", strtotime($dateTime)); // Save time in 24 hours format ("H:i")
}

function convertTimeLocalToUtc($value,$timezone){
    $date = Carbon::createFromFormat("H:i A", $value, $timezone);
    $dateTime = $date->setTimezone('UTC');
    return date("H:i", strtotime($dateTime)); // Save time in 24 hours format ("H:i")
}

// function convertTimeUtcToLocal($value,$timezone){
//     $dateTime = Carbon::parse($value)->timezone($timezone);
//     return date("h:i A", strtotime($dateTime));  // Return time in 12 hours format ("h:i A")
// } 

function convertDateTimeLocalToUtc($value,$timezone){
    $date = Carbon::createFromFormat("m/d/Y H:i A", $value, $timezone);
    $dateTime = $date->setTimezone('UTC');
    return date("Y-m-d H:i", strtotime($dateTime)); // Save time in 24 hours format ("H:i")
}

function convertDateTimeUtcToLocal($value,$timezone){
    $dateTime = Carbon::parse($value)->timezone($timezone);
    return date("m/d/Y h:i A", strtotime($dateTime));   // Return time in 12 hours format ("h:i A")
}

function checkFirstHalfLeave($array1, $array2){
    foreach ($array1 as $key1 => $value1) {
        if (in_array($value1, $array2)) {
            unset($array1[$key1]);
        }
    }
    return array_values($array1);
}

function checkSecondHalfLeave($array1, $array2){
    foreach ($array1 as $key1 => $value1) {
        if (in_array($value1, $array2)) {
            unset($array1[$key1]);
        }
    }
    return array_values($array1);
}
function checkLeaveProvider($array1, $array2){

    foreach ($array2 as $value2) {
        foreach ($array1 as $date => &$entries) {
            foreach ($entries as $key => $value1) {
                if ($value1 == $value2) {
                    unset($array1[$date][$key]);
                }
            }
        }
    }
    
    // Optional: If you want to remove empty subarrays after unsetting values
    foreach ($array1 as $date => &$entries) {
        $entries = array_values($entries);
    }
    
    // Display the updated array
    return array_values($array1);
}

function checkLeaveCustomer($array1, $array2){
    // dd($array1, $array2);
    // foreach ($array1 as $key1 => $value1) {
    //     foreach ($array2 as $key2 => $value2) {
    //         if ($value1['time'] == $value2['time'] && $value1['available'] == $value2['available']) {
    //             unset($array1[$key1]);
    //             break;
    //         }
    //     }
    // }
    
    // return array_values($array1);
    
    
    foreach ($array2 as $value2) {
        foreach ($array1 as $date => &$entries) {
            foreach ($entries as $key => $value1) {
                if ($value1 == $value2) {
                    
                    unset($array1[$date][$key]);
                }
            }
        }
    }
    
    // Optional: If you want to remove empty subarrays after unsetting values
    foreach ($array1 as $date => &$entries) {
        $entries = array_values($entries);
    }
    
    return array_values($array1);
}

function checkLeave($array1, $array2){
    foreach ($array1 as $key1 => $value1) 
    {
        if (in_array($value1, $array2)) {
            unset($array1[$key1]);
        }
    }
    return array_values($array1);
}

function checkLeaveNew($array1, $array2){
    foreach ($array1 as $key1 => $value1) 
    {
        if (in_array($value1, $array2)) {
            unset($array1[$key1]);
        }
    }
    return $array1;
}

function update_remaining_appointment($user_id,$appointment_status){
    $customer=Customer::where(['id'=>$user_id])->first();
    $remaining_appointment=0;
    if($appointment_status=='add'){
        $remaining_appointment=$customer->appointment_session-1;
    }else{
        $remaining_appointment=$customer->appointment_session+1;
    }
    $customer->appointment_session=$remaining_appointment;
    $customer->save();

}

function convertTimeUtcToLocal($value){

    // $ip = file_get_contents("http://ipecho.net/plain");
    // $ipInfo = file_get_contents('http://ip-api.com/json/' . $ip);
    // $ipInfo = json_decode($ipInfo);
    // $timezone = $ipInfo->timezone;
    // date_default_timezone_set($timezone);
    // $timeZoneLocal = date_default_timezone_get();

    // $changetime = new DateTime($value, new DateTimeZone('UTC'));
    // $changetime->setTimezone(new DateTimeZone($timeZoneLocal));
    // return $changetime->format("h:i A"); // Return time in 12 hours format ("h:i A")
    return $value;
}

function convertcurrenttimetoUtc($value){
    // $ip = file_get_contents("http://ipecho.net/plain");
    // $ipInfo = file_get_contents('http://ip-api.com/json/' . $ip);
    // $ipInfo = json_decode($ipInfo);
    // $timezone = $ipInfo->timezone;
    // date_default_timezone_set($timezone);
    // $timeZoneLocal = date_default_timezone_get();
    
    
    // $newDateTime = new DateTime($value, new DateTimeZone($timeZoneLocal)); 
    // $newDateTime->setTimezone(new DateTimeZone("UTC")); 
    // $dateTimeUTC = $newDateTime->format("Y-m-d H:i:s");
    // return $dateTimeUTC;

    return $value;
}

function changeTimeFormat($value){

    $newDateTime = new DateTime($value); 
    $dateTimeUTC = $newDateTime->format("h:i A");
    return $dateTimeUTC;

}

function changeTimeFormat24hrs($value){

    $newDateTime = new DateTime($value); 
    $dateTimeUTC = $newDateTime->format("H:i:s");
    return $dateTimeUTC;

}


function convertTimeUtcToLocalv3($value, $timezone){

    // $ip = file_get_contents("http://ipecho.net/plain");
    // $ipInfo = file_get_contents('http://ip-api.com/json/' . $ip);
    // $ipInfo = json_decode($ipInfo);
    // $timezone = $ipInfo->timezone;
    // date_default_timezone_set($timezone);
    // $timeZoneLocal = date_default_timezone_get();

    $changetime = new DateTime($value, new DateTimeZone('UTC'));
    $changetime->setTimezone(new DateTimeZone($timezone));
    return $changetime->format("h:i A"); 
}

function convertDateTimeUtcToESTv3($value, $timezone){

    $changetime = new DateTime($value, new DateTimeZone('UTC'));
    $changetime->setTimezone(new DateTimeZone($timezone));
    return $changetime->format("Y-m-d H:i"); 
}

function strToTimeConvert($value){

    $changetime = new DateTime($value, new DateTimeZone('UTC'));
    // $changetime->setTimezone(new DateTimeZone($timezone));
    return $changetime->format("Y-m-d H:i"); 
}

function convertTimeUtcToLocalNewv3($value,$timezone){

    $changetime = new DateTime($value, new DateTimeZone('UTC'));
    $changetime->setTimezone(new DateTimeZone($timezone));
    return $changetime->format("Y-m-d h:i A"); // Return time in 12 hours format ("h:i A")
    
}


function moengageCallCustomEventApiRequest($data)
{
    $url = "https://api-".env('MOENGAGE_ID').".moengage.com/v1/event/".env('MOENGAGE_APP_ID')."?app_id=".env('MOENGAGE_APP_ID')."";
    $payload = json_encode($data);
    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $payload);
    curl_setopt($ch, CURLOPT_HTTPHEADER, [
        'Content-Type: application/json',
        'Authorization: Basic ' . base64_encode(env('MOENGAGE_API_ID').':'.env('MOENGAGE_API_KEY'))
    ]);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    
    $response = curl_exec($ch);
    
    curl_close($ch);
    return $response;

}

function moengageCallUserAttributesCustomerApiRequest($data)
{
    $url = "https://api-".env('MOENGAGE_ID').".moengage.com/v1/customer/".env('MOENGAGE_APP_ID')."?app_id=".env('MOENGAGE_APP_ID')."";
    $payload = json_encode($data);
    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $payload);
    curl_setopt($ch, CURLOPT_HTTPHEADER, [
        'Content-Type: application/json',
        'Authorization: Basic ' . base64_encode(env('MOENGAGE_API_ID').':'.env('MOENGAGE_API_KEY'))
    ]);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    
    $response = curl_exec($ch);
    
    curl_close($ch);
    return $response;

}


function moengageCallUserAttributesProviderApiRequest($data)
{
    $url = "https://api-".env('MOENGAGE_ID').".moengage.com/v1/customer/".env('MOENGAGE_PROVIDER_APP_ID')."?app_id=".env('MOENGAGE_PROVIDER_APP_ID')."";
    $payload = json_encode($data);
    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $payload);
    curl_setopt($ch, CURLOPT_HTTPHEADER, [
        'Content-Type: application/json',
        'Authorization: Basic ' . base64_encode(env('MOENGAGE_PROVIDER_API_ID').':'.env('MOENGAGE_PROVIDER_API_KEY'))
    ]);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    
    $response = curl_exec($ch);
    
    curl_close($ch);
    return $response;

}


function moengageCallCustomEventProviderApiRequest($data)
{
    $url = "https://api-".env('MOENGAGE_PROVIDER_ID').".moengage.com/v1/event/".env('MOENGAGE_PROVIDER_APP_ID')."?app_id=".env('MOENGAGE_PROVIDER_APP_ID')."";
    $payload = json_encode($data);
    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $payload);
    curl_setopt($ch, CURLOPT_HTTPHEADER, [
        'Content-Type: application/json',
        'Authorization: Basic ' . base64_encode(env('MOENGAGE_PROVIDER_API_ID').':'.env('MOENGAGE_PROVIDER_API_KEY'))
    ]);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    
    $response = curl_exec($ch);
    
    curl_close($ch);
    return $response;

}
