<?php

namespace App\Http\Controllers\Contractors;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

use App\Models\Contractors\Address;

class ContractorAddressController extends Controller
{
    /**
     * The user repository implementation.
     *
     * @var UserRepository
     */
    protected $user_address;
    protected  $user_id;

    public function __construct(Address $user_address, Request $request) {
        $this->user_address = $user_address;
        $this->user_id = $request->user_id;
    }

    /**
     * @ user with its all address list api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function index(Request $request, $contractor_id) {
        try
        {
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;
            if($limit > 500) {
                $limit = 25;
            }
            if(isset($request->order) && strtolower($request->order)=="asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if(isset($request->order_by_column_name)) {
                if(in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }

            $id = $contractor_id;
            $users = $this->user_address::where([
                        ['user_id', '=', $id],
                        ['user_type', '=', 'contractor'],
                    ])
                    ->where(function($q) use($request) {
                        // Global searching...
                        if(isset($request->search) && !empty($request->search) && $request->search!="") {
                            if(!empty($this->getColumnName())) {
                                foreach ($this->getColumnName() as $key => $col) {
                                    if($key==0) {
                                        $q->where($col, 'like', '%' .$request->search. '%');
                                    } else {
                                        $q->orWhere($col, 'like', '%' .$request->search. '%');
                                    }
                                }
                            }
                        }
                    })
                    ->where(function($q) use($request) {
                        // Column wise searching filter ...
                        $allowableColumn = $this->getColumnName();
                        if(isset($request->column_name) && !empty($request->column_name) && $request->column_name!="" && in_array($request->column_name, $allowableColumn)) {
                            $q->where($request->column_name,  'like', '%' .$request->column_value. '%');
                        }
                    })
                    ->orderBy($order_by_column_name, $order)
                    ->paginate($limit);

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.ADDRESS_LIST'), 'records' => $users]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    /**
     * Add user address api.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request, $contractor_id)
    {
        $this->validate($request, [
            'street_address' => 'required|max:255',
            'street_name' => 'required|max:255',
            'apartment_number' => 'required|numeric|min:1',
            'city' => 'required|max:255',
            'state' => 'required|max:255',
            'pincode' => 'required|digits_between:6,6',
            'landmark' => 'required|max:255',
            'location' => 'required|max:255',
        ]);

        try {

            $input = $request->all();
            $input['user_type'] = 'contractor';
            $input['user_id'] = $contractor_id;
            $address = $this->user_address::create($input);

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.ADDRESS_STORE'), 'records' => $address]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * @ Service List api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function show($contractor_id, $address_id) {
        try {
            $address = $address = $this->user_address::where([
                ['id', '=', $address_id],
                ['user_type', '=', 'contractor'],
                ['user_id', '=', $contractor_id],
            ])->first();
            if( empty($address) ) {
                throw new Exception(config('service.CONTRACTOR_API.MESSAGE.ADDRESS_NOT_FOUND'));
            }

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.ADDRESS_DETAILS'), 'records' => $address]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * @ update user api .
     * @return \Illuminate\Http\Response
    */
    public function update(Request $request, $contractor_id, $address_id) {
        $this->validate($request, [
            'street_address' => 'required|max:255',
            'street_name' => 'required|max:255',
            'apartment_number' => 'required|numeric|min:1',
            'city' => 'required|max:255',
            'state' => 'required|max:255',
            'pincode' => 'required|digits_between:6,6',
            'landmark' => 'required|max:255',
            'location' => 'required|max:255',
        ]);

        try
        {
            $address = $this->user_address::where([
                ['id', '=', $address_id],
                ['user_type', '=', 'contractor'],
                ['user_id', '=', $contractor_id],
            ]);
            if(!$address->update($request->all())) {
                throw new Exception(config('service.CONTRACTOR_API.MESSAGE.SOMETHING_WENT_WRONG'), 1);
            }

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.ADDRESS_UPDATE'), 'records' => $address->first()]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * @ set setDefaultAddress user api .
     * @return \Illuminate\Http\Response
    */
    public function setDefaultAddress(Request $request, $contractor_id, $address_id) {
        $this->validate($request, [
            'default_address' => 'required|in:YES,NO'
        ]);
        try
        {
            $input['default_address'] = $request->default_address;
            $address = $this->user_address::where([
                ['id', '=', $address_id],
                ['user_type', '=', 'contractor'],
                ['user_id', '=', $contractor_id],
            ]);
            if($address->exists()) {
                $upd['default_address'] = 'NO';
                $this->user_address::where([
                    ['user_id', '=', $contractor_id],
                    ['user_type', '=', 'contractor'],
                ])->update($upd);
                $address->update($input);
                return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.ADDRESS_UPDATE'), 'records' => $address->first()]);
            }
            throw new Exception(config('service.USER_API.MESSAGE.CUSTOMER_OR_ADDRESS_NOT_EXISTS'), 1);

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * Remove the specified addres from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function delete($contractor_id, $address_id)
    {
        try {
            $address = $this->user_address::where([
                ['id', '=', $address_id],
                ['user_type', '=', 'contractor'],
                ['user_id', '=', $contractor_id],
            ])->delete();
            if(!$address) {
                throw new Exception(config('service.USER_API.MESSAGE.SOMETHING_WENT_WRONG'), 1);
            }

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.ADDRESS_DELETED'), 'records' => $address]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    private function getColumnName() {
        $address = $this->user_address::first();
        if(!empty($address) && $address!="") {
            return array_keys($address->toArray());
        }

        return ['id'];
    }
}
