<?php

namespace App\Http\Controllers\Contractors;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

use App\Models\Contractors\ContractorAppSetting;

class ContractorAppSettingController extends Controller
{
    /**
     * The user repository implementation.
     *
     * @var UserRepository
     */
    protected  $con_app_settings;
    protected  $user_id;
    protected  $role_fk;

    public function __construct(ContractorAppSetting $con_app_settings, Request $request) {
        $this->con_app_settings = $con_app_settings;
        $this->user_id = $request->user_id;
    }

    public function index(Request $request) {
        try
        {
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;
            if($limit > 500) {
                $limit = 25;
            }
            if(isset($request->order) && strtolower($request->order)=="asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if(isset($request->order_by_column_name)) {
                if(in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }

            $id = $this->user_id;
            $users = $this->con_app_settings::where('contractor_id', '=', $id)
                    ->where(function($q) use($request) {
                        // Global searching...
                        if(isset($request->search) && !empty($request->search) && $request->search!="") {
                            if(!empty($this->getColumnName())) {
                                foreach ($this->getColumnName() as $key => $col) {
                                    if($key==0) {
                                        $q->where($col, 'like', '%' .$request->search. '%');
                                    } else {
                                        $q->orWhere($col, 'like', '%' .$request->search. '%');
                                    }
                                }
                            }
                        }
                    })
                    ->where(function($q) use($request) {
                        // Column wise searching filter ...
                        $allowableColumn = $this->getColumnName();
                        if(isset($request->column_name) && !empty($request->column_name) && $request->column_name!="" && in_array($request->column_name, $allowableColumn)) {
                            $q->where($request->column_name,  'like', '%' .$request->column_value. '%');
                        }
                    })
                    ->orderBy($order_by_column_name, $order)
                    ->paginate($limit);

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.CONTRACTOR_APP_SETTING'), 'records' => $users]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function update(Request $request) {
        $this->validate($request, [
            'new_inquiry' => 'required|in:YES,NO',
            'new_alert' => 'required|in:YES,NO',
            'job_completed' => 'required|in:YES,NO',
            'job_started' => 'required|in:YES,NO',
        ]);

        try
        {
            $contractor_id = $this->user_id;
            $setting = $this->con_app_settings::where('contractor_id', '=', $contractor_id);
            $input = $request->all();
            if($setting->exists()) {
                if(!$setting->update($input)) {
                    throw new Exception(config('service.CONTRACTOR_API.MESSAGE.SOMETHING_WENT_WRONG'), 1);
                }
            } else {
                $input['contractor_id'] = $contractor_id;
                if(!$setting->create($input)) {
                    throw new Exception(config('service.CONTRACTOR_API.MESSAGE.SOMETHING_WENT_WRONG'), 1);
                }
            }

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.CONTRACTOR_SETTING_UPDATED'), 'records' => $setting->first()]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    private function getColumnName() {
        $address = $this->con_app_settings::first();
        if(!empty($address) && $address!="") {
            return array_keys($address->toArray());
        }
        return ['id'];
    }
}
