<?php

namespace App\Http\Controllers\Contractors;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\Contractors\Address;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;

use App\Models\Contractors\Contractor;
use App\Models\Contractors\ContractorCategory;
use App\Models\Contractors\ContractorService;
use App\Models\Contractors\Jobs;
use App\Models\Contractors\Oauth;
use App\Models\Contractors\ServiceCategory;
use App\Notifications\CreateNewContractorNotification;

class ContractorController extends Controller
{
    /**
     * The user repository implementation.
     *
     * @var UserRepository
     */
    protected $contractor;
    protected $contractor_category;
    protected $contractor_service;
    protected $oauth_user;
    protected $service;
    protected $service_cat;
    protected $jobs;
    protected $user_id;
    protected $role_fk;
    protected $image_url;
    protected $user_image_path;
    protected $contractor_address;

    public function __construct(Contractor $contractor, ContractorCategory $contractor_category, Oauth $oauth_user, Jobs $jobs, ContractorService $contractor_service, ServiceCategory $service_cat, Request $request, Address $contractor_address) {
        $this->contractor = $contractor;
        $this->oauth_user = $oauth_user;
        $this->contractor_service = $contractor_service;
        $this->contractor_category = $contractor_category;
        $this->service_cat = $service_cat;
        $this->jobs = $jobs;
        $this->user_id = $request->user_id;
        $this->role_fk = $request->role_fk;
        $this->contractor_address = $contractor_address;

        $this->image_url = str_replace('SIS941-API', 'SIS941', url('uploads/image/contractors'));
        $this->user_image_path = str_replace('SIS941-API', 'SIS941', url('uploads/image/users'));
    }

    /**
     * @ Contractor list List api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function index(Request $request) {
        try
        {
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;
            if($limit > 500) {
                $limit = 25;
            }
            if(isset($request->order) && strtolower($request->order)=="asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if(isset($request->order_by_column_name)) {
                if(in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }

            $contractor = $this->contractor::where('status', '=', 1)
                        ->where(function($q) use($request) {
                            // Global searching...
                            if(isset($request->search) && !empty($request->search) && $request->search!="") {
                                if(!empty($this->getColumnName())) {
                                    foreach ($this->getColumnName() as $key => $col) {
                                        if($key==0) {
                                            $q->where($col, 'like', '%' .$request->search. '%');
                                        } else {
                                            $q->orWhere($col, 'like', '%' .$request->search. '%');
                                        }
                                    }
                                }
                            }
                        })
                        ->where(function($q) use($request) {
                            // Column wise searching filter ...
                            $allowableColumn = $this->getColumnName();
                            if(isset($request->column_name) && !empty($request->column_name) && $request->column_name!="" && in_array($request->column_name, $allowableColumn)) {
                                $q->where($request->column_name,  'like', '%' .$request->column_value. '%');
                            }
                        })
                        ->orderBy($order_by_column_name, $order)
                        ->paginate($limit);
            if( empty($contractor) ) {
                throw new Exception(config('service.CONTRACTOR_API.MESSAGE.USER_NOT_FOUND'));
            }

            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.USER_DETAILS'), 'records' => $contractor]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

     /**
     * @ Contractor list List api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function findContractor(Request $request) {
        try
        {
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;
            if($limit > 500) {
                $limit = 25;
            }
            if(isset($request->order) && strtolower($request->order)=="asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if(isset($request->order_by_column_name)) {
                if(in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }

            $service_id = isset($request->service_id) ? explode(',', $request->service_id) : [];
            $service_cat_ids = isset($request->service_categories) ? explode(',', $request->service_categories) : [];

            // print_r($service_id); die;

            $contractor = $this->contractor::select(['id', 'first_name', 'last_name', 'email', 'phone', 'expertise', 'exprence', 'about_company', 'image', 'rate', 'rate_type'])
            ->with('contractor_service')->whereHas('contractor_service', function($query) use($service_id, $service_cat_ids) {
                // if(isset($service_id) && count($service_id) > 0){
                //     $query->with('service')->whereIn('service_id', $service_id);
                // }
                // $query->with(['contractor_service_category' => function($q) use($service_cat_ids) {
                //     if(isset($service_cat_ids) && count($service_cat_ids)>0){
                //         $q->with('service_category')->whereIn('service_cat_id', $service_cat_ids);
                //     }
                // }]);
            })
            ->with(['rating' => function($query) {
                $query->select('contractor_id', DB::raw('AVG(rating) as avarage_rating'))->groupBy('contractor_id');
            }])
            ->where('status', '=', 1)
            ->where(function($q) use($request) {
                // Global searching...
                if(isset($request->search) && !empty($request->search) && $request->search!="") {
                    if(!empty($this->getColumnName())) {
                        foreach ($this->getColumnName() as $key => $col) {
                            if($key==0) {
                                $q->where($col, 'like', '%' .$request->search. '%');
                            } else {
                                $q->orWhere($col, 'like', '%' .$request->search. '%');
                            }
                        }
                    }
                }
            })
            ->where(function($q) use($request) {
                // Column wise searching filter ...
                $allowableColumn = $this->getColumnName();
                if(isset($request->column_name) && !empty($request->column_name) && $request->column_name!="" && in_array($request->column_name, $allowableColumn)) {
                    $q->where($request->column_name,  'like', '%' .$request->column_value. '%');
                }
            })
            ->orderBy($order_by_column_name, $order)
            ->paginate($limit);
            if( empty($contractor) ) {
                throw new Exception(config('service.CONTRACTOR_API.MESSAGE.USER_NOT_FOUND'));
            }

            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.USER_DETAILS'), 'records' => $contractor, 'contractor_image_path' =>$this->image_url]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function showMyInfo() {
        try
        {
            $contractor = [];
            if(strtolower($this->role_fk) == 'contractor') {
                    $contractor = $this->contractor::with(['address' => function($q) {
                        $q->where('user_type', '=', $this->role_fk);
                    }, 'jobs'=> function($q) {
                        $q->select('contractor_id',
                            DB::raw('(CASE WHEN status = 1 THEN COUNT(status) ELSE 0 END) AS confirmed'),
                            DB::raw('(CASE WHEN status = 2 THEN COUNT(status) ELSE 0 END) AS on_the_way'),
                            DB::raw('(CASE WHEN status = 3 THEN COUNT(status) ELSE 0 END) AS in_process'),
                            DB::raw('(CASE WHEN status = 4 THEN COUNT(status) ELSE 0 END) AS completed'),
                            DB::raw('(CASE WHEN status = 5 THEN COUNT(status) ELSE 0 END) AS cancelled'),
                        )->groupBy(['contractor_id', 'status']);
                    }])
                    ->find($this->user_id);
                    $store = [];
                    if(isset($contractor->jobs) && count($contractor->jobs) > 0 ) {
                        foreach ($contractor->jobs as $key => $val) {
                            if($val->confirmed > 0) {
                                $store[0]['confirmed_job'] = $val->confirmed;
                            }
                            if($val->completed > 0) {
                                $store[0]['completed_job'] = $val->completed;
                            }
                            if($val->cancelled > 0) {
                                $store[0]['cancelled_job'] = $val->cancelled;
                            }
                    }
                }
            }

            if( empty($contractor) ) {
                throw new Exception(config('service.CONTRACTOR_API.MESSAGE.USER_NOT_FOUND'));
            }

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.USER_DETAILS'), 'records' => $contractor]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * @ Contractor List api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function show($id) {
        try {
            $contractor = $this->contractor::select(['id', 'first_name', 'last_name', 'email', 'phone', 'expertise', 'exprence', 'rate', 'rate_type', 'licence_no', 'image', 'status', 'about_company'])
            ->with(['address' => function($q) {
                $q->where('user_type', '=', 'contractor');
            }, 'contractor_service.service', 'contractor_service.contractor_service_category.service_category', 'avg_rating' => function($query) {
                $query->select('contractor_id', DB::raw('AVG(rating) as avarage_rating'))->where('user_type', '=', 'customer')->groupBy('contractor_id');
            }, 'rating' => function($query) {
                $query->with(['user' => function($q) {
                    $q->select(['id', 'first_name', 'last_name', 'image']);
                }])->where('user_type', '=', 'customer');
            }, 'jobs'=> function($q){
                $q->select('contractor_id',
                    DB::raw('(CASE WHEN status = 1 THEN COUNT(status) ELSE 0 END) AS confirmed'),
                    DB::raw('(CASE WHEN status = 2 THEN COUNT(status) ELSE 0 END) AS on_the_way'),
                    DB::raw('(CASE WHEN status = 3 THEN COUNT(status) ELSE 0 END) AS in_process'),
                    DB::raw('(CASE WHEN status = 4 THEN COUNT(status) ELSE 0 END) AS completed'),
                    DB::raw('(CASE WHEN status = 5 THEN COUNT(status) ELSE 0 END) AS cancelled'),
                )->groupBy(['contractor_id', 'status']);
            }])->find($id);
            if( empty($contractor) ) {
                throw new Exception(config('service.CONTRACTOR_API.MESSAGE.USER_NOT_FOUND'));
            }

            $store = [];
            if(isset($contractor->jobs) && count($contractor->jobs) > 0 ) {
                foreach ($contractor->jobs as $key => $val) {
                    if($val->confirmed > 0) {
                        $store[0]['confirmed_job'] = $val->confirmed;
                    }
                    if($val->on_the_way > 0) {
                        $store[0]['on_the_way_job'] = $val->on_the_way;
                    }
                    if($val->in_process > 0) {
                        $store[0]['in_process_job'] = $val->in_process;
                    }
                    if($val->completed > 0) {
                        $store[0]['completed_job'] = $val->completed;
                    }
                    if($val->cancelled > 0) {
                        $store[0]['cancelled_job'] = $val->cancelled;
                    }
                }
            }

            $contractor->setRelation('jobs',  collect($store));
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.USER_DETAILS'), 'records' => $contractor, 'contractor_image_path' =>$this->image_url, 'user_image_path' =>$this->user_image_path]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * @ Register contractor api.
     * @return \Illuminate\Http\Response
    */
    public function store(Request $request) {
        $message = array(
            'password.regex' => config('service.USER_API.MESSAGE.PASSWORD_REGEX'),
            'rate.required' => config('service.USER_API.MESSAGE.CONTRACTOR_RATE')
        );
        $this->validate($request, [
            'first_name' => 'required',
            'last_name' => 'required',
            'email' => 'required|email|max:255|unique:contractors',
            'contractor_service' => 'required',
            'contractor_category' => 'required',
            'phone' => 'required|regex:/^([0-9\s\-\+\(\)]*)$/|min:10|max:10|unique:contractors',
            'expertise' => 'required',
            'exprence' => 'required',
            'licence_no' => 'required',
            'rate' => 'required',
            'latitude' => 'required|max:256',
            'longitude' => 'required|max:256',
            'about_company' => 'required',
            'password' => [
                'required',
                'min:8',             // must be at least 8 characters in length
                'regex:/[a-z]/',     // must contain at least one lowercase letter
                'regex:/[A-Z]/',     // must contain at least one uppercase letter
                'regex:/[0-9]/',     // must contain at least one digit
                'regex:/[@$!%*#?&]/',// must contain a special character
             ],
            'confirm_password' => 'required|same:password',
        ], $message);

        try {
            DB::beginTransaction();
            $input = $request->all();
            $input['password'] = $this->hashPassword($request->password);
            if($contractor = $this->contractor->create($input)) {
                $contractor_service = explode(',', $request->contractor_service);
                if(!empty($contractor_service) && count($contractor_service)>0) {
                    foreach ($contractor_service as $ser) {
                        // store contractor services
                        $contractor_service = new $this->contractor_service;
                        $contractor_service->contractor_id = $contractor->id;
                        $contractor_service->service_id = $ser;
                        $contractor_service->status = 'active';
                        $contractor_service->save();

                        // store contractor service category
                        $contractor_category = explode(',', $request->contractor_category);
                        if(!empty($contractor_category) && count($contractor_category)>0) {
                            foreach ($contractor_category as $cat) {
                                // check service id of category
                                $check = $this->service_cat::find($cat);
                                if(!empty($check) && $check!="") {
                                    if($check->service_id == $ser) {
                                        $contractor_category = new $this->contractor_category;
                                        $contractor_category->contractor_id = $contractor->id;
                                        $contractor_category->contractor_service_id  = $contractor_service->id;
                                        $contractor_category->service_cat_id  = $cat;
                                        $contractor_category->save();
                                    }
                                }
                            }
                        }
                    }
                }

                $oauth = new $this->oauth_user;
                $oauth->user_id = $contractor->id;
                $oauth->role_fk = 'contractor';
                $oauth->username = $request->email;
                $oauth->password = $input['password'];
                $oauth->first_name = $request->first_name;
                $oauth->last_name = $request->last_name;
                $oauth->email = $request->email;
                $oauth->phone = $request->phone;
                $oauth->save();
                DB::commit();

                $details = [
                    'greeting' => 'Hi '. $request->first_name .'!',
                    'body'=> 'Your email : '. $request->email .' and your login password is : '.$request->password,
                    'database_notification_message' => 'A new contractor has been created successfully.',
                    'thanks' => 'Welcome to ZION!!',
                ];
                $role = 'contractor';
                $contractor->notify(new CreateNewContractorNotification($role, $details));
                $upd['user_type'] = $role;
                $notif_id = $contractor->notifications[0]->id;
                $contractor->notifications()->find($notif_id)->update($upd);

                return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.USER_REGISTER'), 'records' => $contractor]);
            }
            throw new Exception(config('service.CONTRACTOR_API.MESSAGE.SOMETHING_WENT_WRONG'));
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            // This should work as well
            DB::rollBack();
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * @ update Contractor api .
     * @return \Illuminate\Http\Response
    */
    public function update(Request $request, $id) {
        $message = array(
            'password.regex' => config('service.USER_API.MESSAGE.PASSWORD_REGEX')
        );
        $this->validate($request, [
            'first_name' => 'required',
            'last_name' => 'required',
            'email' => 'required|email|unique:contractors,email,'.$id,
            'phone' => 'required|regex:/^([0-9\s\-\+\(\)]*)$/|min:10|max:10|unique:contractors,phone,'.$id,
            'latitude' => 'required|max:256',
            'longitude' => 'required|max:256',
            'about_company' => 'required',
            'password' => [
                'min:8',             // must be at least 8 characters in length
                'regex:/[a-z]/',     // must contain at least one lowercase letter
                'regex:/[A-Z]/',     // must contain at least one uppercase letter
                'regex:/[0-9]/',     // must contain at least one digit
                'regex:/[@$!%*#?&]/',// must contain a special character
             ],
            'image' => $request->has('image') ? 'required|mimes:jpeg,png,jpg,svg,gif' : "",
            'address' => 'required|max:255',
            'city' => 'required|max:255',
            'state' => 'required|max:255',
            'pincode' => 'required|digits_between:6,6',
            'country' => 'required|max:255',
            'description' => 'max:255',
        ], $message);

        try {
            DB::beginTransaction();
            $contractor = $this->contractor::find($id);
            // ecsho "<pre>"; print_r($contractor); die;
            $input = $request->all();
            if(!empty($input['password'])) {
                $input['password'] = $this->hashPassword($input['password']);
            } else {
                $input = Arr::except($input, ['password']);
            }

            if($request->has('image')) {
                $image = $request->file('image');
                $imagename = time().'.'.$image->getClientOriginalName();
                $image->move(public_path('assets/images/contractors'), $imagename);
                $input['image'] = @$imagename;
            }

            if( $contractor->update($input) )
            {
                $check_exists = $this->oauth_user::where([
                    ['role_fk', '=', 'contractor'],
                    ['user_id', '=', $id],
                ])->first();
                if( !empty($check_exists) && $check_exists ) {
                    $oauth = $this->oauth_user::find($check_exists->id);
                    $oauth->username = $request->email;
                    if(!empty($input['password'])) {
                        $oauth->password = $input['password'];
                    }
                    $oauth->first_name = $request->first_name;
                    $oauth->last_name = $request->last_name;
                    $oauth->email = $request->email;
                    $oauth->phone = $request->phone;
                    $oauth->update();
                }

                // address update
                $contractor_address = $this->contractor_address::where([
                    ['user_id', '=', $id],
                    ['user_type', '=', 'contractor'],
                ]);
                if($contractor_address->exists()) {
                    $inp_addr['street_address'] = $request->address;
                    $inp_addr['state'] = $request->state;
                    $inp_addr['city'] = $request->city;
                    $inp_addr['landmark'] = $request->country;
                    $inp_addr['country'] = $request->country;
                    $inp_addr['pincode'] = $request->pincode;
                    die('okk');
                    $contractor_address->update($inp_addr);
                } else {
                    $inp_addr['user_id'] = $id;
                    $inp_addr['user_type'] = 'contractor';
                    $inp_addr['default_address'] = 'YES';
                    $inp_addr['street_address'] = $request->address;
                    $inp_addr['state'] = $request->state;
                    $inp_addr['city'] = $request->city;
                    $inp_addr['landmark'] = $request->country;
                    $inp_addr['country'] = $request->country;
                    $inp_addr['pincode'] = $request->pincode;
                    $this->contractor_address->create($inp_addr);
                }

                $users['image_path'] = url('/assets/images/contractors');
                DB::commit();

                return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.USER_UPDATED'), 'records' => $contractor]);
            }
            throw new Exception(config('service.CONTRACTOR_API.MESSAGE.SOMETHING_WENT_WRONG'));
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            // This should work as well
            DB::rollBack();
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function contractorCount()
    {
        try {
            $contractor = $this->contractor::count();

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.USER_TOTAL'), 'records' => $contractor]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function isEmailAvailable(Request $request)
    {
        $this->validate($request, [
            'email' => 'required|email'
        ]);

        try {
            $email = $request->email;
            $contractor = $this->contractor::where('email', '=', $email)->exists();
            if($contractor) {
                return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.EMAIL_ALREADY_EXISTS'), 'records' => $contractor]);
            }

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.EMAIL_NOT_EXISTS'), 'records' => $contractor]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    /**
     * @ contact us list api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function allContractorJobs(Request $request, $id) {
        try
        {
            $order = "desc";
            $limit = isset($request->limit) && $request->limit > 500 ? 25 : $request->limit;
            if(isset($request->order) && strtolower($request->order)=="asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if(isset($request->order_by_column_name)) {
                if(in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }

            $jobs = $this->jobs::select(['booking.id', 'booking.user_id', 'booking.contractor_id', 'service-category.sub_category_name', 'booking.booking_date', 'booking.booking_time', 'booking.status'])
                    ->join('booking', 'booking.id', '=', 'job_tracking.booking_id')
                    ->join('services', 'services.id', '=', 'booking.service_id')
                    ->join('service-category', 'service-category.id', '=', 'booking.service_cat_id')
                    ->where(function($q) use($id) {
                        $q->where('booking.contractor_id', '=', $id);
                    })
                    ->where(function($q) use($request) {
                        // Global searching...
                        if(isset($request->search) && !empty($request->search) && $request->search!="") {
                            if(!empty($this->getColumnName())) {
                                foreach ($this->getColumnName() as $key => $col) {
                                    if($key==0) {
                                        $q->where($col, 'like', '%' .$request->search. '%');
                                    } else {
                                        $q->orWhere($col, 'like', '%' .$request->search. '%');
                                    }
                                }
                            }
                        }
                    })
                    ->where(function($q) use($request) {
                        // Column wise searching filter ...
                        $request->column_name = 'booking.'.$request->column_name;
                        $allowableColumn = $this->getColumnName();
                        if(isset($request->column_name) && !empty($request->column_name) && $request->column_name!="" && in_array($request->column_name, $allowableColumn)) {
                            $q->where($request->column_name,  'like', '%' .$request->column_value. '%');
                        }
                    })
                    ->orderBy($order_by_column_name, $order)
                    ->paginate($limit);

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.JOBS_LIST'), 'records' => $jobs]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function delete(Request $request, $id)
    {
        try {
            $contractor = $this->contractor::find($id)->delete();

            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.USER_DELETED'), 'records' => $contractor]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    private function getColumnName() {
        $contractor = $this->contractor::first();
        if(!empty($contractor) && $contractor!="") {
            return array_keys($contractor->toArray());
        }

        return ['id'];
    }

    private function hashPassword($password) {
        return sha1($password);
    }
}
