<?php
namespace App\Http\Controllers\MembershipPlan;
use Exception;
use Throwable;
use Carbon\Carbon;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;
use App\Models\MembershipPlan\Plan;
use App\Models\MembershipPlan\Payment;
use App\Models\Users\User;
use Illuminate\Support\Facades\App;
use App\Jobs\CustomerCommonMailJob;

class MembershipPlanController extends Controller {
    /**
    * The user repository implementation.
    *
    * @var UserRepository
    */
    protected $plan;
    protected $user_id;
    protected $role_fk;
    public function __construct(Plan $plan, Request $request)
    {
        $this->plan = $plan;
        $this->user_id = $request->user_id;
        $this->role_fk = $request->role_fk;
        $this->user_fk = $request->user_fk;
        //has_scope('customers.read');
    }

    /**
    * @ user list List api using pagination.
    * @return \Illuminate\Http\Response
    */
    public function index(Request $request)
    {
        try {
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;
            if (isset($request->order) && strtolower($request->order) == "asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            /*if ($request->filled('order_by_column_name')) {
                if (in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }*/
            $plans = $this->plan::where(function ($q) use ($request) {
                // Global searching...
                if (isset($request->search) && !empty($request->search) && $request->search != "") {
                    //if (!empty($this->getColumnName())) {
                        /*foreach ($this->getColumnName() as $key => $col) {
                            if ($key == 0) {*/
                                $q->where('price', 'like', '%' . $request->search . '%');
                                $q->orWhere("no_of_session","%".$request->search."%");
                                $q->orWhere("expiry_days","%".$request->search."%");
                                $q->orWhereDate("created_at","%".dateFormat($request->search)."%");
                                $q->orWhereTranslationLike("title","%".$request->search."%");
                            /*} else {
                                $q->orWhere($col, 'like', '%' . $request->search . '%');
                            }*/
                        //}
                    //}
                }
                // filters
                if($request->filled('plan_name')) {
                    $q->whereTranslationLike("title","%".$request->plan_name."%");
                }
                if($request->filled('plan_price')) {
                    $q->where('price', '=', $request->plan_price);
                }
                if($request->filled('date_from') && $request->filled('date_to')) {
                    $date_from = date("Y-m-d", strtotime($request->date_from));
                    $date_to = date("Y-m-d", strtotime($request->date_to));
                    $q->whereBetween('created_at', [$date_from." "."00:00:00", $date_to." "."23:59:59"]);
                }
            })
            ->orderBy($order_by_column_name, $order)
            ->paginate($limit);

            if (empty($plans)) {
                throw new Exception(config('service.USER_API.MESSAGE.PLAN_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.USER_API.MESSAGE.PLAN_LISTING'), 'records' => $plans]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function activeInactive(Request $request, $id) 
    {
        $this->validate($request,[
            'status' => 'required',
        ]);
        try {
            $save_data['status'] = $request->status;
            $result = Plan::where('id', $id)->update($save_data);

            if($request->status=="active")
            {
                return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.PLAN_STATUS_ACTIVE'),'action'=>$request->status]);
            }
            else
            {
                return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.PLAN_STATUS_INACTIVE'),'action'=>$request->status]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
    }
    public function deletePlan(Request $request, $id) 
    {
        $plan = Plan::findOrFail($id);
        try {
            $plan  = $plan->delete();
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.PLAN_DELETED'), 'result' => $plan]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }
    public function store(Request $request) 
    {
       
        $this->validate($request, [
            'en_plan_name' => 'required|max:191',
            'en_description'  => 'required|max:2000',
            'es_plan_name'  => 'required|max:191',
            'es_plan_description'  => 'required|max:2000',
            'price'  => 'required',
            'expiry_days'  => 'required',
            'no_of_session'  => 'required',
        ]);
        try {
            $input = $request->all();
          
            $data = [
                'price' =>$input['price'],
                'expiry_days' =>$input['expiry_days'],
                'no_of_session' =>$input['no_of_session'],
      
                'en' => [
                    'title'  => $input['en_plan_name'],
                    'description'  => $input['en_description'],
                ],
                'es' => [
                    'title'  => $input['es_plan_name'],
                    'description'  => $input['es_plan_description']
                ],
            ];
            $result  = Plan::create($data);
            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.PLAN_ADDED'), 'records' => $result]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function show(Request $request,$id) 
    {
        try
        {

            $plan =  Plan::find($id);
            if(empty($plan)) {
                throw new Exception(config('service.USER_API.MESSAGE.PLAN_NOT_FOUND'));
            }
            $planEn = [
                'id' => $plan->id,
                'title' => $plan->title,
                'description' => $plan->description
            ];
            app()->setLocale('es');
            $planEs =  Plan::find($id);
            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.PLAN_DETAILS'), 'recordEn' => $planEn,'recordEs' => $planEs]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function update(Request $request, $id) 
    {
        $this->validate($request, [
            'en_plan_name' => 'required|max:191',
            'en_description'  => 'required|max:2000',
            'es_plan_name'  => 'required|max:191',
            'es_plan_description'  => 'required|max:2000',
            'price'  => 'required',
            'expiry_days'  => 'required',
            'no_of_session'  => 'required',
        ]);
        try {
            $plan = Plan::findOrFail($id);

            $input = $request->all();
            $data = [
                'price' =>$input['price'],
                'expiry_days' =>$input['expiry_days'],
                'no_of_session' =>$input['no_of_session'],
                'updated_at'=>convertcurrenttimetoUtc(date('Y-m-d H:i:s')),
                'en' => [
                    'title'  => $input['en_plan_name'],
                    'description'  => $input['en_description'],
                ],
                'es' => [
                    'title'  => $input['es_plan_name'],
                    'description'  => $input['es_plan_description']
                ],
            ];
            $data  = $plan->update($data);
            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.PLAN_UPDATED'), 'records' => $data]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function purchaseHistory(Request $request)
    {
        try {
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;
            if (isset($request->order) && strtolower($request->order) == "asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if ($request->filled('order_by_column_name')) {
                if (in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }

            $pieces = explode(" ", $request->consumer_name);
            if(count($pieces)){
                $fName = !empty($pieces[0]) ? $pieces[0] : $request->consumer_name; 
                $lName = !empty($pieces[1]) ? $pieces[1] : $request->consumer_name; 
            }

            $query = Payment::with(['customer','plan'])
            /*->where(function ($q) use ($request) {
                // Global searching...
                if (isset($request->search) && !empty($request->search) && $request->search != "") {
                    if (!empty($this->getColumnName())) {
                        foreach ($this->getColumnName() as $key => $col) {
                            if ($key == 0) {
                                $q->where($col, 'like', '%' . $request->search . '%');
                            } else {
                                $q->orWhere($col, 'like', '%' . $request->search . '%');
                            }
                        }
                    }
                }
            })*/
            ->whereHas("customer", function($q) use ($request) {
                if($request->filled('search')) {
                    $pieces = explode(" ", $request->search);
                    if(count($pieces)){
                        $fName = !empty($pieces[0]) ? $pieces[0] : $request->search; 
                        $lName = !empty($pieces[1]) ? $pieces[1] : $request->search; 
                    }
                    $q->where("first_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                    $q->orWhere("last_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                    $q->orWhere(DB::raw('CONCAT(first_name, " ",last_name)'), 'like', '%'.encryptString(ucfirst(strtolower($fName)))." ".encryptString(ucfirst(strtolower($lName))).'%');
                }
            })
            /*->orWhereHas("plan", function($q) use ($request) {
                if($request->filled('search')) {
                    $q->whereTranslationLike("title","%".$request->search."%");
                }
            })*/
            ->whereHas("plan", function($q) use ($request) {
                if($request->filled('plan_name')) {
                    $q->whereTranslationLike("title","%".$request->plan_name."%");
                }
            })
            ->whereHas("customer", function($q) use ($request) {
                if($request->filled('consumer_name')) {
                    $pieces = explode(" ", $request->consumer_name);
                    if(count($pieces)){
                        $fName = !empty($pieces[0]) ? $pieces[0] : $request->consumer_name; 
                        $lName = !empty($pieces[1]) ? $pieces[1] : $request->consumer_name; 
                    }
                    $q->where("first_name","like","%".encryptString(ucfirst(strtolower($request->consumer_name)))."%");
                    $q->orWhere("last_name","like","%".encryptString(ucfirst(strtolower($request->consumer_name)))."%");
                    $q->orWhere(DB::raw('CONCAT(first_name," ",last_name)'), 'like', '%' .encryptString(ucfirst(strtolower($fName))).' '.encryptString(ucfirst(strtolower($lName))).'%');
                }
            })
            ->where(function ($q) use ($request) {
                if($request->filled('transaction_id')) {
                    $q->where("transaction_id",'like',"%".$request->transaction_id."%");
                }
                if($request->filled('date_from') && $request->filled('date_to')) {
                    $date_from = date("Y-m-d", strtotime($request->date_from));
                    $date_to = date("Y-m-d", strtotime($request->date_to));
                    $q->whereBetween('purchased_date', [$date_from." "."00:00:00", $date_to." "."23:59:59"]);
                }
            })
            ->orwhere(function ($q) use ($request) {
                if($request->filled('search')) {
                    $q->where("transaction_id",'like',"%".$request->search."%");
                }
            })
            ->orderBy($order_by_column_name, $order)
            ->paginate($limit);
            if (empty($query)) {
                throw new Exception(config('service.USER_API.MESSAGE.PLAN_PURCHASE_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.USER_API.MESSAGE.PLAN_PURCHASE_LISTING'), 'records' => $query]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function totalRevenue()
    {
        try {

            $totalPayement = Payment::select('amount')->sum('amount');

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.USER_API.MESSAGE.TOTAL_REVENUE'), 'revenue' => $totalPayement]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function latestTransactions(Request $request)
    {
        try {
            /*$pag_no = $request->has('page') ? $request->get('page') : 1;
            $per_page = 2;
            $skip = ($pag_no - 1) * $per_page ;*/

            $query = Payment::with(['customer','plan'])
            ->orderBy('id', 'DESC')
            //->skip($skip)
            //->take($per_page)
            ->take(10)
            ->get();

            
            // $query = Payment::with(['customer' => function($query) {
            //     // user_id is required here*
            //     $query->select(['id', 'first_name', 'last_name']);
            // },
            // 'plan' => function($query1) {
            //     // user_id is required here*
            //     $query1->select(['id','type as title','expiry_days ']);
            // },
            // ])
            // ->orderBy('id', 'DESC')
            // ->take(1)
            // ->get();

            if (empty($query)) {
                throw new Exception(config('service.USER_API.MESSAGE.LATEST_TRANSACTION_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.USER_API.MESSAGE.LATEST_TRANSACTION'), 'records' => $query]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    
    public function latestTransactions2(Request $request)
    {
        try {

            // $query = Payment::with(['customer','plan'])
            // ->orderBy('id', 'DESC')
            // //->skip($skip)
            // //->take($per_page)
            // ->take(1)
            // ->get();

            
            $query = Payment::with(['customer' => function($query) {
                $query->select(['id', 'first_name', 'last_name']);
            },
            'plan'
            ])
            ->orderBy('id', 'DESC')
            ->take(5)
            ->get();

            if (empty($query)) {
                throw new Exception(config('service.USER_API.MESSAGE.LATEST_TRANSACTION_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.USER_API.MESSAGE.LATEST_TRANSACTION'), 'records' => $query]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function salesReport()
    {
        try {
            $query = Payment::selectRaw('COUNT(*) as count, YEAR(purchased_date) year, MONTH(purchased_date) month, SUM(amount) amount')->groupBy('year', 'month')->get();
            if (empty($query)) {
                throw new Exception(config('service.USER_API.MESSAGE.SALE_REPORT_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.USER_API.MESSAGE.SALE_REPORT'), 'records' => $query]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function currentMonthSubscription(Request $request)
    {
        try {
            $query = Payment::selectRaw('COUNT(*) as count, MONTH(purchased_date) month')->where(DB::raw('MONTH(purchased_date)'),'=',date('m'))->groupBy('month')->get();

            if (empty($query)) {
                throw new Exception(config('service.USER_API.MESSAGE.MONTH_SUBSCRIPTION_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.USER_API.MESSAGE.MONTH_SUBSCRIPTION'), 'records' => $query]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function salesReportFilter(Request $request)
    {
        try {
            $query = Payment::selectRaw('COUNT(*) as count, YEAR(purchased_date) year, MONTH(purchased_date) month, SUM(amount) amount')->where(DB::raw('YEAR(purchased_date)'),'=',$request->year)->groupBy('year', 'month')->get();

            if (empty($query)) {
                throw new Exception(config('service.USER_API.MESSAGE.SALE_REPORT_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.USER_API.MESSAGE.SALE_REPORT'), 'records' => $query]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    private function getColumnName()
    {
        $address = Payment::first();
        if (!empty($address) && $address != "") {
            return array_keys($address->toArray());
        }
        return ['id'];
    }
}