<?php

namespace App\Http\Controllers\Payments;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use App\Http\Controllers\Controller;

use App\Models\Colors\Color;
use App\Models\Products\ProductVariant;
use App\Models\Payments\Payment;
use App\Models\Settings\Setting;

class PaymentController extends Controller
{
    
    public function index(Request $request) {
        has_scope('payments.read');
        try {
            $order = "desc";
            $limit = isset($request->limit) && $request->limit > 500 ? 25 : $request->limit;
    
            $obj = payment::with('order');
    
            if($request->filled('transaction')) {
                $obj = $obj->where('transaction',$request->transaction);
            }

            if($request->filled('commission_received')) {
                $obj = $obj->where('commission_received',$request->commission_recived);
            }
    
            $obj = $obj->orderBy('created_at', $order)->paginate($limit);
           
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.PAYMENT_LIST'), 'records' => $obj]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
       
    }
  
      
    
    public function store (Request $request) {
        has_scope('payments.write');
        
        $this->validate($request, [
           'order_id' => 'required',
           'user_id' => 'required',
           'price' => 'required',
           'comission' => 'required',
        ]);

        try {
            //get transaction prefix
            $prefix  = Setting::where('key','tansaction_prefix')->first();
            $prefix  = $prefix->value ? $prefix->value : '';

            $input = $request->all();
            $input['transcation_id'] = $prefix.rand(1111, 9999);

            $save  = payment::create($input);  
            $save->created_at=convertcurrenttimetoUtc(date('Y-m-d H:i:s'));
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.PAYMENT_DONE'), 'records' => $save]);

        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

   
}
