<?php

namespace App\Http\Controllers\Reports;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use App\Http\Controllers\Controller;

use App\Models\Reports\Report;

class ReportController extends Controller
{
    /**
     * The user repository implementation.
     *
     * @var UserRepository
     */

    public function index(Request $request) {
       
        try {

            $order = "desc";
            $limit = isset($request->limit) && $request->limit > 500 ? 25 : $request->limit;

            $obj  = Report::with('flagged_by');

            if(isset($request->order) && strtolower($request->order) == "asc") {
            
                $order  = $request->order;  
            }

            if(isset($request->keyword) &&$request->keyword!=''){
            
                $obj =  $obj->where(function ($query) use($request) {
                                $query->where('reason', 'like', '%' . $request->keyword . '%');
                             });
            }
            
            $result = $obj->orderBy('created_at', $order)->paginate($limit);
    
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.REPORT_LIST'), 'records' => $result]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }

    }

    public function store (Request $request) {
    
        $this->validate($request, [
           'node_id' => 'required|integer',
           'user_id' => 'required|integer',
           'type'    => 'required',
           'reason'  => 'required|string',

        ]);

        try {

            $input = $request->all();
            $data  = Report::create($input); 

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.REPORT_ADDED'), 'records' => $data]);

        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function delete(Request $request, $id) {

        $report = Report::findOrFail($id);

        try {

            $save  = $report->delete();

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.REPORT_DELETED'), 'records' => $save]);

        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function count() 
    {
        try {
            $count =  Report::count();

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.REPORT_COUNT'), 'count' => $count]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function show($id) 
    {
    
        $report = Report::findOrFail($id);

        try {
            $report = Report::with('flagged_by')->where('id',$id)->first();
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.REPORT_DETAIL'), 'result' => $report]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

}
