<?php
namespace App\Http\Controllers\TemplateManagement;
use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;
use App\Models\EmailTemplate\EmailTemplate;
use App\Jobs\SendMailJob;
class TemplateManagementController extends Controller
{
    /**
    * The user repository implementation.
    *
    * @var UserRepository
    */

    /**
    * @ cms pages listing api using pagination.
    * @return \Illuminate\Http\Response
    */
    public function listTemplateManagement(Request $request) 
    {
        try
        {
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;
            if (isset($request->order) && strtolower($request->order) == "asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";

            if($request->language == 'es') {
                app()->setLocale('es');
                $emailTemplates = EmailTemplate::where(function ($q) use ($request) {
                    // Global searching...
                    if (isset($request->search) && !empty($request->search) && $request->search != "") {
                        //if (!empty($this->getColumnName())) {
                           /* foreach ($this->getColumnName() as $key => $col) {
                                if ($key == 0) {*/
                                    $q->where('name', 'like', '%' . $request->search . '%');
                                    $q->orWhereTranslationLike("title","%".$request->search."%");
                                    $q->orWhereTranslationLike("text","%".$request->search."%");
                                /*} else {
                                    $q->orWhere($col, 'like', '%' . $request->search . '%');
                                    $q->orWhere(DB::raw('CONCAT(first_name, " ",last_name)'), 'like', '%' . $request->search . '%');
                                }*/
                            //}
                        //}
                    }
                })
                ->orderBy($order_by_column_name, $order)
                ->paginate($limit);
            }

            $emailTemplates = EmailTemplate::where(function ($q) use ($request) {
                    // Global searching...
                    if (isset($request->search) && !empty($request->search) && $request->search != "") {
                        //if (!empty($this->getColumnName())) {
                           /* foreach ($this->getColumnName() as $key => $col) {
                                if ($key == 0) {*/
                                    $q->where('name', 'like', '%' . $request->search . '%');
                                    $q->orWhereTranslationLike("title","%".$request->search."%");
                                    $q->orWhereTranslationLike("text","%".$request->search."%");
                                /*} else {
                                    $q->orWhere($col, 'like', '%' . $request->search . '%');
                                    $q->orWhere(DB::raw('CONCAT(first_name, " ",last_name)'), 'like', '%' . $request->search . '%');
                                }*/
                            //}
                        //}
                    }
                })
                ->orderBy($order_by_column_name, $order)
                ->paginate($limit);
            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.TEMPLATE_LISTING'), 'records' => $emailTemplates]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    /**
    * @ Cms view details
    * @return \Illuminate\Http\Response
    */


    public function activeInactive(Request $request, $id) 
    {
        $this->validate($request,[
            'status' => 'required',
        ]);
        try {
            $save_data['status'] = $request->status;
            $result = EmailTemplate::where('id', $id)->update($save_data);
            if($request->status=="active")
            {
                return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.TEMPLATE_STATUS_ACTIVE'),'action'=>$request->status]);
            }
            else
            {
                return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.TEMPLATE_STATUS_INACTIVE'),'action'=>$request->status]);
            }

        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
    }
    public function store(Request $request) 
    {
        $this->validate($request, [
            'es_title'  => 'required|max:2000',
            'en_title'  => 'required|max:2000',
            'es_text'   => 'required|max:20000',
            'en_text'   => 'required|max:20000',
            'name'      => 'required|max:191',
        ]);
        try {
            $input = $request->all();
            $data = [
                'name' =>$input['name'],
                'en' => [
                    'title'     => $input['en_title'],
                    'text'      => $input['en_text'],
                ],
                'es' => [
                    'title'  => $input['es_title'],
                    'text'   => $input['es_text']
                ],
            ];
            $result  = EmailTemplate::create($data);
            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.TEMPLATE_ADDED'), 'records' => $result]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function edit(Request $request,$id) 
    {
        try
        {

            $EmailTemplate =  EmailTemplate::find($id);
            if(empty($EmailTemplate)) {
                throw new Exception(config('service.USER_API.MESSAGE.TEMPLATE_NOT_FOUND'));
            }
            $TemplateEn = [
                'id' => $EmailTemplate->id,
                'title' => $EmailTemplate->title,
                'text' => $EmailTemplate->text
            ];
            app()->setLocale('es');
            $TemplatEs =  EmailTemplate::find($id);
            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.TEMPLATE_DETAILS'), 'recordEn' => $TemplateEn,'recordEs' => $TemplatEs]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function update(Request $request, $id) 
    {
        $this->validate($request, [
            'es_title'  => 'required|max:2000',
            'en_title'  => 'required|max:2000',
            'es_text'   => 'required|max:20000',
            'en_text'   => 'required|max:20000',
            'name'      => 'required|max:191',
        ]);
        try {
            $EmailTemplate = EmailTemplate::findOrFail($id);

            $input = $request->all();
            $data = [
                'name' =>$input['name'],
                'en' => [
                    'title'     => $input['en_title'],
                    'text'      => $input['en_text'],
                ],
                'es' => [
                    'title'  => $input['es_title'],
                    'text'   => $input['es_text']
                ],
            ];
            $data  = $EmailTemplate->update($data);
            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.TEMPLATE_UPDATED'), 'records' => $data]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

}