<?php
namespace App\Http\Controllers\v3\CMS;
use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;
use App\Models\CMS\CmsPage;
use App\Jobs\SendMailJob;
class PageController extends Controller
{
    /**
    * The user repository implementation.
    *
    * @var UserRepository
    */
    protected  $cmspage;
    protected  $user_id;
    public function __construct(CmsPage $cmspage, Request $request) {
        $this->cmspage = $cmspage;
        $this->user_id = $request->user_id;
    }
    /**
    * @ cms pages listing api using pagination.
    * @return \Illuminate\Http\Response
    */
    public function index(Request $request) {
        try
        {
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;
            if (isset($request->order) && strtolower($request->order) == "asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if($request->language == 'es') {
                app()->setLocale('es');
                $cmspage = $this->cmspage::where(function ($q) use ($request) {
                    // Global searching...
                    if (isset($request->search) && !empty($request->search) && $request->search != "") {
                        //if (!empty($this->getColumnName())) {
                           /* foreach ($this->getColumnName() as $key => $col) {
                                if ($key == 0) {*/
                                    $q->WhereTranslationLike("title","%".$request->search."%");
                                    $q->orWhereTranslationLike("description","%".$request->search."%");
                                /*} else {
                                    $q->orWhere($col, 'like', '%' . $request->search . '%');
                                    $q->orWhere(DB::raw('CONCAT(first_name, " ",last_name)'), 'like', '%' . $request->search . '%');
                                }*/
                            //}
                        //}
                    }
                })
                ->orderBy($order_by_column_name, $order)
                ->paginate($limit);
            }
            $cmspage = $this->cmspage::where(function ($q) use ($request) {
                    // Global searching...
                    if (isset($request->search) && !empty($request->search) && $request->search != "") {
                        //if (!empty($this->getColumnName())) {
                           /* foreach ($this->getColumnName() as $key => $col) {
                                if ($key == 0) {*/
                                    $q->WhereTranslationLike("title","%".$request->search."%");
                                    $q->orWhereTranslationLike("description","%".$request->search."%");
                                /*} else {
                                    $q->orWhere($col, 'like', '%' . $request->search . '%');
                                    $q->orWhere(DB::raw('CONCAT(first_name, " ",last_name)'), 'like', '%' . $request->search . '%');
                                }*/
                            //}
                        //}
                    }
                })
                ->orderBy($order_by_column_name, $order)
                ->paginate($limit);
            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.CMS_PAGE_LISTING'), 'records' => $cmspage]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    /**
    * @ Cms view details
    * @return \Illuminate\Http\Response
    */
    public function show(Request $request,$id) {
        try
        {
    /*if($request->language == 'en') {
    $result = $this->cmspage::select('id','title_en as title','description_en as description','status','created_at','updated_at');
    }else if($request->language == 'es') {
    $result = $this->cmspage::select('id','title_es as title','description_es as description','status','created_at','updated_at');
    }else{
    $result =  $this->cmspage::select('*');
    }*/

    $cmspage =  $this->cmspage::find($id);
    if( empty($cmspage) ) {
        throw new Exception(config('service.USER_API.MESSAGE.CMS_PAGE_NOT_FOUND'));
    }
    $cmsPageEn = [
        'id' => $cmspage->id,
        'title' => $cmspage->title,
        'description' => $cmspage->description,
    ];
    app()->setLocale('es');
    $cmsPageEs =  $this->cmspage::find($id);
    return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.CMS_PAGE_DETAILS'), 'recordEn' => $cmsPageEn,'recordEs' => $cmsPageEs]);
    } catch (Exception $e) {
        return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
    } catch(Throwable $e) {
        return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
    }
    }
    public function cmsPages() {
        try
        {
            if(!empty(request()->page)){
                $cmsPages =  $this->cmspage::where(['slug'=>request()->page])->first();
            }else{
                $cmsPages =  $this->cmspage::All();
            }
            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.CMS_PAGES'), 'records' => $cmsPages]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    private function getColumnName() {
        $col = $this->cmspage::first();
        if(!empty($col) && $col!="") {
            return array_keys($col->toArray());
        }
        return ['id'];
    }
    public function store(Request $request)
    {

        $this->validate($request, [
            'en_title' => 'required|max:190',
            'en_description' => 'required|max:50000',
            'es_title' => 'required|max:190',
            'es_description' => 'required|max:50000',
        ]);
        try {
            $input = $request->all();
            $pageName = strtolower($input['en_title']);
            $pagename_slug = preg_replace('/\s+/', '-', $pageName);
            $data = [
                'slug' => $pagename_slug,
                'en' => [
                    'title'  => $input['en_title'],
                    'description'   => $input['en_description']
                ],
                'es' => [
                    'title'  => $input['es_title'],
                    'description'   => $input['es_description']
                ],
            ];
            $result  = CmsPage::create($data);
            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.CMS_PAGE_ADDED'), 'records' => $result]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'en_title' => 'required|max:190',
            'en_description' => 'required|max:50000',
            'es_title' => 'required|max:190',
            'es_description' => 'required|max:50000',
        ]);
        try {

            $page = CmsPage::findOrFail($id);

            $input = $request->all();
            $pageName = strtolower($input['en_title']);
            $pagename_slug = preg_replace('/\s+/', '-', $pageName);
            $data = [
                'slug' => $pagename_slug,
                'en' => [
                    'title'  => $input['en_title'],
                    'description'   => $input['en_description']
                ],
                'es' => [
                    'title'  => $input['es_title'],
                    'description'   => $input['es_description']
                ],
            ];
            $data  = $page->update($data);
    //admin mail
    /*$greetingAdmin = 'Dear Admin';
    $bodyAdmin     = 'The page information is updated successfully.';
    $typeAdmin     = 'CMS Page Updated';

    $adminMail = new SendMailJob(env('ADMIN_EMAIL'), $greetingAdmin, $bodyAdmin, $typeAdmin);
    dispatch($adminMail);*/
    return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.CMS_PAGE_UPDATED'), 'records' => $data]);
    } catch (Exception $e) {
        return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
    } catch(Throwable $e) {
        return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
    }
    }
    public function delete($id) {

        $page = CmsPage::findOrFail($id);
        try {

            $data  = $page->delete();
    //admin mail
    /*$greetingAdmin = 'Dear Admin';
    $bodyAdmin     = 'Page name is deleted successfully';
    $typeAdmin     = 'CMS Page deleted';

    $adminMail = new SendMailJob(env('ADMIN_EMAIL'), $greetingAdmin, $bodyAdmin, $typeAdmin);
    dispatch($adminMail);*/
    return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.CMS_PAGE_DELETED'), 'records' => $data]);
    } catch (Exception $e) {
        return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
    } catch(Throwable $e) {
        return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
    }
    }
    public function activeInactive(Request $request, $id) {

        $this->validate($request,[
            'status' => 'required',
        ]);
        try {
            $save_data['status'] = $request->status;
            $result = CmsPage::where('id', $id)->update($save_data);

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.CMS_STATUS')]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
    }
    public function publishAction(Request $request, $id) {
        has_scope('cms.pages.write');
        $page = StaticPage::find($id);
        if(!$page) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.USER_API.MESSAGE.NOT_FOUND')]);
        }
        $this->validate($request, [
            'action' => 'required|in:publish,unpublish',
        ]);
        try {
            $status  = $request->action == 'publish' ? 1 : 0;
            $message = $request->action == 'publish' ? config('service.USER_API.MESSAGE.CMS_PAGE_PUBLISHED') : config('service.USER_API.MESSAGE.CMS_PAGE_UNPUBLISHED');
            $page->status = $status;
            $data = $page->update();
            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => $message, 'records' => $data]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
}