<?php

namespace App\Http\Controllers\v3\CMS;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;

use App\Models\CMS\ReportAProblem;

class ReportAProblemController extends Controller
{
    /**
     * The user repository implementation.
     *
     * @var UserRepository
     */
    protected  $report_problem;
    protected  $user_id;

    public function __construct(ReportAProblem $report_problem, Request $request) {
        $this->report_problem = $report_problem;

        $this->user_id = $request->user_id;
    }

    /**
     * @ report a problem list api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function index(Request $request) {
        try
        {
            $order = "desc";
            $limit = isset($request->limit) && $request->limit > 500 ? 25 : $request->limit;
            if(isset($request->order) && strtolower($request->order)=="asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if(isset($request->order_by_column_name)) {
                if(in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }

            $report_problem = $this->report_problem::where(function($q) use($request) {
                // Global searching...
                if(isset($request->keyword) && !empty($request->keyword) && $request->keyword!="") {
                    if(!empty($this->getColumnName())) {
                        foreach ($this->getColumnName() as $key => $col) {
                            if($key==0) {
                                $q->where($col, 'like', '%' .$request->keyword. '%');
                            } else {
                                $q->orWhere($col, 'like', '%' .$request->keyword. '%');
                            }
                        }
                    }
                }
            })
            ->where(function($q) use($request) {
                // Column wise searching filter ...
                $allowableColumn = $this->getColumnName();
                if(isset($request->column_name) && !empty($request->column_name) && $request->column_name!="" && in_array($request->column_name, $allowableColumn)) {
                    $q->where($request->column_name,  'like', '%' .$request->column_value. '%');
                }
            })
            ->orderBy($order_by_column_name, $order)
            ->paginate($limit);

            $image_path = url('/assets/images/report_a_problem');
            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.REPORT_PROBLEM_LIST'), 'records' => $report_problem, 'image_path'=> $image_path]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    /**
     * report a problem api.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'subject' => 'required|max:256',
            'problem' => 'required|max:256',
            'image' => $request->has('image') ? 'required|mimes:jpeg,png,jpg,svg,gif' : "",
        ]);

        try {

            $input = $request->all();
            $input['user_id'] = $this->user_id;
            if($request->has('image')) {
                $image = $request->file('image');
                $imagename = time().'.'.$image->getClientOriginalName();
                $image->move(public_path('assets/images/report_a_problem'), $imagename);
                $input['image'] = @$imagename;
            }
            $report_problem = $this->report_problem::create($input);
            $report_problem['image_path'] = url('/assets/images/report_a_problem');

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.REPORT_PROBLEM_STORE'), 'records' => $report_problem]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * @ report a problem  details api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function show($id) {
        try
        {
            $report_problem = $this->report_problem::find($id);

            if( empty($report_problem) ) {
                throw new Exception(config('service.USER_API.MESSAGE.REPORT_PROBLEM_NOT_FOUND'));
            }

            $image_path = url('/assets/images/report_a_problem');
            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.REPORT_PROBLEM_INFO'), 'records' => $report_problem, 'image_path'=> $image_path]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * Remove the specified contact us query from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function delete(Request $request, $id)
    {
        try {
            $report_problem = $this->report_problem::find($id)->delete();

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.REPORT_PROBLEM_DELETED'), 'records' => $report_problem]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    private function getColumnName() {
        $col = $this->report_problem::first();
        if(!empty($col) && $col!="") {
            return array_keys($col->toArray());
        }

        return ['id'];
    }
}
