<?php

namespace App\Http\Controllers\v3\Chat;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;

use App\Models\Chat\MessageRoom;
use App\Models\Chat\MessageRoomChat;
use App\Models\Chat\AudioVideoCallHistory;
use Validator;

class ChatController extends Controller
{
    /**
     * The user repository implementation.
     *
     * @var UserRepository
     */
    protected  $room;
    //protected  $users;
    protected  $user_message;
    protected  $user_id;
    //protected  $user_role;
    protected  $customer_image_path;
    protected  $contractor_image_path;
    protected  $image_path;

    public function __construct(MessageRoom $room, Request $request, MessageRoomChat $user_message) {
        $this->room = $room;
        //$this->users = $users;
        $this->user_message = $user_message;
        $this->user_id = $request->user_id;
        //$this->user_role = $request->user_role;

        /*$this->customer_image_path = url('assets/images/users');
        $this->contractor_image_path = url('assets/images/contractors');
        $this->image_path = url('assets/images/chats');*/
    }

    public function index(Request $request)
    {
        try
        {
            $service_id = isset($request->service_id) ? explode(',', $request->service_id) : [];

            if(strtolower($request->user_type) == 'provider') {
                $whr = array(['provider_id', '=', $this->user_id],['deleted_by_provider','=',null]);
            } 
            if(strtolower($request->user_type) == 'customer') {
                $whr = array(['customer_id', '=', $this->user_id],['deleted_by_customer','=',null]);
            }
            
            $all_room = $this->room::has('customer')
                        ->has('provider')
                        ->with(['customer' => function($q) {
                        $q->select(['id', 'first_name', 'last_name', 'image']);
                        },'provider:id,first_name,last_name,image', 
                        'unread_message' => function($q) {
                        $q->select('room_id', DB::raw('COUNT(id) as unread_message_count'))
                            ->where('status', '=', 0)
                            ->groupBy('room_id');
                        }]);

            /*if($request->filled('service_id')) {
                    $all_room  =  $all_room->whereHas('contractor.contractor_service', function($q) use($service_id) {
                                    if(!empty($service_id) && count($service_id)>0) {
                                            $q->whereIn('service_id', $service_id);
                                        }
                                    });
            }*/
                
            $all_room = $all_room->where($whr)->paginate(10);    

            return response()->json(['status'=> config('service.SUCCESS_STATUS'), 'message' => config('service.ROOM_LIST'), 'records' => $all_room]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function getMessageById(Request $request, $id)
    {
        $roomId = $id;
        try
        {
            $room = $this->room::find($roomId);
            if(strtolower($request->user_type) == 'customer') {
                $join = ['provider' => function($q) {
                     $q->select(['id', 'first_name', 'last_name', 'image']);
                }];
            } 
            if(strtolower($request->user_type) == 'provider') {
                $join = ['customer' => function($q) {
                    $q->select(['id', 'first_name', 'last_name', 'image']);
                }];
            }
            if(!empty($room))
            {
                $order = "desc";
                $limit = isset($request->limit) ? $request->limit : 25;
                if($limit > 500) {
                    $limit = 25;
                }
                if(isset($request->order) && strtolower($request->order)=="asc") {
                    $order = $request->order;
                }
                $order_by_column_name = "id";
                if(isset($request->order_by_column_name)) {
                    if(in_array($request->order_by_column_name, $this->getColumnName())) {
                        $order_by_column_name = $request->order_by_column_name;
                    }
                }

                $all_message = $this->user_message::where(function($q) use($room,$request) {
                    if(strtolower($request->user_type) == 'customer') {
                        $q->where([
                            ['room_id', '=', $room->room_id],
                            //['customer_delete_chat_status', '=', 'ACTIVE'],
                        ]);
                    } else {
                        $q->where([
                            ['room_id', '=', $room->room_id],
                            //['contractor_delete_chat_status', '=', 'ACTIVE'],
                        ]);
                    }
                })
                ->where(function($q) use($request) {
                    // Global searching...
                    if(isset($request->search) && !empty($request->search) && $request->search!="") {
                        if(!empty($this->getColumnName())) {
                            foreach ($this->getColumnName() as $key => $col) {
                                if($key==0) {
                                    $q->where($col, 'like', '%' .$request->search. '%');
                                } else {
                                    $q->orWhere($col, 'like', '%' .$request->search. '%');
                                }
                            }
                        }
                    }
                })
                ->where(function($q) use($request) {
                    // Column wise searching filter ...
                    $allowableColumn = $this->getColumnName();
                    if(isset($request->column_name) && !empty($request->column_name) && $request->column_name!="" && in_array($request->column_name, $allowableColumn)) {
                        $q->where($request->column_name,  'like', '%' .$request->column_value. '%');
                    }
                })
                ->orderBy($order_by_column_name, $order)
                ->paginate($limit);

                return response()->json(['status'=> config('service.SUCCESS_STATUS'), 'message' => config('service.CHAT_MESSAGES_LIST'), 'records' => $all_message]);
            }
            else
            {
                throw new Exception(config('service.ROOM_NOT_EXISTS'));
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /*
        # $id id receiver id
    */
    public function sendMessageById(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'message' => isset($request->message) ? 'required|max:256' : "",
            'room_id' => isset($request->room_id) ? 'required|max:256' : "",
            'firebase_sender_id' => 'required',
            'firebase_peer_id' => 'required',
            'image' => $request->has('image') ? 'required|mimes:jpeg,png,jpg,svg,gif' : "",
        ]);

        if($validator->fails()){
            return response()->json(['status' => config('service.ERROR_STATUS'),'message' => $validator->errors()->first()],200);      
        }

        try
        {
            if(strtolower($request->user_type) == 'customer') {
                $customer_id = $this->user_id;
                $provider_id = $request->id;
                $receiver_type = "provider"; 
            }
            if(strtolower($request->user_type) == 'provider') {
                $provider_id = $this->user_id;
                $customer_id = $request->id;
                $receiver_type = "customer"; 
            }

            $uniq_sender_reciever_id = $customer_id . '-'. $provider_id;
            $uniqueRoomId = $this->generateUniqueRoomID($uniq_sender_reciever_id);
            $roomId = !empty($request->room_id) ? $request->room_id : $uniqueRoomId;

            $room = $this->room::where('room_id',$roomId)->first();
            if(!empty($room))
            {
                // Update room with latest message
                $room = $this->room::find($room->id);
                $input['last_message'] = $request->message;
                $input['last_message_date_time'] = date('Y-m-d H:i:s');
                $room->update($input);

                if(strtolower($request->user_type) == 'customer') {
                    $reciever_id = !empty($room->provider_id) ? $request->id : $room->provider_id;
                } 
                if(strtolower($request->user_type) == 'provider') {
                    $reciever_id = !empty($room->customer_id) ? $request->id : $room->customer_id;
                }
                // save user message
                $input['room_id_db'] = $this->user_id . '-' .$reciever_id;
                $input['sender_id']   = $this->user_id;
                $input['sender_type']   = $request->user_type;
                $input['receiver_type']   = $receiver_type;
                $input['receiver_id'] = $reciever_id;
                $input['room_id'] = $roomId;
                $input['message'] = $request->message;
                $input['firebase_sender_id'] = $request->firebase_sender_id;
                $input['firebase_peer_id'] = $request->firebase_peer_id;
                if($request->has('image')) {
                    /*$image = $request->file('image');
                    $imagename = time().'.'.$image->getClientOriginalName();
                    $image->move(public_path('assets/images/chats'),$imagename);
                    $input['image'] = $imagename;*/
                }

                $message = $this->user_message::create($input);

                if(strtolower($request->user_type) == 'customer') {
                    $join = ['provider' => function($q) {
                        $q->select(['id', 'first_name', 'last_name', 'image']);
                    }];
                } 
                if(strtolower($request->user_type) == 'provider') {
                    $join = ['customer' => function($q) {
                        $q->select(['id', 'first_name', 'last_name', 'image']);
                    }];
                }
                $resp = MessageRoomChat::with($join)->with('chat_room')->find($message->id);

                return response()->json(['status'=> config('service.SUCCESS_STATUS'), 'message' => config('service.CHAT_MESSAGE_SENT'), 'records' => $resp]);
            }
            else
            {
                // check sender & reciever pair is already exists or not
                $checkRoomExists = MessageRoom::where('room_id', '=', $roomId)->exists();
                if($checkRoomExists) {
                    $user_type = "customer";
                    if(strtolower($request->user_type) == 'customer') {
                        $user_type = "provider";
                    }
                    throw new Exception("Sorry you have already exists room_id pair with this ".$user_type.". please use room_id to send message to this ".$user_type);
                }

                $input['customer_id'] = $customer_id;
                $input['provider_id'] = $provider_id;
                $input['room_id'] = $roomId;
                $input['last_message'] = $request->message;
                $input['last_message_date_time'] = date('Y-m-d H:i:s');
                MessageRoom::create($input);

                 // save user message
                $input['room_id_db'] = $this->user_id . '-' . $request->id;
                $input['sender_id']   = $this->user_id;
                $input['sender_type']   = $request->user_type;
                $input['receiver_type']   = $receiver_type;
                $input['receiver_id'] = $request->id;
                $input['message'] = $request->message;
                $input['firebase_sender_id'] = $request->firebase_sender_id;
                $input['firebase_peer_id'] = $request->firebase_peer_id;
                if($request->has('image')) {
                    /*$image = $request->file('image');
                    $imagename = time().'.'.$image->getClientOriginalName();
                    $image->move(public_path('assets/images/chats'),$imagename);
                    $input['image'] = $imagename;*/
                }
                $message = MessageRoomChat::create($input);

                if(strtolower($request->user_type) == 'customer') {
                    $join = ['provider' => function($q) {
                        $q->select(['id', 'first_name', 'last_name', 'image']);
                    }];
                } 
                if(strtolower($request->user_type) == 'provider') {
                    $join = ['customer' => function($q) {
                        $q->select(['id', 'first_name', 'last_name', 'image']);
                    }];
                }
                $resp = MessageRoomChat::with($join)->with('chat_room')->find($message->id);

                return response()->json(['status'=> config('service.SUCCESS_STATUS'), 'message' => config('service.CHAT_MESSAGE_SENT'), 'records' => $resp]);
            }

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function markAsRead(Request $request, $id)
    {
        try
        {
            $input['status'] = 1;
            $message = $this->user_message::find($id);
            $message->update($input);

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.MESSAGE_CHAT_SEEN'), 'records' => $message]);

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    private function getColumnName() {
        $col_name = [];
        $col = $this->user_message::first();
        if(!empty($col) && $col!="") {
             return array_keys($col->toArray());
           /* foreach (array_keys($col->toArray()) as $key => $value) {
                $col_name[] = 'booking.'.$value;
            }
            $extraCol = ['customers.first_name', 'customers.image', 'contractors.first_name', 'contractors.image'];
            return array_merge($col_name, $extraCol);*/
        }

        return ['id'];
    }

    private function generateUniqueRoomID($uniq_sender_reciever_id) {
        return md5($uniq_sender_reciever_id);
    }

    // here $id = chat room's ID
    public function deleteMyChat(Request $request, $id)
    {
        try
        {
            $roomId = $id;
            $room = $this->room::find($roomId);
            if(!empty($room) && $room)
            {
                if(strtolower($this->user_role) == 'customer') {
                    $input['customer_delete_chat_status'] = 'DELETED';
                } else {
                    $input['contractor_delete_chat_status'] = 'DELETED';
                }

                $this->user_message::where('room_id', '=', $room->room_id)->update($input);

                return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.MESSAGE_CHAT_DELETE'), 'records' => []]);
            }
            else
            {
                throw new Exception(config('service.USER_API.MESSAGE.ROOM_NOT_EXISTS'), 1);
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function deleteChatRoom(Request $request, $id) {
        try {
        
            $room   = MessageRoom::findorFail($id);
            if($request->user_type == "customer"){
                $room->deleted_by_customer = date("Y-m-d");
            }else{
                $room->deleted_by_provider = date("Y-m-d");
            }
            if($room->save()){
                return response()->json(['status'=> config('service.SUCCESS_STATUS'), 'message' => config('service.ROOM_DELETE')]);
            }

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function upload(Request $request)
    {

        $this->validate($request, [
           'image' => 'required|mimes:jpeg,png,jpg,svg,gif|max:5120',
        ]);
        
        try {
    
        if ($request->has('image')) {
                $image = $request->file('image');
                $imagename = time().'.'.$image->getClientOriginalName();
                $image->move(public_path('assets/images/chats'),$imagename);
                $url = url('/assets/images/chats').'/'.$imagename;
        }
           
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'records' => $url]);
        } 
        catch (Exception $e) {

            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } 
        catch (Throwable $e) {

            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function audioVideoSessionList(Request $request)
    {
        try
        {
            $validator = Validator::make($request->all(), [
                'user_type'  => 'required',
            ]);
            if($validator->fails()){
                return response()->json(['status' => config('service.ERROR_STATUS'),'message' => $validator->errors()->first()],200);      
            }
            if(strtolower($request->user_type) == 'provider') {
                $whr = array(['provider_id', '=', $this->user_id]);

                 $join = ['customer' => function($q) {
                    $q->select(['id', 'first_name', 'last_name', 'image']);
                }];
            } 
            if(strtolower($request->user_type) == 'customer') {
                $whr = array(['customer_id', '=', $this->user_id]);

                 $join = ['provider' => function($q) {
                     $q->select(['id', 'first_name', 'last_name', 'image']);
                }];
            }
            
            $audioVideoCallHistory = AudioVideoCallHistory::with($join)
                                    ->where($whr)
                                    ->paginate(10);
     
            return response()->json(['status'=> config('service.SUCCESS_STATUS'), 'message' => config('service.AUDIO_VIDEO_SESSION_List'), 'records' => $audioVideoCallHistory]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
}
