<?php

namespace App\Http\Controllers\v3\Contractors;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\Contractors\Jobs;
use GuzzleHttp\Client;
use Illuminate\Support\Facades\DB;

use App\Models\Contractors\Customer;
use App\Models\Contractors\Oauth;
use App\Notifications\CreateNewUserNotification;

class SocialAuthController extends Controller
{
    /**
     * The user repository implementation.
     *
     * @var UserRepository
     */
    protected $users;
    protected $oauth_user;
    protected $jobs;

    public function __construct(Customer $users, Oauth $oauth_user, Jobs $jobs) {
        $this->users = $users;
        $this->oauth_user = $oauth_user;
        $this->jobs = $jobs;
    }

    /**
     * @ user list List api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function index(Request $request) {
        try
        {
            $order = "desc";
            $limit = isset($request->limit) && $request->limit > 500 ? 25 : $request->limit;
            if(isset($request->order) && strtolower($request->order)=="asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if(isset($request->order_by_column_name)) {
                if(in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }

            $users = $this->users::where('status', '=', 1)
                    ->where(function($q) use($request) {
                        // Global searching...
                        if(isset($request->search) && !empty($request->search) && $request->search!="") {
                            if(!empty($this->getColumnName())) {
                                foreach ($this->getColumnName() as $key => $col) {
                                    if($key==0) {
                                        $q->where($col, 'like', '%' .$request->search. '%');
                                    } else {
                                        $q->orWhere($col, 'like', '%' .$request->search. '%');
                                    }
                                }
                            }
                        }
                    })
                    ->where(function($q) use($request) {
                        // Column wise searching filter ...
                        $allowableColumn = $this->getColumnName();
                        if(isset($request->column_name) && !empty($request->column_name) && $request->column_name!="" && in_array($request->column_name, $allowableColumn)) {
                            $q->where($request->column_name,  'like', '%' .$request->column_value. '%');
                        }
                    })
                    ->orderBy($order_by_column_name, $order)
                    ->paginate($limit);

            if( empty($users) ) {
                throw new Exception(config('service.USER_API.MESSAGE.USER_NOT_FOUND'));
            }

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.USER_DETAILS'), 'records' => $users]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function socialLogin(Request $request) {
        $this->validate($request, [
            'social_type'  => 'required|in:facebook,instagram,apple,google',
            'social_id'    => 'required',
            'device_id'    => 'required',
            'device_type'  => 'required',
            'device_token' => 'required',
        ]);

        try {
            DB::beginTransaction();
            $email = $request->email;
            $phone = $request->phone;
            switch ($request->social_type) {
                case 'facebook':
                    $check = $this->users::where('fb_social_id', '=', $request->social_id)
                        ->orWhere('email',$email)
                        ->orWhere('phone',$phone);
                    $pass = $this->generateRandomPassword();
                    if(! $check->exists() ) {
                        $input = $request->all();
                        $input['password'] = $this->hashPassword($pass);
                        $input['fb_social_id'] = $request->social_id;
                        if($user = $this->users->create($input)) {
                            $oauth = new $this->oauth_user;
                            $oauth->user_id = $user->id;
                            $oauth->role_fk = 'customer';
                            $oauth->username = $request->email;
                            $oauth->password = $input['password'];
                            $oauth->first_name = $request->first_name;
                            $oauth->last_name = $request->last_name;
                            $oauth->email = $request->email;
                            $oauth->save();
                            DB::commit();

                            $details = [
                                'greeting' => 'Hi '. $request->first_name .'!',
                                'body'=> 'Your email : '. $request->email .' and your login password is : '.$pass,
                                'database_notification_message' => 'New user has been created and login as social login successfully.',
                                'thanks' => 'Welcome to ZION!!',
                            ];
                            $role = 'customer';
                            $user->notify(new CreateNewUserNotification($role, $details));
                            $upd['user_type'] = $role;
                            $notif_id = $user->notifications[0]->id;
                            $user->notifications()->find($notif_id)->update($upd);

                            // now hit auth2.0 create token api
                            $client = new Client();
                            $url = url('api/v2/oauth2/token');
                            $resp = $client->post($url, [
                                "form_params" => [
                                        "grant_type" => config('service.passport.grant_type'),
                                        "client_secret" => config('service.passport.client_secret'),
                                        "client_id" => config('service.passport.client_id'),
                                        "username" => $request->email,
                                        "password" => $pass
                                ]
                            ]);
                            // Response body content (JSON string).
                            $responseJson = $resp->getBody()->getContents();
                            // Response body content as PHP array.
                            $responseData = json_decode($responseJson, true);

                            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.USER_SOCIAL_LOGIN'), 'records' => $responseData]);
                        }
                        else
                        {
                            throw new Exception(config('service.USER_API.MESSAGE.SOMETHING_WENT_WRONG'), 1);
                        }
                    }
                    else
                    {
                        // now hit auth2.0 create token api
                        $client = new Client();
                        $email = $request->email;
                        $pass = 'GhcMODUi';
                        $url = url('api/v2/oauth2/token');
                        $resp = $client->post($url, [
                            "form_params" => [
                                    "grant_type" => config('service.passport.grant_type'),
                                    "client_secret" => config('service.passport.client_secret'),
                                    "client_id" => config('service.passport.client_id'),
                                    "username" => $email,
                                    "password" => $pass
                            ]
                        ]);

                        // Response body content (JSON string).
                        $responseJson = $resp->getBody()->getContents();
                        // Response body content as PHP array.
                        $responseData = json_decode($responseJson, true);
                        print_r($responseData);
                        die;

                    }
                    break;

                default:
                    # code...
                    break;
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function checkSociaLogin(Request $request)
    {
        return response()->json([
            'id' => 1,
            'data'=>'kkk',
        ]);
        $validator = Validator::make($request->all(), [
            'social_type'  => 'required',
            'social_id'    => 'required',
            'device_id'    => 'required',
            'device_type'  => 'required',
            'device_token' => 'required',
        ],
        [
            'email.required'=> 'Please enter an email',
        ]);

            $email  = $request->input('email') ?? '';
            $phone = $request->input('mobile') ?? '';

        if ($validator->fails()) {
            return response()->json(['error'=>$validator->errors()], 401);
        }

        if ($request->social_type == 'facebook') {
        $user_data = User::where('fb_social_id',$request->social_id)
                        ->orWhere('email',$email)
                        ->orWhere('phone',$phone)
                        ->first();
        if (empty($user_data)) {
            $success['success'] = "true";
            $success['message'] = "social details not found";
            return response()->json($success, $this-> successStatus);
            }
        }
            if ($request->social_type == 'google') {
            $user_data = User::where('google_social_id',$request->social_id)
                        ->orWhere('email',$email)
                        ->orWhere('phone',$phone)
                        ->first();
            if (empty($user_data)) {
            $success['success'] = "true";
            $success['message'] = "social details not found";
            return response()->json($success, $this-> successStatus);
            }
            }
            $user = User::find($user_data['id']);

            if(@$email || @$mobile){
                if($request->social_type == 'google'){
                    $user->google_social_id = @$request->input('social_id');
                }else{
                    $user->fb_social_id    = @$request->input('social_id');
                }
            }
            $user->save();

            // token //
            $deviceToken = DeviceToken::where('user_id', @$user_data['id'])->where('device_id', @$request->input('device_id'))->first();
            if (!empty($deviceToken)) {
                DeviceToken::where('user_id', @$user_data['id'])->where('device_id', @$request->input('device_id'))->update(['device_token' =>@$request->input('device_token')]);
            }else{
            $DeviceToken = new DeviceToken;
            $DeviceToken->user_id         = @$user_data['id'];
            $DeviceToken->device_id       = $request->input('device_id');
            $DeviceToken->device_token    = $request->input('device_token');
            $DeviceToken->device_type     = $request->input('device_type');
            $save = $DeviceToken->save();
            }


            $success['success'] = "true";
            $success['message'] = "You are logged in successfully";
            if ($user->user_type == 2) {
                $success['data'] = [
                'id'            => $user->id,
                'name'          => $user->name,
                'busker_unique_id'=> $user->busker_unique_id,
                'email'         => $user->email,
                'phone'         => $user->phone,
                'user_type'     => $user->user_type,
                'latitude'      => $user->latitude,
                'longitude'     => $user->longitude,
                'firebase_id'   => $user->firebase_id,
                'profile_img'   => $user->profile_image ? public_path('/uploads/profile_image/'.$user->profile_image.'')  : null,
                'created_at'    => $user->created_at,
                'access_token'  =>  $user->createToken('MyApp')-> accessToken
            ];
            }else{
                $success['data'] = [
                'id'            => $user->id,
                'name'          => $user->name,
                'email'         => $user->email,
                'phone'         => $user->phone,
                'user_type'     => $user->user_type,
                'latitude'      => $user->latitude,
                'longitude'     => $user->longitude,
                'firebase_id'   => $user->firebase_id,
                'profile_img'   => $user->profile_image ? public_path('/uploads/profile_image/'.$user->profile_image.'')  : null,
                'created_at'    => $user->created_at,
                'access_token'  =>  $user->createToken('MyApp')-> accessToken
            ];
            }
            return response()->json($success, $this-> successStatus);
    }

    function generateRandomPassword() {
        $alphabet = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890';
        $pass = array(); //remember to declare $pass as an array
        $alphaLength = strlen($alphabet) - 1; //put the length -1 in cache
        for ($i = 0; $i < 8; $i++) {
            $n = rand(0, $alphaLength);
            $pass[] = $alphabet[$n];
        }
        return implode($pass); //turn the array into a string
    }

    private function getColumnName() {
        $address = $this->users::first();
        if(!empty($address) && $address!="") {
            return array_keys($address->toArray());
        }

        return ['id'];
    }

    private function hashPassword($password) {
        return sha1($password);
    }
}
