<?php

namespace App\Http\Controllers\v3\Customers;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\Customers\Booking;
use App\Models\Customers\Contractor;
use Illuminate\Support\Facades\DB;

use App\Models\Customers\Jobs;
use App\Models\Customers\Oauth;
use App\Notifications\JobsNotification;

class JobsController extends Controller
{
    /**
     * The user repository implementation.
     *
     * @var UserRepository
     */
    protected  $jobs;
    protected  $user_id;
    protected  $role_fk;
    protected  $bookings;
    protected  $oauth_user;
    protected  $contractor;

    public function __construct(Jobs $jobs, Request $request, Booking $bookings, Oauth $oauth_user, Contractor $contractor)
    {
        
        $this->jobs = $jobs;
        $this->bookings = $bookings;
        $this->user_id = $request->user_id;
        $this->oauth_user = $oauth_user;
        $this->role_fk = $request->role_fk;
        $this->contractor = $contractor;
    }

    
    /**
     * @ contact details api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function show($id)
    {
        
        try
        {
            $jobs = $this->bookings::with(['user.address' => function ($query) {
                $query->where('user_type', '=', 1);
            }, 'contractor.address' => function ($query) {
                $query->where('user_type', '=', 2);
            }, 'service.category', 'jobs'])
            ->where([
                ['id', '=', $id],
                ['user_id', '=', $this->user_id],
            ])->get();

            if( empty($jobs) ) {
                throw new Exception(config('service.USER_API.MESSAGE.JOBS_NOT_FOUND'));
            }

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.JOBS_DETAILS'), 'records' => $jobs]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    
    public function updateJobStatus(Request $request, $id)
    {
        $this->validate($request, [
            'job_status' => 'required|in:1,2,3,4,5',
            'specific_reason' => $request->job_status == 5 ? 'required|max:256' : "",
        ]);

        try {
            $input = $request->all();
            $input['status'] = $request->job_status;
            $input['reason'] = $request->specific_reason;
            $jobs = $this->jobs::find($id);
            $jobs->update($input);
            $booking_id = $jobs->booking_id;

            // // also update booking table status
            $this->bookings::find($booking_id)->update($input);

            // database notification
            if($request->job_status==1) {
                $msg = "Jobs has been accepted";
                $body = "Jobs has been accepted";
            }
            if($request->job_status==2) {
                $msg = "Jobs on the way";
                $body = "Jobs on the way";
            }
            if($request->job_status==3) {
                $msg = "Jobs is in the progress";
                $body = "Jobs is in the progress";
            }
            if($request->job_status==4) {
                $msg = "Jobs has been completed";
                $body = "Jobs has been completed";
            }
            if($request->job_status==5) {
                $body =  $request->specific_reason;
                $msg = "Jobs has been decline";
            }
            $user = $this->contractor::find($this->user_id);
            $details = [
                'greeting' => 'Hi '. $user->first_name .'!',
                'body'=> @$body,
                'database_notification_message' => $msg,
                'thanks' => 'Welcome to ZION!!',
            ];
            $role = 'contractor';
            $user->notify(new JobsNotification($role, $details));
            $upd['user_type'] = $role;
            $notif_id = $user->notifications[0]->id;
            $user->notifications()->find($notif_id)->update($upd);


            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.JOBS_DECLINE_SUCCESS'), 'records' => $jobs]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    
    public function startEndJob(Request $request, $id)
    {
        $this->validate($request, [
            'job_started_datetime' => isset($request->job_started_datetime) && $request->job_started_datetime ? 'required' : "",
            'job_end_datetime' => isset($request->job_end_datetime) && $request->job_end_datetime ? 'required' : "",
        ]);

        // echo "<pre>"; print_r($request->all()); die;

        try {
            $msg = '';
            $input = $request->all();
            if(isset($request->job_started_datetime) && $request->job_started_datetime){
                $input['job_started'] = date('Y-m-d H:i:s', strtotime($request->job_started_datetime));
                $msg = config('service.USER_API.MESSAGE.JOBS_START');
            }
            if(isset($request->job_end_datetime) && $request->job_end_datetime){
                $input['job_end'] = date('Y-m-d H:i:s', strtotime($request->job_end_datetime));
                $msg = config('service.USER_API.MESSAGE.JOBS_END');
            }
            $jobs = $this->jobs::find($id);
            $jobs->update($input);

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => $msg, 'records' => $jobs]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    
    
    public function allCustomerJobs(Request $request, $id) 
    {
        try {
            
            $order = "desc";
            $limit = isset($request->limit) && $request->limit > 500 ? 25 : $request->limit;
            if (isset($request->order) && strtolower($request->order) == "asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if (isset($request->order_by_column_name)) {
                if (in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }

            $jobs = $this->jobs::select(['booking.id', 'booking.user_id', 'booking.contractor_id', 'service-category.sub_category_name', 'booking.booking_date', 'booking.booking_time', 'job_tracking.status'])
                    ->join('booking', 'booking.id', '=', 'job_tracking.booking_id')
                    ->join('services', 'services.id', '=', 'booking.service_id')
                    ->join('service-category', 'service-category.id', '=', 'booking.service_cat_id')
                    ->where(function ($q) use ($id) {
                        $q->where('booking.user_id', '=', $id);
                    })
                    ->where(function ($q) use ($request) {
                        // Global searching...
                        if (isset($request->search) && !empty($request->search) && $request->search != "") {
                            if (!empty($this->getJobsColumnName())) {
                                foreach ($this->getJobsColumnName() as $key => $col) {
                                    if ($key == 0) {
                                        $q->where($col, 'like', '%' . $request->search . '%');
                                    } else {
                                        $q->orWhere($col, 'like', '%' . $request->search . '%');
                                    }
                                }
                            }
                        }
                    })
                    ->where(function ($q) use ($request) {
                        // Column wise searching filter ...
                        $allowableColumn = $this->getJobsColumnName();
                        $request->column_name = 'job_tracking.' . $request->column_name;
                        // echo "<pre>"; print_r($request->column_name); die;
                        if (isset($request->column_name) && !empty($request->column_name) && $request->column_name != "" && in_array($request->column_name, $allowableColumn)) {
                            $q->where($request->column_name, 'like', '%' . $request->column_value . '%');
                        } else {
                            $request->column_name = 'booking.' . $request->column_name;
                            if (isset($request->column_name) && !empty($request->column_name) && $request->column_name != "" && in_array($request->column_name, $allowableColumn)) {
                                $q->where($request->column_name, 'like', '%' . $request->column_value . '%');
                            }
                        }
                    })
                    ->orderBy($order_by_column_name, $order)
                    ->paginate($limit);

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => "List of jobs belongs to customer.", 'records' => $jobs]);
        } 
        catch (Exception $e) {
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } 
        catch (Throwable $e) {
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    
    private function getColumnName()
    {
        $address = $this->users::first();
        if (!empty($address) && $address != "") {
            return array_keys($address->toArray());
        }

        return ['id'];
    }

    
    private function getJobsColumnName()
    {
        $col_name = [];
        $col = $this->jobs::first();
        if (!empty($col) && $col != "") {
            // return array_keys($col->toArray());
            foreach (array_keys($col->toArray()) as $key => $value) {
                $col_name[] = 'job_tracking.' . $value;
            }
            $extraCol = ['booking.id', 'booking.user_id', 'booking.contractor_id', 'service-category.sub_category_name', 'booking.booking_date', 'booking.booking_time'];
            return array_merge($col_name, $extraCol);
        }

        return ['id'];
    }

}
