<?php

namespace App\Http\Controllers\v3\ReviewAndRating;
use Exception;
use Throwable;
use Carbon\Carbon;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;
use App\Models\Customers\ProviderRatingReview;
use App\Models\Users\User;
use Illuminate\Support\Facades\App;

class ReviewAndRatingController extends Controller 
{
    /**
    * The user repository implementation.
    *
    * @var UserRepository
    */

    protected $providerRatingReview;
    protected $user_id;
    protected $role_fk;


    public function __construct(ProviderRatingReview $providerRatingReview, Request $request)
    {
        $this->providerRatingReview = $providerRatingReview;
        $this->user_id = $request->user_id;
        $this->role_fk = $request->role_fk;
        $this->user_fk = $request->user_fk;
        //has_scope('customers.read');
    }


    public function index(Request $request)
    {
        try {
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;
            if (isset($request->order) && strtolower($request->order) == "asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if ($request->filled('order_by_column_name')) {
                if (in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }
            $query = $this->providerRatingReview
                     ->where(function ($qry) use ($request) {
                             $qry->whereHas('provider', function ($qry) use ($request) {
                                 if($request->filled('search')) {
                                     $pieces = explode(" ", $request->search);
                                     if(count($pieces)){
                                         $fName = !empty($pieces[0]) ? $pieces[0] : $request->search; 
                                         $lName = !empty($pieces[1]) ? $pieces[1] : $request->search; 
                                     }
                                     $qry->where("first_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                                     $qry->orWhere("last_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                                     $qry->orWhere(DB::raw('CONCAT(first_name," ",last_name)'), 'like', '%' .encryptString(ucfirst(strtolower($fName))).' '.encryptString(ucfirst(strtolower($lName))).'%');
                                 }
                             });
                         })
                         ->whereHas("provider", function($q) use ($request) {
                             if($request->filled('provider_name')) {
                                 $pieces = explode(" ", $request->provider_name);
                                 if(count($pieces)){
                                     $fName = !empty($pieces[0]) ? $pieces[0] : $request->provider_name; 
                                     $lName = !empty($pieces[1]) ? $pieces[1] : $request->provider_name; 
                                 }
                                 $q->where("first_name","like","%".encryptString(ucfirst(strtolower($request->provider_name)))."%");
                                 $q->orWhere("last_name","like","%".encryptString(ucfirst(strtolower($request->provider_name)))."%");
                                 $q->orWhere(DB::raw('CONCAT(first_name," ",last_name)'), 'like', '%' .encryptString(ucfirst(strtolower($fName))).' '.encryptString(ucfirst(strtolower($lName))).'%');
                             }
                         })
                        ->with(['provider'])
                        ->orderBy($order_by_column_name, $order)
                        ->paginate($limit);

            if (empty($query)) {
                throw new Exception(config('service.USER_API.MESSAGE.REVIEW_RATING_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.USER_API.MESSAGE.RATING_REVIEW_LISTING'), 'records' => $query]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    private function getColumnName()
    {
        $address = ProviderRatingReview::first();
        if (!empty($address) && $address != "") {
            return array_keys($address->toArray());
        }
        return ['id'];
    }


    public function activeInactive(Request $request, $id) 
    {
        //has_scope('cms.pages.write');
        $this->validate($request,[
            'status' => 'required',
        ]);
        try {
            $save_data['published'] = $request->status;
            $result = ProviderRatingReview::where('id', $id)->update($save_data);

            if($request->status=="active")
            {
                return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.RATING_REVIEW_STATUS_ACTIVE'),'action'=>$request->status]);
            }
            else
            {
                return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.RATING_REVIEW_STATUS_INACTIVE'),'action'=>$request->status]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
    }


    public function deleteReviewRating(Request $request, $id) 
    {
        //has_scope('roles.delete');
        $ratingReview = ProviderRatingReview::findOrFail($id);
        try {
            $ratingReview  = $ratingReview->delete();
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.REVIEW_RATING_DELETED'), 'result' => $ratingReview]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }


    public function show(Request $request,$id) 
    {
        try
        {
            $ratingReview =  ProviderRatingReview::with(['provider'])->find($id);
            if(empty($ratingReview)) {
                throw new Exception(config('service.USER_API.MESSAGE.REVIEW_RATING_NOT_FOUND'));
            }
            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.REVIEW_RATING_DETAILS'), 'records' => $ratingReview]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
}