<?php

namespace App\Http\Controllers\v3\SubscriptionPlan;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\PurchagePlan;

class PurchagePlanController extends Controller
{
    /**
     * The user repository implementation.
     *
     * @var UserRepository
     */
    protected $purchage_plan;
    protected $user_id;
    protected $role_fk;

    public function __construct(PurchagePlan $purchage_plan, Request $request) {
        $this->purchage_plan = $purchage_plan;
        $this->user_id = $request->user_id;
        $this->role_fk = $request->role_fk;
    }

    /**
     * @ SubscriptionPlan list List api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function index(Request $request) {
        try
        {
            $order = "desc";
            $limit = isset($request->limit) && $request->limit > 500 ? 25 : $request->limit;
            if(isset($request->order) && strtolower($request->order)=="asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if(isset($request->order_by_column_name)) {
                if(in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }

            $subs_plan = $this->purchage_plan::select(['subscription_plan.*', 'create_subscription_plan.plane_name'])
                        ->join('create_subscription_plan', 'create_subscription_plan.id', '=', 'subscription_plan.plan_id')
                        ->where('contractor_id', '=', $this->user_id)
                        ->where(function($q) use($request) {
                            // Global searching...
                            if(isset($request->search) && !empty($request->search) && $request->search!="") {
                                if(!empty($this->getColumnName())) {
                                    foreach ($this->getColumnName() as $key => $col) {
                                        if($key==0) {
                                            $q->where($col, 'like', '%' .encryptString($request->search). '%');
                                        } else {
                                            $q->orWhere($col, 'like', '%' .encryptString($request->search). '%');
                                        }
                                    }
                                }
                            }
                        })
                        ->where(function($q) use($request) {
                            // Column wise searching filter ...
                            $allowableColumn = $this->getColumnName();
                            if(isset($request->column_name) && !empty($request->column_name) && $request->column_name!="" && in_array($request->column_name, $allowableColumn)) {
                                $q->where($request->column_name,  'like', '%' .$request->column_value. '%');
                            }
                        })
                        ->orderBy($order_by_column_name, $order)
                        ->paginate($limit);
            if( empty($subs_plan) ) {
                throw new Exception(config('service.CONTRACTOR_API.MESSAGE.PURCHAGE_PLAN_NOT_FOUND'));
            }

            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.PURCHAGE_PLAN_LISTING'), 'records' => $subs_plan]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * @ SubscriptionPlan List api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function show($id) {
        try {
            $purchage_plan = $this->purchage_plan::with(['contractor.address', 'membership_plan.subscription_services'])->find($id);
            if( empty($purchage_plan) ) {
                throw new Exception(config('service.CONTRACTOR_API.MESSAGE.PURCHAGE_PLAN_NOT_FOUND'));
            }

            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.PURCHAGE_PLAN_DETAILS'), 'records' => $purchage_plan]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    /**
     * @ Take SubscriptionPlan api.
     * @return \Illuminate\Http\Response
    */
    public function store(Request $request) {
        $message = array(
            'plan_id' => config('service.CONTRACTOR_API.MESSAGE.PURCHAGE_PLAN')
        );
        $this->validate($request, [
            'plan_id' => 'required|numeric',
            'order_id' => 'required|max:256',
            'product_id' => 'required|numeric',
            'start_date' => 'required',
            'end_date' => 'required',
            'cancelled_date' => isset($request->cancelled_date) && !empty($request->cancelled_date) ? 'required' : "",
            'status' => 'required|in:0,1'
        ], $message);

        try
        {
            if(strtolower($this->role_fk)=='admin' || strtolower($this->role_fk)=='customer') {
                throw new Exception("Sorry you have no access to perform this action. Please login as a contractor for this.");
            }

            $input = $request->all();
            $input['contractor_id'] = $request->user_id;
            $input['start_date'] = date('Y-m-d H:i:s', strtotime($request->start_date));
            $input['end_date'] = date('Y-m-d H:i:s', strtotime($request->end_date));
            $input['cancelled_date'] = isset($request->cancelled_date) && !empty($request->cancelled_date) ? date('Y-m-d H:i:s', strtotime($request->cancelled_date)) : NULL;
            $subs_plan = $this->purchage_plan->create($input);

            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.PURCHAGE_PLAN_STORE'), 'records' => $subs_plan]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            // This should work as well
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * @ update Contractor api .
     * @return \Illuminate\Http\Response
    */
    public function update(Request $request, $id) {
        // die('i have implemented this but ....');
        $message = array(
            'plan_id' => config('service.CONTRACTOR_API.MESSAGE.PURCHAGE_PLAN')
        );
        $this->validate($request, [
            'plan_id' => 'required|numeric',
            'order_id' => 'required|max:256',
            'product_id' => 'required|numeric',
            'start_date' => 'required',
            'end_date' => 'required',
            'cancelled_date' => isset($request->cancelled_date) && !empty($request->cancelled_date) ? 'required' : "",
            'status' => 'required|in:0,1'
        ], $message);

        try
        {
            if(strtolower($this->role_fk)=='admin' || strtolower($this->role_fk)=='customer') {
                throw new Exception("Sorry you have no access to perform this action. Please login as a contractor for this.");
            }

            $subs_plan = $this->purchage_plan::find($id);
            $input = $request->all();
            $input['start_date'] = date('Y-m-d H:i:s', strtotime($request->start_date));
            $input['end_date'] = date('Y-m-d H:i:s', strtotime($request->end_date));
            $input['cancelled_date'] = isset($request->cancelled_date) && !empty($request->cancelled_date) ? date('Y-m-d H:i:s', strtotime($request->cancelled_date)) : NULL;

            $subs_plan->update($input);

            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.PURCHAGE_PLAN_UPDATE'), 'records' => $subs_plan]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            // This should work as well
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function delete(Request $request, $id)
    {
        try {
            $update['status'] = 0;
            $subs_plan = $this->purchage_plan::find($id)->update($update);

            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.PURCHAGE_PLAN_DELETED'), 'records' => $subs_plan]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    private function getColumnName() {
        $col_name = [];
        $col = $this->purchage_plan::first();
        if(!empty($col) && $col!="") {
            // return array_keys($col->toArray());
            foreach (array_keys($col->toArray()) as $key => $value) {
                $col_name[] = 'subscription_plan.'.$value;
            }
            $extraCol = ['create_subscription_plan.plane_name'];
            return array_merge($col_name, $extraCol);
        }

        return ['id'];
    }
}
