<?php

namespace App\Http\Controllers\v4\AppLabels;

use Exception;
use Throwable;
use Carbon\Carbon;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;
use App\Models\Customers\Customer;
use App\Models\Customers\Oauth;
use App\Models\Users\User;



class AppLabelController extends Controller {

    /**
     * The user repository implementation.
     *
     * @var UserRepository
     */
    protected $users;
    protected $oauth_user;
    protected $user_id;
    protected $role_fk;

    public function __construct(Customer $users, Oauth $oauth_user, Request $request)
    {

        $this->users = $users;
        $this->oauth_user = $oauth_user;
        $this->user_id = $request->user_id;
        $this->role_fk = $request->role_fk;
        $this->user_fk = $request->user_fk;

        //has_scope('customers.read');
    }

    /**
     * @ service List api using pagination.
     * @return \Illuminate\Http\Response
     */
    public function showLabels() 
    {
        try {
            
            $labels = DB::table('labels')->get();

            $data = [];
            foreach ($labels as $key => $value) 
            {
                $data[$value->label_name] = [
                    'en' => $value->en_value,
                    'es' => $value->es_value,
                ];
            }

            if (empty($labels)) {
                throw new Exception(config('service.LABEL_NOT_FOUND'));
            }

            return response()->json(['status' => config('service.SUCCESS_STATUS'), 'message' => config('service.LABEL_LIST'), 'records' => $data]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function languages() 
    {
        try {
            
            $languages = DB::table('languages')->select('name','code')->get();

            if (empty($languages)) {
                throw new Exception(config('service.LANGUAGE_NOT_FOUND'));
            }

            return response()->json(['status' => config('service.SUCCESS_STATUS'), 'message' => config('service.LANGUAGE_LIST'), 'records' => $languages]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

   
}
