<?php

namespace App\Http\Controllers\v4\ChangeProviderRequest;
use App\Http\Controllers\Customers\CustomerNotificationController;
use Carbon\Carbon;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;
use App\Models\ChangeProvider\ChangeProvider;
use Illuminate\Support\Facades\App;
use App\Models\Customers\Customer;
use App\Models\Customers\Appointment;
use App\Models\EmailTemplate\EmailTemplate;
use App\Jobs\CustomerCommonMailJob;
use Throwable;
use Exception;

class ChangeProviderRequestController extends Controller 
{
    /**
    * The user repository implementation.
    *
    * @var UserRepository
    */

    protected $changeProvider;
    protected $user_id;
    protected $role_fk;


    public function __construct(ChangeProvider $changeProvider, Request $request)
    {
        $this->changeProvider = $changeProvider;
        $this->user_id = $request->user_id;
        $this->role_fk = $request->role_fk;
        $this->user_fk = $request->user_fk;
    }


    public function index(Request $request)
    {
        try {
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;
            if (isset($request->order) && strtolower($request->order) == "asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if ($request->filled('order_by_column_name')) {
                if (in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }
            $query = $this->changeProvider
                     ->where(function ($qry) use ($request) {
                            $qry->whereHas('provider', function ($qry) use ($request) {
                                if($request->filled('search')) {
                                    $pieces = explode(" ", $request->search);
                                    if(count($pieces)){
                                        $fName = !empty($pieces[0]) ? $pieces[0] : $request->search; 
                                        $lName = !empty($pieces[1]) ? $pieces[1] : $request->search; 
                                    }
                                    $qry->where("first_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                                    $qry->orWhere("last_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                                    $qry->orWhere(DB::raw('CONCAT(first_name," ",last_name)'), 'like','%'.encryptString(ucfirst(strtolower($fName))).' '.encryptString(ucfirst(strtolower($lName))).'%');
                                }
                            })->orwhereHas('customer', function ($qry) use ($request) {
                                if($request->filled('search')) {
                                    $pieces = explode(" ", $request->search);
                                    if(count($pieces)){
                                        $fName = !empty($pieces[0]) ? $pieces[0] : $request->search; 
                                        $lName = !empty($pieces[1]) ? $pieces[1] : $request->search; 
                                    }
                                    $qry->where("first_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                                    $qry->orWhere("last_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                                    $qry->orWhere(DB::raw('CONCAT(first_name," ",last_name)'),'like','%'.encryptString(ucfirst(strtolower($fName))).' '.encryptString(ucfirst(strtolower($lName))).'%');
                                }
                            });
                        })
                    ->with(['customer','provider'])
                    ->orderBy($order_by_column_name, $order)
                    ->paginate($limit);

            if (empty($query)) {
                throw new Exception(config('service.USER_API.MESSAGE.CHANGE_PROVIDER_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.USER_API.MESSAGE.CHANGE_PROVIDER_LISTING'), 'records' => $query]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    private function getColumnName()
    {
        $address = ChangeProvider::first();
        if (!empty($address) && $address != "") {
            return array_keys($address->toArray());
        }
        return ['id'];
    }


    public function approvedChangeProviderRequest($id) 
    {
        try {
            
            $changeProvider = ChangeProvider::where(['id' => $id, 'status' => 'pending'])->first();
            if($changeProvider){
                $changeProvider->status = "approved";
                $changeProvider->save();

                $customer = Customer::where(['id' => $changeProvider->customer_id])->first();
                $customer->current_provider_id = null;
                $customer->save();

                $moengageData =  [
                    "type" => "event",
                    "customer_id" => decryptString($customer->email),
                    "actions" => [
                        [
                            "action" => "change_provider_accepted",
                            "attributes" => [
                                "change_provider_accepted" => "True",
                                "consumer_email" => decryptString($customer->email)
                            ],
                            "platform" => "web"
                        ]
                    ]
                ];
    
                $moResult = moengageCallCustomEventApiRequest($moengageData);

                $currentDateTimeUtc = strtotime(date('Y-m-d H:i'));
                $appointment = Appointment::where(['customer_id' => $changeProvider->customer_id,'provider_id' => $changeProvider->provider_id])
                //->where(DB::raw('CONCAT(date," ",time)'), '>=',  date("Y-m-d")." ".date("H:i"))
                ->where('date_time', '>=', $currentDateTimeUtc)
                ->first();
                if($appointment){
                    $appointment->delete();
                }

                
                $templateName = "Provider changed";
                $templateData = EmailTemplate::where('name',$templateName)->first();
                if($templateData)
                {
                    /* Changed provider email */
                        $greeting = 'Hi ' . decryptString($customer->first_name)." ".decryptString($customer->last_name) . ',';
                        $body     = $templateData->text;
                        $type     = $templateData->title;
                        $gratitude     = config('constant.gratitude_english');
                        $regard     = config('constant.regard_english');
                        $footer     = config('constant.footer_english');
                        // $sendMail = new CustomerCommonMailJob(decryptString($customer->email), $greeting, $body, $type, $gratitude, $regard, $footer);
                        // $this->dispatch($sendMail);
                    /* End change provider email  */
                }

                /* Push and db notification */
                    (new CustomerNotificationController)->customerChangeProviderRequestApproved($customer->id);
                /* End push and db notification */


                return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.PROVIDER_CHANGE_REQUEST_APPROVE')]);  
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


}