<?php
namespace App\Http\Controllers\v4\Faq;
use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\App;
use App\Notifications\CreateNewUserNotification;
use App\Models\Faq\Faq;

class FaqController extends Controller 
{
    /**
    * The user repository implementation.
    *
    * @var UserRepository
    */
    protected $faq;
    public function __construct(Faq $faq)
    {
        $this->faq = $faq;
    //has_scope('faq.read');
    }

    /**
    * @ user list List api using pagination.
    * @return \Illuminate\Http\Response
    */
    public function index($locale, Request $request)
    {
        try {
            
            if($locale =="en")
            {
                App::setLocale('en');
            }
            else
            {
                App::setLocale('es');
            }

            if(!empty($request->user_type) && $request->user_type == "provider"){
                $result =  $this->faq::where('status', 'active')
                                ->WhereTranslation('user_type', 'provider')    
                                ->get();
            }else if(!empty($request->user_type) && $request->user_type == "consumer"){
                $result =  $this->faq::where('status', 'active')
                                ->WhereTranslation('user_type', 'consumer')    
                                ->get();
            }else{
                $result =  $this->faq::where('status', 'active')
                                ->WhereTranslation('user_type', 'consumer')    
                                ->get();
            }
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'Faq List.', 'records' => $result]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
    * @ Register user api.
    * @return \Illuminate\Http\Response
    */
    public function store(Request $request)
    {
        $this->validate($request, [
            'en_question' => 'required|max:190',
            'en_answer' => 'required|max:1000',
            'es_question' => 'required|max:190',
            'es_answer' => 'required|max:1000',
        ], 
        [
            'en_question.required' => 'The EN question field is required.',
            'en_answer.required' => 'The EN answer field is required.',
            'es_question.required' => 'The ES question field is required.',
            'es_answer.required' => 'The ES answer field is required.'
        ]
    );

        
        try {

            $input = $request->all();
            $user_type = 'consumer';
            if(!empty($input['user_type'])){
                $user_type = $input['user_type'];
            }

            $data = [
                'en' => [
                    'question'  => $input['en_question'],
                    'answer'   => $input['en_answer'],
                    'user_type'    =>  $user_type
                ],
                'es' => [
                    'question'  => $input['es_question'],
                    'answer'   => $input['es_answer'],
                    'user_type'    =>  $user_type
                ],
            ];
            $result  = Faq::create($data);

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.FAQ_ADDED'), 'records' => $result]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }
    public function edit($id)
    {
        try {

            $faq = Faq::find($id);
            if( empty($faq) ) {
                throw new Exception(config('service.USER_API.MESSAGE.FAQ_NOT_FOUND'));
            }
            $faqEn = [
                'id' => $faq->id,
                'question' => $faq->question,
                'answer' => $faq->answer,
            ];
            app()->setLocale('es');
            $faqEs =  Faq::find($id);
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'FAQ Details', 'recordEn' => $faqEn,'recordEs' => $faqEs]);
        }
        catch (Exception $e) {

            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {

            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    /**
    * @ Service List api using pagination.
    * @return \Illuminate\Http\Response
    */
    public function show($id)
    {
        try {

            $users = DB::table('roles')->where('id', $id)->first();
            $users->scope = isset($users->scope) ? explode(' ', $users->scope) : [];
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'Roles Details', 'records' => $users]);
        }
        catch (Exception $e) {

            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {

            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
    * @ update user api .
    * @return \Illuminate\Http\Response
    */
    public function update(Request $request, $id) 
    {
        $this->validate($request, [
            'en_question' => 'required|max:190',
            'en_answer' => 'required|max:1000',
            'es_question' => 'required|max:190',
            'es_answer' => 'required|max:1000',
        ]);
        try {
                $faq = Faq::findOrFail($id);
                $input = $request->all();
                $data = [
                    'en' => [
                        'question'  => $input['en_question'],
                        'answer'   => $input['en_answer']
                    ],
                    'es' => [
                        'question'  => $input['es_question'],
                        'answer'   => $input['es_answer']
                    ],
                ];
                $data  = $faq->update($data);
                //admin mail
                /*$greetingAdmin = 'Dear Admin';
                $bodyAdmin     = 'The page information is updated successfully.';
                $typeAdmin     = 'CMS Page Updated';

                $adminMail = new SendMailJob(env('ADMIN_EMAIL'), $greetingAdmin, $bodyAdmin, $typeAdmin);
                dispatch($adminMail);*/
                return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.FAQ_UPDATED'), 'records' => $data]);
            } catch (Exception $e) {
                return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
            } catch(Throwable $e) {
                return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
            }
    }
    public function activeInactive(Request $request, $id) 
    {
        $this->validate($request,[
            'status' => 'required',
        ]);
        try {
            $save_data['status'] = $request->status;
            $result = Faq::where('id', $id)->update($save_data);

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.FAQ_STATUS')]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
    }
    /**
    * Remove the specified resource from storage.
    *
    * @param  int  $id
    * @return \Illuminate\Http\Response
    */
    public function delete(Request $request, $id)
    {
        try {
            $result = Faq::where('id', $id)->delete();
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'FAQ deleted', 'result' => $result]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }
    public function faqs(Request $request) 
    {
        try
        {
            if(!empty($request->user_type) && $request->user_type == "provider"){
                $faqs =  Faq::where('status', 'active')
                                ->WhereTranslation('user_type', 'provider')    
                                ->get();
            }else if(!empty($request->user_type) && $request->user_type == "consumer"){
                $faqs =  Faq::where('status', 'active')
                                ->WhereTranslation('user_type', 'consumer')    
                                ->get();
            }else{
                $faqs =  Faq::where('status', 'active')
                                ->WhereTranslation('user_type', 'consumer')    
                                ->get();
            }

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.FAQ'), 'records' => $faqs]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

}