<?php

namespace App\Http\Controllers\v4\Providers;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;
use App\Models\Providers\Provider;
use App\Models\Providers\Language;
use App\Models\Customers\Customer;
use App\Notifications\Providers\Notification;
use App\Notifications\Customers\Notification as CustomerNotification;
use Validator;

class ProviderNotificationController extends Controller
{
    

    public function register($provider_id)
    {
        try
        {
            $provider = Provider::findOrFail($provider_id);
        
            /*  push notification   */
            $deviceToken = $provider->device_token;
            $title = "Sign Up";
            $message = "Hi ".decryptString($provider->first_name)." ".decryptString($provider->last_name).", Welcome to PM Now.";
            sendPushNotification($deviceToken,$title,$message);
            /* end push notification   */

            /*  database notification  */
            $details = [
                'title'    => $title,
                'message'    => $message,
            ];
            $provider->notify(new Notification($details));
            /*  end database */

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function providerConfirmEmail($provider_id)
    {
        try
        {
            $provider = Provider::findOrFail($provider_id);
        
            /*  push notification   */
            $deviceToken = $provider->device_token;
            $title = "Account Verified";
            $message = "Your account has been verified.";
            sendPushNotification($deviceToken,$title,$message);
            /* end push notification   */

            /*  database notification  */
            $details = [
                'title'    => $title,
                'message'    => $message,
            ];
            $provider->notify(new Notification($details));
            /*  end database */

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function approveProvider($provider_id)
    {
        try
        {
            $provider = Provider::findOrFail($provider_id);
        
            /*  push notification   */
            $deviceToken = $provider->device_token;
            $title = "Account Approved";
            $message = "Congratulations! Your profile has been verified by Admin. Please set your availability now.";
            sendPushNotification($deviceToken,$title,$message);
            /* end push notification   */

            /*  database notification  */
            $details = [
                'title'    => $title,
                'message'    => $message,
            ];
            $provider->notify(new Notification($details));
            /*  end database */

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function changePassword($provider_id)
    {
        try
        {
            $provider = Provider::findOrFail($provider_id);
        
            /*  push notification   */
            $deviceToken = $provider->device_token;
            $title = "Password Reset";
            $message = "Your password has been changed successfully.";
            sendPushNotification($deviceToken,$title,$message);
            /* end push notification   */

            /*  database notification  */
            $details = [
                'title'    => $title,
                'message'    => $message,
            ];
            $provider->notify(new Notification($details));
            /*  end database */

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function forgotPassword($provider_id)
    {
        try
        {
            $provider = Provider::findOrFail($provider_id);
        
            /*  push notification   */
            $deviceToken = $provider->device_token;
            $title = "Forget Password";
            $message = "Your password has been recently changed using forget password.";
            sendPushNotification($deviceToken,$title,$message);
            /* end push notification   */

            /*  database notification  */
            $details = [
                'title'    => $title,
                'message'    => $message,
            ];
            $provider->notify(new Notification($details));
            /*  end database */

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function profileUpdate($provider_id)
    {
        try
        {
            $provider = Provider::findOrFail($provider_id);
        
            /*  push notification   */
            $deviceToken = $provider->device_token;
            $title = "Profile Updated";
            $message = "Profile has been updated by you successfully.";
            sendPushNotification($deviceToken,$title,$message);
            /* end push notification   */

            /*  database notification  */
            $details = [
                'title'    => $title,
                'message'    => $message,
            ];
            $provider->notify(new Notification($details));
            /*  end database */

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function providerAvailabilityCreate($provider_id)
    {
        try
        {
            $provider = Provider::findOrFail($provider_id);
        
            /*  push notification   */
            $deviceToken = $provider->device_token;
            $title = "Availability Set";
            $message = "Availability is set successfully.";
            sendPushNotification($deviceToken,$title,$message);
            /* end push notification   */

            /*  database notification  */
            $details = [
                'title'    => $title,
                'message'    => $message,
            ];
            $provider->notify(new Notification($details));
            /*  end database */

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function providerManageTimeOffCreate($provider_id)
    {
        try
        {
            $provider = Provider::findOrFail($provider_id);
        
            /*  push notification   */
            $deviceToken = $provider->device_token;
            $title = "Day Offs Added";
            $message = "Your day offs have been added.";
            sendPushNotification($deviceToken,$title,$message);
            /* end push notification   */

            /*  database notification  */
            $details = [
                'title'    => $title,
                'message'    => $message,
            ];
            $provider->notify(new Notification($details));
            /*  end database */

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function providerAddAppointmentNote($customer_id)
    {
        try
        {
            $customer = Customer::findOrFail($customer_id);
            $language = Language::findOrFail($customer->language_id);
            if($language->code == "es"){
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Notas de sesión publicadas";
                $message = "Su proveedor ha agregado sus notas de sesión.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */
            }else{
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Session Notes Posted";
                $message = "Your provider has added your session notes.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */
            }

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function providerSendMessage($customer_id)
    {
        try
        {
            $customer = Customer::findOrFail($customer_id);
            $language = Language::findOrFail($customer->language_id);
            if($language->code == "es"){
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Nuevo mensaje";
                $message = "Tienes un nuevo mensaje.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */
            }else{
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "New Message";
                $message = "You have got a new message.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */
            }

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function providerJoinSession($customer_id)
    {
        try
        {
            $customer = Customer::findOrFail($customer_id);
            $language = Language::findOrFail($customer->language_id);
            if($language->code == "es"){
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Llamada unida";
                $message = "El proveedor se ha unido y esperará 10 minutos.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */
            }else{
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Call Joined";
                $message = "Provider has joined and will wait for 10 minutes.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */
            }
            
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function providerJoinSessionAgain($customer_id)
    {
        try
        {
            $customer = Customer::findOrFail($customer_id);
            $language = Language::findOrFail($customer->language_id);
            if($language->code == "es"){
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Llamada unida";
                $message = "El proveedor se ha unido y te está esperando..";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */
            }else{
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Call Joined";
                $message = "Provider has joined and waiting for you.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */
            }
            
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function customerDidNotJoinSession($customer_id)
    {
        try
        {
            $customer = Customer::findOrFail($customer_id);
            $language = Language::findOrFail($customer->language_id);
            if($language->code == "es"){
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Cita";
                $message = "Parece que no te uniste a la sesión. Habrá una deducción de tu sesión.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */
            }else{
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Appointment";
                $message = "It seems you did not join the session, There will be a deduction of your session.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */
            }
            
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function providerSessionCompleted($provider_id)
    {
        try
        {
            $provider = Provider::findOrFail($provider_id);
        
            /*  push notification   */
            $deviceToken = $provider->device_token;
            $title = "Session Completed";
            $message = "You have taken your session. Post notes now.";
            sendPushNotification($deviceToken,$title,$message);
            /* end push notification   */

            /*  database notification  */
            $details = [
                'title'    => $title,
                'message'    => $message,
            ];
            $provider->notify(new Notification($details));
            /*  end database */

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }



}
