<?php

namespace App\Http\Controllers\v4\RateTheApp;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

use App\Models\AppRating;

class RateAppController extends Controller
{
    /**
     * The user repository implementation.
     *
     * @var UserRepository
     */
    protected  $app_rating;
    protected  $user_id;
    protected  $role_fk;

    public function __construct(AppRating $app_rating, Request $request) {
        $this->app_rating = $app_rating;
        $this->user_id = $request->user_id;
        $this->role_fk = $request->role_fk;
    }

    public function index(Request $request) {
        try
        {
            $order = "desc";
            $limit = isset($request->limit) && $request->limit > 500 ? 25 : $request->limit;
            if(isset($request->order) && strtolower($request->order)=="asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if(isset($request->order_by_column_name)) {
                if(in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }

            $app_rating = $this->app_rating::where(function($q) use($request) {
                // Global searching...
                if(isset($request->search) && !empty($request->search) && $request->search!="") {
                    if(!empty($this->getColumnName())) {
                        foreach ($this->getColumnName() as $key => $col) {
                            if($key==0) {
                                $q->where($col, 'like', '%' .$request->search. '%');
                            } else {
                                $q->orWhere($col, 'like', '%' .$request->search. '%');
                            }
                        }
                    }
                }
            })
            ->where(function($q) use($request) {
                // Column wise searching filter ...
                $allowableColumn = $this->getColumnName();
                if(isset($request->column_name) && !empty($request->column_name) && $request->column_name!="" && in_array($request->column_name, $allowableColumn)) {
                    $q->where($request->column_name,  'like', '%' .$request->column_value. '%');
                }
            })
            ->orderBy($order_by_column_name, $order)
            ->paginate($limit);

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.RATE_APP_LISTING'), 'records' => $app_rating]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function store(Request $request)
    {
        $this->validate($request, [
            'rating' => 'required|in:1,2,3,4,5',
            'review' => 'required|max:256',
        ]);

        try {
            $input = $request->all();
            if(strtolower($this->role_fk)=='customer') {
                $input['user_id'] = $this->user_id;
                $check = $this->app_rating::where('user_id', '=', $input['user_id'])->exists();
                if($check) {
                    throw new Exception(config('service.USER_API.MESSAGE.REVIEW_RATING_ALREADY_EXISTS'), 1);
                }
            } else {
                $input['contractor_id'] = $this->user_id;
                $check = $this->app_rating::where('contractor_id', '=', $input['contractor_id'])->exists();
                if($check) {
                    throw new Exception(config('service.USER_API.MESSAGE.REVIEW_RATING_ALREADY_EXISTS'), 1);
                }
            }

            $app_rating = $this->app_rating::create($input);
            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.REVIEW_RATING_STORED'), 'records' => $app_rating]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'rating' => 'required|in:1,2,3,4,5',
            'review' => 'required|max:256',
        ]);

        try {
            $input = $request->all();

            $app_rating = $this->app_rating::find($id);
            $app_rating->update($input);
            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.REVIEW_RATING_STORED'), 'records' => $app_rating]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function delete($id)
    {
        try {
            $app_rating = $this->app_rating::find($id)->delete();

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.APP_RATING_DELETED'), 'records' => $app_rating]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    private function getColumnName() {
        $col = $this->app_rating::first();
        if(!empty($col) && $col!="") {
            return array_keys($col->toArray());
        }

        return ['id'];
    }
}
