<?php

namespace App\Http\Controllers\v4\SubscriptionPlan;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;

use App\Models\MembershipPlan;

class SubscriptionPlanController extends Controller
{
    /**
     * The user repository implementation.
     *
     * @var UserRepository
     */
    protected $subscription_plan;

    public function __construct(MembershipPlan $subscription_plan) {
        $this->subscription_plan = $subscription_plan;
    }

    /**
     * @ SubscriptionPlan list List api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function index(Request $request) {
        try
        {
            $order = "desc";
            $limit = isset($request->limit) && $request->limit > 500 ? 25 : $request->limit;
            if(isset($request->order) && strtolower($request->order)=="asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if(isset($request->order_by_column_name)) {
                if(in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }

            $subs_plan = $this->subscription_plan::where('status', '=', 1)
                        ->where(function($q) use($request) {
                            // Global searching...
                            if(isset($request->search) && !empty($request->search) && $request->search!="") {
                                if(!empty($this->getColumnName())) {
                                    foreach ($this->getColumnName() as $key => $col) {
                                        if($key==0) {
                                            $q->where($col, 'like', '%' .$request->search. '%');
                                        } else {
                                            $q->orWhere($col, 'like', '%' .$request->search. '%');
                                        }
                                    }
                                }
                            }
                        })
                        ->where(function($q) use($request) {
                            // Column wise searching filter ...
                            $allowableColumn = $this->getColumnName();
                            if(isset($request->column_name) && !empty($request->column_name) && $request->column_name!="" && in_array($request->column_name, $allowableColumn)) {
                                $q->where($request->column_name,  'like', '%' .$request->column_value. '%');
                            }
                        })
                        ->orderBy($order_by_column_name, $order)
                        ->paginate($limit);
            if( empty($subs_plan) ) {
                throw new Exception(config('service.CONTRACTOR_API.MESSAGE.SUBCRIPTION_PLAN_NOT_FOUND'));
            }

            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.SUBCRIPTION_PLAN_LISTING'), 'records' => $subs_plan]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * @ SubscriptionPlan List api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function show($id) {
        try {
            $subscription_plan = $this->subscription_plan::with(['subscription_services'])->find($id);
            if( empty($subscription_plan) ) {
                throw new Exception(config('service.CONTRACTOR_API.MESSAGE.SUBCRIPTION_PLAN_NOT_FOUND'));
            }

            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.SUBCRIPTION_PLAN_DETAILS'), 'records' => $subscription_plan]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    private function getColumnName() {
        $subs_plan = $this->subscription_plan::first();
        if(!empty($subs_plan) && $subs_plan!="") {
            return array_keys($subs_plan->toArray());
        }

        return ['id'];
    }
}
