<?php
namespace App\Http\Controllers\v4\Users;

use Exception;
use Throwable;

use Illuminate\Support\Str;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Mail;

use App\Models\Users\User;
use App\Models\Users\Oauth;

use App\Notifications\Users\Notification;

use App\Jobs\SendMailJob;
use App\Jobs\SendForgotMailJob;

use Carbon\Carbon;

class ForgotPasswordController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Password Reset Controller
    |--------------------------------------------------------------------------
    |
    | This controller is responsible for handling password reset emails and
    | includes a trait which assists in sending these notifications from
    | your application to your users. Feel free to explore this trait.
    |
    */

    /**
     * Send a reset link to the given user.
     *
     * @param Request $request
     * @return Response
     */
    public function sendResetPasswordLink(Request $request)
    {  
        $message = array(
            'email.exists'   => config('service.USER_API.MESSAGE.NOTEXSIST_EMAIL'),
            'email.email'   => config('service.USER_API.MESSAGE.VALID_EMAIL')
        );

        $this->validate($request, 
            ['email' => 'required|email|exists:admins', 
        ],$message);

        try
        {
            // create token
            $minutes = 20;
            $token   = Str::random(60);
            $upd['token']  = $token;
            $upd['token_expire'] =  Carbon::now()->addMinutes($minutes)->toDateTimeString();
            DB::table('admins')->where(['email' => $request->email])->update($upd);

            $user = DB::table('admins')->select('first_name')->where('email', $request->email)->first();

            $data['name'] = $user->first_name;
            $data['link_expire_time'] = $minutes;

             // Genereate link for reset password

            //$data['link'] = url('/') . '/' . 'api/v2/users/password_update' . '/' . $token;
            if($request->filled('type')) {
                $data['link'] = env('FRONT_END') . '/' . 'admin/user/reset_password' . '/' . $token;
            }else {
                $data['link'] = env('FRONT_END') . '/' . 'admin/change_password' . '/' . $token;
            }
          
            
            
            // Trigger mail
            /*$greeting = 'Hi ' . $user->first_name . '!';
            $body     = 'You recently requested to reset your password. Press this button and you are on your way to a new password.';
            $type     = 'Reset Password';
            $link     = $data['link'];
            $link_expire_time    = $data['link_expire_time'];
            
            $sendMail = new SendForgotMailJob($request->email, $greeting, $body, $type, $link, $link_expire_time);
            dispatch($sendMail);*/
            Mail::send('emails.Users.reset_password',['data'=>$data], function($message) use ($request) {
                $message->to($request->email);
                $message->subject('Reset Password');
            });

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS') , 'message' => config('service.USER_API.MESSAGE.MAIL_SENT') ]);
        }
        catch(Exception $e)
        {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS') , 'message' => $e->getMessage() , 'records' => []]);
        }
        catch(Throwable $e)
        {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS') , 'message' => $e->getMessage() , 'records' => []]);
        }
    }

    public function sendChangeEmailLink(Request $request)
    { 

        $message = array(
            'email.unique'   => config('service.USER_API.MESSAGE.EMAIL_ALREADY_EXISTS'),
            'email.email'   => config('service.USER_API.MESSAGE.VALID_EMAIL')
        );

        $this->validate($request, 
            ['email' => 'required|email|unique:admins,email', 
        ],$message);

        try
        {
            // create token
            $minutes = 40;
            $token   = Str::random(60);
            $digits = 4;
            $otpCode = rand(pow(10, $digits-1), pow(10, $digits)-1);

            $upd['token']  = $token;
            $upd['otp']  = $otpCode;

            $upd['token_expire'] =  Carbon::now()->addMinutes($minutes)->toDateTimeString();
            $upd['otp_expire'] =  Carbon::now()->addMinutes($minutes)->toDateTimeString();

            DB::table('admins')->where(['id' => $request->user_id])->update($upd);

            $user = DB::table('admins')->select('first_name','email')->where('id', $request->user_id)->first();

            $data['name'] = $user->first_name;
            $data['otp'] = $otpCode;


            // Trigger mail on old email Id
            Mail::send('emails.Users.change_email_otp',['data'=>$data], function($message) use ($user) {
                $message->to($user->email);
                $message->subject('Change Email');
            });


            // Genereate link for change email
            $email = base64_encode($request->email);
            $newData['name'] = $user->first_name;
            $newData['link'] = env('FRONT_END') . '/' . 'admin/user/change-email' . '/' .$email. '/' . $token;
            $newData['link_expire_time'] = $minutes;
            // Trigger mail on new email
            Mail::send('emails.Users.change_email',['data'=>$newData], function($message) use ($request) {
                $message->to($request->email);
                $message->subject('Change Email');
            });

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS') , 'message' => config('service.USER_API.MESSAGE.MAIL_SENT_TO_CHANGE_EMAIL') ]);
        }
        catch(Exception $e)
        {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS') , 'message' => $e->getMessage() , 'records' => []]);
        }
        catch(Throwable $e)
        {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS') , 'message' => $e->getMessage() , 'records' => []]);
        }
    }

    private function validateResetPassword(Request $req, $id)
    {
        $message = array(
            'password.regex' => config('service.USER_API.MESSAGE.PASSWORD_REGEX')
        );
        
        $this->validate($req, [
            'old_password' => 'required',
            'new_password' => [
                'required',
                'min:8', // must be at least 8 characters in length
                'regex:/[a-z]/', // must contain at least one lowercase letter
                'regex:/[A-Z]/', // must contain at least one uppercase letter
                'regex:/[0-9]/', // must contain at least one digit
                'regex:/[@$!%*#?&]/', // must contain a special character
            ],
            'confirm_password' => 'required|same:new_password',
                ], $message);
    }

    public function resetPassword(Request $req, $token) 
    {        
        //has_scope('users.write');

         $user = DB::table('admins')->select('id','first_name','email','token_expire')->where('token', $token)->first();


        if(!$user) {
            return response()->json(['status' => config('service.ERROR_STATUS') , 'message' => config('service.INVALID_TOKEN') ]);
        }


        if($user->token_expire <= Carbon::now()->toDateTimeString() ) {
           return response()->json(['status' => config('service.ERROR_STATUS') , 'message' => config('service.TOKEN_EXPIRED') ]);
        }


        
        $old_password = $req->input('old_password');
        $new_password = $req->input('new_password');

        $old_password_encrypted = hashPassword($old_password);


        $user_detail = oauth::where('password', $old_password_encrypted)
                              ->where('user_fk', $user->id)->first();

        if (!isset($user_detail)) {
            
            $error = ["old_password" => ['The specified password does not match']];

            return response()->json($error, 422);
        }
        
        $this->validateResetPassword($req, $user->id);
        
        try {
                        
            $user = DB::table('admins')->where('id', $user->id)->first();

            $save_data['password'] = hashPassword($new_password);
            $result = DB::table('oauth_users')
                        ->where('user_fk', $user->id)  // find your user by their id
                        ->update($save_data);
            $msg = 'New Password updated successfully!';

            $greeting = 'Hi ' . $user->first_name . '!';
            $body     = 'New Password updated successfully. your login password is : '.$new_password;
            $type     =  'Password Updated';

            $details = [
                'greeting' => $greeting,
                'body' => $body,
                'type' => $type,
                'notification_message' => 'New Password updated successfully!',
                'thanks' => 'Regards,
                 Thanks PM Nowments',
            ];
            
            // $role = '1';
            // $user->notify(new Notification($details));
            // $upd['role'] = $role;
            // $notif_id = $user->notifications[0]->id;
            // $user->notifications()->find($notif_id)->update($upd);

            DB::table('admins')->where('token', $token)->update(['token' => null,'token_expire' => null]);

            $sendMail = new SendMailJob($user->email, $greeting, $body, $type);
            dispatch($sendMail);
            
            return response()->json(['status' => config('service.SUCCESS_STATUS'), 'message' => $msg, 'records' => $result]);
        } 
        catch (Exception $e) {
            
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        } 
        catch (Throwable $e) {
            
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
    }

    public function changeEmail(Request $req, $token) 
    {        
        
         $user = DB::table('admins')->select('id','first_name','email','token_expire','otp','otp_expire')->where('token', $token)->first();


        if(!$user) {
            return response()->json(['status' => config('service.ERROR_STATUS') , 'message' => config('service.INVALID_URL') ]);
        }

        if($user->otp_expire <= Carbon::now()->toDateTimeString() ) {
           return response()->json(['status' => config('service.ERROR_STATUS') , 'message' => config('service.OTP_EXPIRED') ]);
        }

        if($user->token_expire <= Carbon::now()->toDateTimeString() ) {
           return response()->json(['status' => config('service.ERROR_STATUS') , 'message' => config('service.TOKEN_EXPIRED') ]);
        }


        $otp = $req->input('otp');
        $email = base64_decode($req->input('email'));

        $user_detail = DB::table('admins')->where('otp', $otp)
                         ->where('id', $user->id)->first();

        if (!isset($user_detail)) {
            
            $error = ["otp" => ['The otp does not match']];

            return response()->json($error, 422);
        }
        
        try {
                        
            $updateEmail = DB::table('admins')->where('id', $user->id)->update(["email"=>$email]);

            $result = Oauth::where('user_fk', $user->id)->update(["email"=>$email]);

            $msg = 'New Email updated successfully!';


            $greeting = 'Hi ' . $user->first_name . '!';
            $body     = 'New Email updated successfully. your updated email is : '.$email;
            $type     =  'Email Updated';

        

            DB::table('admins')->where('token', $token)->update(['token' => null,'token_expire' => null,'otp' => null,'otp_expire' => null]);

            $sendMail = new SendMailJob($email, $greeting, $body, $type);
            dispatch($sendMail);
            
            return response()->json(['status' => config('service.SUCCESS_STATUS'), 'message' => $msg, 'records' => $result]);
        } 
        catch (Exception $e) {
            
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        } 
        catch (Throwable $e) {
            
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
    }

    public function updatePassword(Request $request, $token)
    {  
        $message = array(
            'password.regex' => config('service.USER_API.MESSAGE.PASSWORD_REGEX'),
        );

        $this->validate($request, [
            'password' => 'required|string|min:8|regex:/^(?=.*[a-z])(?=.*[A-Z])(?=.*\d).+$/',
            'password_confirmation' => 'required|same:password', 
        ],$message);

        try
        {

            $user = User::select('id','first_name','email','token_expire')->where('token', $token)->first();

            if(!$user) {
                return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS') , 'message' => config('service.USER_API.MESSAGE.INVALID_TOKEN') ]);
            }

            if($user->token_expire <= Carbon::now()->toDateTimeString() ) {
                return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS') , 'message' => config('service.USER_API.MESSAGE.TOKEN_EXPIRED') ]);
            }

            $save_data['password'] = hashPassword($request->password);
            
            $result = Oauth::where('user_fk', $user->id)
                    ->where('email', $user->email)
                    ->update($save_data);
            
            $greeting = 'Hi ' . $user->first_name . '!';
            $body     = 'Your password to login as admin on PM Nowments Admin pannel has been recently changed';
            $type     = 'Password Updated';

            $details = [
                'greeting' => $greeting,
                'body' => $body,
                'type' => $type,
                'notification_message' => 'You have change your password successfully.',
                'thanks' => 'Regards,
                 Thanks PM Nowments',
            ];
            
            $sendMail = new SendMailJob($user->email, $greeting, $body, $type);
            dispatch($sendMail);
        
           

            DB::table('admins')->where('token', $token)->update(['token' => null,'token_expire' => null]);

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS') , 'message' => config('service.USER_API.MESSAGE.PASSWORD_UPDATED') ]);
        }
        catch(Exception $e)
        {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS') , 'message' => $e->getMessage() , 'records' => []]);
        }
        catch(Throwable $e)
        {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS') , 'message' => $e->getMessage() , 'records' => []]);
        }
    }

}

