<?php

namespace App\Http\Middleware;

use App\Models\Auth\Oauth;
use App\Models\Customers\Customer;
use App\Models\Vendors\Vendor;
use Closure;
use Illuminate\Contracts\Auth\Factory as Auth;

class Authenticate
{
    /**
     * The authentication guard factory instance.
     *
     * @var \Illuminate\Contracts\Auth\Factory
     */
    protected $auth;
    protected $oauth_user;

    /**
     * Create a new middleware instance.
     *
     * @param  \Illuminate\Contracts\Auth\Factory  $auth
     * @return void
     */
    public function __construct(Auth $auth, Oauth $oauth_user)
    {
        $this->auth = $auth;
        $this->oauth_user = $oauth_user;
    }

    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure  $next
     * @param  string|null  $guard
     * @return mixed
     */
    public function handle($request, Closure $next, $guard = null)
    {
        $storage = new \OAuth2\Storage\Pdo(array(
            'dsn' => 'mysql:dbname='.getenv('DB_DATABASE').';host='.getenv('DB_HOST'),
            'username' => getenv('DB_USERNAME'),
            'password' => getenv('DB_PASSWORD'),
        ), ['user_table' => 'oauth_users']);

        $server = new \OAuth2\Server($storage);

        if (!$server->verifyResourceRequest(\OAuth2\Request::createFromGlobals())) {

            $server->getResponse()->send();
            die;
        }

        $token = $server->getAccessTokenData(\OAuth2\Request::createFromGlobals());
        
        $o_auth = $this->oauth_user::with(['provider_time_zone','customer_time_zone'])->select('user_fk', 'role_fk')
                ->where('user_id', $token['user_id'])->first();
        
        $request->user_id = isset($o_auth->user_fk) ? $o_auth->user_fk : "";
        $request->role_fk = isset($o_auth->role_fk) ? $o_auth->role_fk : "";
        $request->user_fk = isset($o_auth->user_fk) ? $o_auth->user_fk : "";
        
        $request->time_zone = !empty($o_auth->provider_time_zone->timezone) ? $o_auth->provider_time_zone->timezone:config('app.timezone');
        
        $request->customer_timezone = !empty($o_auth->customer_time_zone->timezone) ? $o_auth->customer_time_zone->timezone:config('app.timezone');

        $request->admin_time_zone = "America/New_York";
        
        return $next($request);
    }
}
