<?php

namespace App\Models\Contractors;

use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Auth\Authenticatable;
use Illuminate\Contracts\Auth\Access\Authorizable as AuthorizableContract;
use Illuminate\Contracts\Auth\Authenticatable as AuthenticatableContract;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Laravel\Lumen\Auth\Authorizable;

use App\Models\ContractorCategory;
use App\Models\Contractors\Address;
use App\Models\Contractors\Review;
use Illuminate\Notifications\Notifiable;

class Contractor extends Model implements AuthenticatableContract, AuthorizableContract
{
    use Authenticatable, Authorizable, HasFactory, SoftDeletes, Notifiable;


    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'first_name',
        'last_name',
        'email',
        'phone',
        'password',
        'expertise',
        'exprence',
        'licence_no',
        'latitude',
        'longitude',
        'address',
        'otp',
        'image',
        'device_id',
        'device_token',
        'device_type',
        'about_company'
    ];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password',
        'deleted_at',
        'otp',
    ];


    /**
     * Get all addresses
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function address() {
        return $this->hasMany(Address::class, 'user_id', 'id');
    }

    /**
     * Get the contractor category.
     */
    public function contractor_category() {
        return $this->hasMany(ContractorCategory::class, 'contractor_id', 'id');
    }

    public function rating(){
    	return $this->hasMany(Review::class, 'contractor_id', 'id');
    }

    public function jobs() {
        return $this->hasMany(Jobs::class, 'contractor_id', 'id');
    }
}
