<?php

namespace App\Models\Customers;

use Illuminate\Database\Eloquent\Model;
use Carbon\Carbon;
use App\Models\Providers\Provider;
use App\Models\Customers\Customer;
use App\Models\Providers\MessageRoomChat;

class MessageRoom extends Model
{
    protected $table = 'message_rooms';

    protected $fillable = [
        'room_id', 
        'customer_id', 
        'provider_id', 
        'appointment_id', 
        'last_message', 
        'last_message_date_time',
        'deleted_by_customer',
        'deleted_by_provider',
    ];

    protected $appends = ['utc_last_date_time'];

    public function provider() {
        return $this->belongsTo(Provider::class, 'provider_id');
    }

    public function customer() {
        return $this->belongsTo(Customer::class, 'customer_id');
    }

    public function unread_message() {
        return $this->belongsTo(MessageRoomChat::class, 'room_id', 'room_id');
    }

    public function getLastMessageDateTimeAttribute ($value) 
    {
        // $timezone = request()->customer_timezone;
        // $dateTime = Carbon::parse($value)->timezone($timezone);
        // return date("m/d/Y, h:i A", strtotime($dateTime));
        return date("m/d/Y, h:i A", strtotime($value));
    }

    public function getUtcLastDateTimeAttribute()
    {
        if(!empty($this->attributes['last_message_date_time'])){
            return  $this->attributes['last_message_date_time'];
         } 
    }
}
