<?php

namespace App\Models\Orders;

use Illuminate\Database\Eloquent\Model;

class Order extends Model
{
    protected $fillable = [ 
       'order_id',
       'price',
       'quantity',
       'customer_id',
       'vendor_id',
       'product_id',
       'variant_id',
       'status',
       'receiver_name',
       'address',
       'state',
       'pincode',
       'message',
       'discount',
       'final_amount'
    ];

    protected $casts = [
        'created_at' => 'datetime:Y-m-d H:i:s',
        'updated_at' => 'datetime:Y-m-d H:i:s',
    ];

    public function payment() {
        return $this->belongsTo('App\Models\Payments\Payment','id','order_id');
    }

    public function variants() 
    {
        return $this->hasMany('App\Models\Products\ProductVariant','id','variant_id')->select('id','product_id','color_id','size_id','price','quantity')->with('colors:id,color','sizes:id,size','variant_images:id,variant_id,image');
    }

    public function product() {
        return $this->belongsTo('App\Models\Products\Product','product_id')->select('id','product_id','vendor_id','name','description','on_sale');
    }

    public function vendor() {
        return $this->belongsTo('App\Models\Vendors\Vendor','vendor_id')->select('id','first_name','last_name','email','phone_code','phone');
    }

    public function customer() {
        return $this->belongsTo('App\Models\Customers\Customer','customer_id')->select('id','first_name','last_name','email','phone_code','phone');
    }


}
