<?php

namespace App\Models\Payments;

use Illuminate\Database\Eloquent\Model;

class Payment extends Model
{
    protected $fillable = [ 
       'order_id',
       'customer_id',
       'method',
       'price',
       'transaction',
       'transcation_id',
       'commission',
       'commission_recived',
       'commission_verified'
    ];

    protected $casts = [
        'created_at' => 'datetime:Y-m-d H:i:s',
        'updated_at' => 'datetime:Y-m-d H:i:s',
    ];

    protected $appends = [
        'gst_no',
        'discount',
    ];

    public function order() {
        return $this->belongsTo('App\Models\Orders\Order','order_id')
                               ->select('id','price','customer_id','vendor_id','product_id')
                               ->with('product','vendor','customer');
    }

    public function getGstNoAttribute () {
        $value = $this->getAttributeValue('id');
        return "GSTNO".$value;
    }

    public function getDiscountAttribute () {
        return 0;
    }
    public function getCreatedAtAttribute ($created_at) 
   { 
      return convertTimeUtcToLocal($created_at);
     
   }

   public function getUpdatedAtAttribute ($updated_at) 
   { 
      return convertTimeUtcToLocal($updated_at);
   }

   public function setCreatedAtAttribute () 
   { 
      $this->attributes['created_at']=convertcurrenttimetoUtc(date('Y-m-d H:i:s'));

   }
   
   public function setUpdatedAtAttribute () 
   { 
      $this->attributes['updated_at']=convertcurrenttimetoUtc(date('Y-m-d H:i:s'));
   } 

}
