<?php

namespace App\Models\Products;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class Product extends Model
{
    use SoftDeletes;
    
    protected $fillable = [       
        'vendor_id', 
        'product_id',
        'name', 
        'price', 
        'status',
        'description',
        'brand_id',
        'on_sale',
        'discount',
        'discount_start',
        'discount_end',
        'keyword'
    ];

    protected $casts = [
        'description'  => 'string',
        'created_at' => 'datetime:Y-m-d H:i:s',
        'updated_at' => 'datetime:Y-m-d H:i:s',
    ];

    protected $hidden = ['deleted_at'];

    public function vendor(){        
    	return $this->belongsTo('App\Models\Vendors\Vendor', 'vendor_id')
                                ->select(
                                         'id','first_name','last_name','latitude','longitude',
                                         'image','address','city','country','state','zipcode'
                                        );
    }

    public function images() {
        return $this->hasMany('App\Models\Products\ProductImage', 'product_id');
    }

    
    public function categories()
    {
        return $this->belongsToMany('App\Models\Products\Category','product_categories','product_id','category_id');
    }

    public function subcategories()
    {
        return $this->belongsToMany('App\Models\Products\Category','product_subcategories','product_id','category_id');
    }

    public function variants() 
    {
        return $this->hasMany('App\Models\Products\ProductVariant','product_id')->select('id','product_id','color_id','size_id','price','quantity')->with('colors:id,color,hex_code','sizes:id,size','variant_images:id,variant_id,image');
    }

    public function brand() {
        return $this->belongsTo('App\Models\Brands\Brand','brand_id');
    }

    public function in_wallet() {
        return $this->belongsTo('App\Models\Wallets\Wallet','id','product_id')->select('id','product_id','user_id');
    }

    public function rating() {
        return $this->hasMany('App\Models\Products\Review','product_id')->where('status',1);
    }

    public function averageRating() {
        return $this->hasMany('App\Models\Products\Review','product_id')->where('status',1)->avg('rating');
    }
}
