<?php

namespace App\Models\Providers;

use Illuminate\Database\Eloquent\Model;
use Carbon\Carbon;

class Appointment extends Model
{
    protected $table = 'appointments';

    protected $fillable = [
        'appointment_id ', 
        'customer_id', 
        'provider_id', 
        'date', 
        'day', 
        'time', 
        'end_time', 
        'customer_note',
        'provider_note',
        'provider_cancel_request',
        'appointment_cancel',
        'appointment_status'
    ];

    protected $casts = [
        'created_at' => 'datetime:Y-m-d H:i:s',
        'updated_at' => 'datetime:Y-m-d H:i:s',
    ];

    protected $appends = ['end_time_utc'];

    public static function getNextAppointmentId()
    {
       $appointment_id = Appointment::orderBy('id','desc')->value('id');

       $appointment_id = ($appointment_id)?$appointment_id+1:1;

       return 'APP-'.str_pad($appointment_id,3,"0",STR_PAD_LEFT);
   }

   public function customer(){        
        return $this->belongsTo('App\Models\Customers\Customer', 'customer_id','id');
   }

   public function provider(){        
        return $this->belongsTo('App\Models\Providers\Provider', 'provider_id','id');
   }

   public function no_show_appointment(){        
        return $this->hasOne('App\Models\AppointmentManagement\AppointmentNoShow', 'appointment_id','id');
   }

   public function cancell_appointment(){        
        return $this->hasOne('App\Models\Customers\AppointmentCancelled', 'appointment_id','id');
   }

   public function getTimeAttribute ($value) 
   {
     //    $timezone = request()->time_zone;
     //    $dateTime = Carbon::parse($value)->timezone($timezone);
        return date("h:i A", strtotime($value));
     // return $value;
   }

   public function getEndTimeAttribute ($value) 
   {
     //    $timezone = request()->time_zone;
     //    $dateTime = Carbon::parse($value)->timezone($timezone);
        return date("h:i A", strtotime($value));
     //    return $value;
   }

   public function getEndTimeUtcAttribute ($value) 
   {
        if(!empty($this->attributes['end_time'])){
            return  $this->attributes['end_time'];
         } 
   }

   public function getDateAttribute ($value) 
   {
       return showDateFormat($value);
   }

}
