<?php

namespace App\Models\Providers;

use Illuminate\Database\Eloquent\Model;
use Carbon\Carbon;
use App\Models\Providers\Provider;
use App\Models\Customers\Customer;
use App\Models\Providers\Appointment;

class AudioVideoCallHistory extends Model
{
    protected $table = 'audio_video_call_history';

    protected $fillable = [
        'appointment_id', 
        'customer_id', 
        'provider_id', 
        'duration', 
        'call_started_at', 
        'call_end_at',
        'temp_call_started_at', 
        'temp_call_end_at',
    ];

    protected $casts = [
        'created_at' => 'datetime:Y-m-d H:i:s',
        'updated_at' => 'datetime:Y-m-d H:i:s',
    ];
    
    protected $appends = ['call_started_at_utc'];

    public function appointment() {
        return $this->belongsTo(Appointment::class, 'appointment_id');
    }

    public function customer() {
        return $this->belongsTo(Customer::class, 'customer_id');
    }

    public function provider() {
        return $this->belongsTo(Provider::class, 'provider_id');
    }

   public function getCallStartedAtAttribute ($value) 
   {
        $timezone = request()->time_zone;
        $dateTime = Carbon::parse($value)->timezone($timezone);
        return date("H:i:s", strtotime($dateTime));
   }

   public function getCallEndAtAttribute ($value) 
   {
        $timezone = request()->time_zone;
        $dateTime = Carbon::parse($value)->timezone($timezone);
        return date("H:i:s", strtotime($dateTime));
   }

   public function getCallStartedAtUtcAttribute ($value) 
   {
        if(!empty($this->attributes['call_started_at'])){
            return  $this->attributes['call_started_at'];
        } 
   }

}
