<?php

namespace App\Models\Providers;

use Illuminate\Auth\Authenticatable;
use Illuminate\Contracts\Auth\Access\Authorizable as AuthorizableContract;
use Illuminate\Contracts\Auth\Authenticatable as AuthenticatableContract;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Laravel\Lumen\Auth\Authorizable;

use Illuminate\Notifications\Notifiable;
use Spatie\Activitylog\Traits\LogsActivity;
use Spatie\Activitylog\LogOptions;


class ProviderLiabilityInsurance extends Model implements AuthenticatableContract, AuthorizableContract
{
    use Authenticatable, Authorizable, HasFactory, Notifiable,LogsActivity;

    protected $table = 'provider_liability_insurances';
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'provider_id',
        'issued_date',
        'expiration_date'
    ];

    /**
     * The attributes excluded from the model's JSON form.
     *
     * @var array
     */

    protected $casts = [
        'created_at' => 'datetime:Y-m-d H:i:s',
        'updated_at' => 'datetime:Y-m-d H:i:s',
    ];

    public function liabilityInsuranceFiles()
    {
        return $this->hasMany('App\Models\Providers\ProviderDocumentFile','provider_liability_insurance_id','id');
    }

    public function getIssuedDateAttribute ($value) 
    {
        return showDateFormat($value);
    }

    public function getExpirationDateAttribute ($value) 
    {
        return showDateFormat($value);
    }

    public function getActivitylogOptions(): LogOptions
    {
        return LogOptions::defaults()
        ->useLogName('Provider Liability insurance')
        ->logOnlyDirty()
        ->logOnly(['issued_date','expiration_date'])
        ->setDescriptionForEvent(fn(string $eventName) => "This liability insurance has been {$eventName}");
    }

    
}
