<?php

namespace App\Models\Providers;

use Illuminate\Auth\Authenticatable;
use Illuminate\Contracts\Auth\Access\Authorizable as AuthorizableContract;
use Illuminate\Contracts\Auth\Authenticatable as AuthenticatableContract;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Laravel\Lumen\Auth\Authorizable;

use Illuminate\Notifications\Notifiable;
use Spatie\Activitylog\Traits\LogsActivity;
use Spatie\Activitylog\LogOptions;


class ProviderLicensesPendingApprove extends Model implements AuthenticatableContract, AuthorizableContract
{
    use Authenticatable, Authorizable, HasFactory, Notifiable,LogsActivity;

    protected $table = 'provider_licenses_pending_approve';
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'license_id',
        'provider_id',
         'license_title', 
         'license_number',
         'license_board',
         'issued_date',
         'expiration_date',
         'status'
    ];

    /**
     * The attributes excluded from the model's JSON form.
     *
     * @var array
     */
    protected $hidden = [
        'password',
    ];

    protected $casts = [
        'created_at' => 'datetime:Y-m-d H:i:s',
        'updated_at' => 'datetime:Y-m-d H:i:s',
    ];

    public function licenseFiles()
    {
        return $this->hasMany('App\Models\Providers\ProviderDocumentFilesPendingApprove','provider_license_id','id');
    }
    
    public function states(){ 
        return $this->belongsTo('App\Models\Providers\State','state','id');       
    }

    public function getIssuedDateAttribute ($value) 
    {
        return showDateFormat($value);
    }

    public function getExpirationDateAttribute ($value) 
    {
        return showDateFormat($value);
    }

    public function getActivitylogOptions(): LogOptions
    {
        return LogOptions::defaults()
        ->useLogName('Provider License')
        ->logOnlyDirty()
        ->logOnly(['license_title', 'license_number','license_board','issued_date','expiration_date'])
        ->setDescriptionForEvent(fn(string $eventName) => "This license has been {$eventName}");
    }

    
}
