<?php

namespace App\Models\Users;

use Illuminate\Auth\Authenticatable;
use Illuminate\Contracts\Auth\Access\Authorizable as AuthorizableContract;
use Illuminate\Contracts\Auth\Authenticatable as AuthenticatableContract;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Laravel\Lumen\Auth\Authorizable;

use Illuminate\Notifications\Notifiable;
use App\Models\Users\Address;
use App\Models\Users\Oauth;
use Illuminate\Database\Eloquent\SoftDeletes;

class Admin extends Model implements AuthenticatableContract, AuthorizableContract
{
    use Authenticatable, Authorizable, HasFactory, Notifiable;

    protected $table = 'admins';
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'username',
        'first_name', 
        'last_name', 
        'email', 
        'password', 
        'image', 
        'phone',
        'phone_code',
        'token_expire'
    ];

    protected $casts = [
        'created_at' => 'datetime:Y-m-d H:i:s',
        'updated_at' => 'datetime:Y-m-d H:i:s',
    ];

    /**
     * The attributes excluded from the model's JSON form.
     *
     * @var array
     */
    protected $hidden = [
        'password',
    ];

    /**
     * Get all user addresses
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function address() {
        return $this->hasMany(Address::class, 'user_id', 'id');
    }

    /**
     * Get all user addresses
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function jobs() {
        return $this->hasMany(Jobs::class, 'user_id', 'id');
    }

    public function oauth(){        
    	return $this->belongsTo('App\Models\Users\Oauth', 'id','user_fk')->where('role_fk','1');
    }

    public function getImageAttribute($value) 
    {
        $target = "../storage/userProfile/".$value;
        
        return image_to_base64($target);
    }

    
}
