<?php
namespace App\Http\Controllers\Providers;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Controllers\Providers\ProviderNotificationController;
use App\Models\Customers\Contractor;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;
use App\Models\Customers\Jobs;
use App\Models\Providers\Provider;
use App\Models\Customers\TempCustomer;
use App\Models\Providers\Oauth;
use App\Models\Providers\ProviderLicense;
use App\Models\Providers\ProviderDocumentFile;
use App\Models\Providers\ProviderCertification;
use App\Models\Providers\ProviderFbiBackground;
use App\Models\Providers\ProviderChildAbuseHistory;
use App\Models\Providers\ProviderLiabilityInsurance;
use App\Models\Providers\ProviderProfessionalReference;
use App\Models\Providers\ProviderSpecialization;
use App\Models\Providers\ProviderLanguage;
use App\Models\Providers\Specialization;
use App\Models\Providers\Language;
use App\Models\Providers\State;
use App\Models\Providers\ProviderAvailability;
use App\Models\Providers\ProviderTimeOff;
use App\Models\Customers\CustomerIntakeQuestion;
use App\Http\Controllers\Auth\OauthServerController;
use App\Notifications\Providers\Notification;
use App\Models\Settings\Setting;
use App\Models\Users\User;
use App\Models\Providers\Appointment;
use App\Models\Customers\ProviderRatingReview;
use App\Models\Journals\Journal;
use App\Models\Providers\FollowUpMessage;
use App\Models\Providers\AudioVideoCallHistory;
use App\Models\Notifications\ScheduledNotification;
use App\Models\Providers\MessageRoom;
use App\Models\Providers\MessageRoomChat;
use App\Models\Providers\Customer;
use App\Models\Providers\ProviderSlot;
use App\Models\AppointmentManagement\NoShow;
use App\Models\UserPasswordHistory\UserPasswordHistory;
use App\Models\Notifications\NotificationSetting;
use App\Jobs\ProviderForgotPasswordMailJob;
use App\Jobs\ProviderMailJob;
use App\Jobs\ProviderCommonMailJob;
use App\Jobs\ProviderCallJoinMailJob;
use App\Jobs\ProviderWelcomeMailJob;
use App\Jobs\ProviderConfirmEmailJob;
use App\Jobs\ProviderTimeOffMailJob;
use App\Jobs\ProviderRegisteredEmailJob;
use App\Jobs\CustomerNoShowEmailJob;
use App\Jobs\TestJob;
use App\Models\Customers\AppointmentCancelled;
use App\Models\EmailTemplate\EmailTemplate;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\App;
use Carbon\Carbon;
use Carbon\CarbonPeriod;
use Validator;
use Exception;
use Throwable;

class ProviderController extends Controller 
{
    /**
    * The user repository implementation.
    *
    * @var UserRepository
    */
    protected $providers;
    protected $oauth_user;
    protected $jobs;
    protected $user_id;
    protected $role_fk;


    public function __construct(Provider $providers, Oauth $oauth_user, Jobs $jobs, Request $request)
    {
        $this->providers = $providers;
        $this->oauth_user = $oauth_user;
        $this->jobs = $jobs;
        $this->user_id = $request->user_id;
        $this->role_fk = $request->role_fk;
        $this->user_fk = $request->user_fk;
        //has_scope('customers.read');
        
        date_default_timezone_set('UTC');
    }


    /**
    * @ user list List api using pagination.
    * @return \Illuminate\Http\Response
    */
    public function index(Request $request)
    {
        try {
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;
            if (isset($request->order) && strtolower($request->order) == "asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if ($request->filled('order_by_column_name')) {
                if (in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }
            $providers = $this->providers::with(['licenses','liabilityInsurances','fbiBackgrounds','childAbuseHistories'])
            ->where(function ($q) use ($request) {
                // Global searching...
                if (isset($request->search) && !empty($request->search) && $request->search != "") {
                    if (!empty($this->getColumnName())) {
                        foreach ($this->getColumnName() as $key => $col) {
                            if ($key == 0) {
                                $q->where($col, 'like', '%' . encryptString($request->search) . '%');
                                $q->orWhere('email', 'like', '%' . encryptString(strtolower($request->search)) . '%');
                                $q->orWhere('first_name', 'like', '%' . encryptString(ucfirst(strtolower($request->search))) . '%');
                                $q->orWhere('last_name', 'like', '%' . encryptString(ucfirst(strtolower($request->search))) . '%');
                            } else {
                                $pieces = explode(" ", $request->search);
                                if(count($pieces)){
                                    $fName = !empty($pieces[0]) ? $pieces[0] : $request->search; 
                                    $lName = !empty($pieces[1]) ? $pieces[1] : $request->search; 
                                }
                                $q->orWhere($col, 'like', '%' . encryptString($request->search) . '%');
                                $q->orWhere(DB::raw('CONCAT(first_name, " ",last_name)'), 'like', '%'.encryptString(ucfirst(strtolower($fName)))." ".encryptString(ucfirst(strtolower($lName))).'%');
                            }
                        }
                    }
                }
                // filters
                if($request->filled('name')) {
                    $pieces = explode(" ", $request->name);
                    if(count($pieces)){
                        $fName = !empty($pieces[0]) ? $pieces[0] : $request->name; 
                        $lName = !empty($pieces[1]) ? $pieces[1] : $request->name; 
                    }
                    $q->where('first_name', 'like', '%'.encryptString(ucfirst(strtolower($request->name))).'%');
                    $q->orwhere("last_name",'like', '%'.encryptString(ucfirst(strtolower($request->name))).'%');
                    $q->orWhere(DB::raw('CONCAT(first_name, " ",last_name)'), 'like', '%' .encryptString(ucfirst(strtolower($fName)))." ".encryptString(ucfirst(strtolower($lName))).'%');
                }
                if($request->filled('email')) {
                    $q->where('email', 'like', '%'.encryptString((strtolower($request->email))).'%');
                }
                if($request->filled('status')) {
                    $q->where('status',$request->status);
                }
                if($request->filled('date_from') && $request->filled('date_to')) {
                    $date_from = date("Y-m-d", strtotime($request->date_from));
                    $date_to = date("Y-m-d", strtotime($request->date_to));
                    $q->whereBetween('created_at', [$date_from." "."00:00:00", $date_to." "."23:59:59"]);
                }
            })
            ->where(function ($q) use ($request) {
                // Status wise records
                $allowableColumn = $this->getColumnName();
                if (isset($request->column_name) && !empty($request->column_name) && $request->column_value == "Pending" && in_array($request->column_name, $allowableColumn)) {
                    $q->where([
                        $request->column_name => $request->column_value,
                        'register_step'=>5,
                        'register_status'=>'completed'
                    ]);
                }
                if (isset($request->column_name) && !empty($request->column_name) && $request->column_value == "Rejected" && in_array($request->column_name, $allowableColumn)) {
                    $q->where([
                        $request->column_name => $request->column_value,
                        'register_step'=>5,
                        'register_status'=>'completed'
                    ]);
                }
                if (isset($request->column_name) && !empty($request->column_name) && $request->column_value == "Approved" && in_array($request->column_name, $allowableColumn)) {
                    $q->where([
                        $request->column_name => $request->column_value,
                        'register_step'=>5,
                        'register_status'=>'completed'
                    ]);
                }
            })
            ->with('oauth:user_fk,username,is_block')
            ->orderBy($order_by_column_name, $order)
            ->paginate($limit);
            if (empty($providers)) {
                throw new Exception(config('service.PROVIDER_API.MESSAGE.PROVIDER_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_LISTING'), 'records' => $providers]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    private function getColumnName()
    {
        $address = $this->providers::first();
        if (!empty($address) && $address != "") {
            return array_keys($address->toArray());
        }
        return ['id'];
    }


    public function approveProvider($id) 
    {
        //has_scope('cms.pages.delete');
        $provider = $this->providers::findOrFail($id);
        try {
            $provider->application_status = "Approved";
            $provider->save();

            $templateName = "Approve provider";
            $templateData = EmailTemplate::where('name',$templateName)->first();
            if($templateData)
            {
                $greeting = 'Hi ' . decryptString($provider->first_name).' '.decryptString($provider->last_name).',';
                //$body     = 'Your account has been successfully approved on PM Now.Thank you!';
                $body     = $templateData->text;
                $type     = $templateData->title;
                $sendMail = new ProviderCommonMailJob(decryptString($provider->email), $greeting, $body, $type);
                $this->dispatchNow($sendMail);
            }

            /* Push and db notification */
                $notificationSetting = NotificationSetting::where('provider_id',$id)->first();
                if($notificationSetting->show_notification == 'active'){
                    (new ProviderNotificationController)->approveProvider($id);
                }
            /* End push and db notification */

            return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_APPROVE'), 'records' => $provider]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function rejectProvider(Request $request, $id) 
    {
        //has_scope('cms.pages.delete');
        $provider = $this->providers::findOrFail($id);
        try {
           
            $provider->application_status = "Rejected";
            $provider->rejection_reason = $request->rejection_reason;
            $provider->rejection_date = date("Y-m-d H:i:s");
            $provider->save();

            $templateName = "Provider account rejected";
            $templateData = EmailTemplate::where('name',$templateName)->first();
            if($templateData)
            {
                $greeting = 'Hi ' .decryptString($provider->first_name).' '.decryptString($provider->last_name).' ,';
                $body     = $templateData->text." ".$request->rejection_reason;
                $type     = $templateData->title;
                $sendMail = new ProviderCommonMailJob(decryptString($provider->email), $greeting, $body, $type);
                $this->dispatchNow($sendMail);
            }
            return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_REJECT'), 'records' => $provider]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function unblockProvider($id) 
    {
        $provider = Provider::findOrFail($id);
        try {
            $provider->login_attempt = Null;
            $provider->is_blocked = "no";
            $provider->login_attempt_time = Null;
            $provider->save();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_UNBLOCK'), 'result' => $provider]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }


    public function deleteProvider(Request $request, $id) 
    {
        //has_scope('roles.delete');
        $provider = Provider::findOrFail($id);
        try {
            DB::beginTransaction();
            $email = $provider->email;
            $name = decryptString($provider->first_name)." ".decryptString($provider->last_name);
            $provider  = $provider->delete();
            $this->oauth_user::where(['user_fk' => $id,'email'=>$email])->delete();

            $templateName = "Provider account delete";
            $templateData = EmailTemplate::where('name',$templateName)->first();
            if($templateData)
            {
                /* Delete provider email */
                    $greeting = 'Hi ' . $name . ',';
                    $body     = $templateData->text;
                    $type     = $templateData->title;
                    $sendMail = new ProviderCommonMailJob(decryptString($email), $greeting, $body, $type);
                    $this->dispatchNow($sendMail);
                /* End delete provider email */
            }

            DB::commit();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_DELETED'), 'result' => $provider]);
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }


    public function activeInactive(Request $request, $id) 
    {
        //has_scope('cms.pages.write');
        $this->validate($request,[
            'status' => 'required',
        ]);
        try {
        //$save_data['status'] = $request->status;
            $result = Provider::findOrFail($id);
            $result->status = $request->status;
            $result->save();

            if($request->status=="active")
            {
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_STATUS_ACTIVE'),'action'=>$request->status]);
            }
            else
            {
                /* delete access token */
                    $user =  $this->oauth_user::select('user_id')->where(['user_fk' => $id])->first();
                    DB::table('oauth_access_tokens')->where('user_id',$user->user_id)->delete(); 
                /* end access token */
                
                $templateName = "Provider Account inactive";
                $templateData = EmailTemplate::where('name',$templateName)->first();
                if($templateData)
                {
                    /* Provider inactive email */
                        $greeting = 'Hi ' . decryptString($result->first_name)." ".decryptString($result->last_name) . ',';
                        $body     = $templateData->text;
                        $type     = $templateData->title;
                        $sendMail = new ProviderCommonMailJob(decryptString($result->email), $greeting, $body, $type);
                        $this->dispatchNow($sendMail);
                    /* End Provider inactive email  */
                }
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_STATUS_INACTIVE'),'action'=>$request->status]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
    }


    public function show(Request $request,$id) 
    {
        try
        {
            $provider =  Provider::with(
                [
                    'languages',
                    'specializations',
                    'licenses.licenseFiles',
                    'licenses.states',
                    'fbiBackgrounds.fbiBackgroundFiles',
                    'certifications.certificationFiles',
                    'liabilityInsurances.liabilityInsuranceFiles',
                    'providerAvailability',
                    

                ])
            ->find($id);
           // 'childAbuseHistories.childAbuseFiles','professionalReferences', remove from uper query
            $specializations = Specialization::where(['status'=>1])->get();
            $specializationIds = [];
            foreach($provider->specializations as $specialization)
            {
                $specializationIds[] = $specialization->id;
            }
            $state=State::where('status',true)->get();
            /*$directoryName = "providerProfileImages";
            $providerProfilePresignedUrl = presignedUrl($directoryName,$provider->image);*/
            if( empty($provider) ) {
                throw new Exception(config('service.PROVIDER_API.MESSAGE.PROVIDER_NOT_FOUND'));
            }
            return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_DETAILS'), 'records' => $provider,'specializations'=>$specializations,'specializationIds'=>$specializationIds,'state'=>$state]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function updateProvider(Request $request, $id)
    {
        has_scope('provider.write');
        $message = array(
            'birth_date.before_or_equal' => config('service.PROVIDER_API.MESSAGE.BIRTH_DATE'),
            'phone.regex' => 'The phone format is invalid. '
        );
        $this->validate($request, [
            'first_name' => 'required|max:30',
            'last_name' => 'required|max:30',
            'birth_date' => 'required|date|before_or_equal:'.\Carbon\Carbon::now()->subYears(18)->format('m/d/Y'),
            //'phone' => "required|regex:/^([0-9\s\-\+\(\)]*)$/|min:10|max:10|unique:providers,phone,$id",
            'phone' => "required|unique:providers,phone,$id",
            //'email' => "required|email|max:100|unique:providers,email,$id",
            'email' => "required|max:100|unique:providers,email,$id",
            'gender' => 'required',
            //'address' => 'required|max:2000',
            'experience' => 'required|max:100',
            'bio' => 'max:1500',
            'specialization' => 'required',
        ], $message);
        try {
            $provider = Provider::findOrFail($id);
            DB::beginTransaction();
            $input = $request->all();
            $input['birth_date'] = dateFormat($request->birth_date);
            $input['first_name'] = encryptString(ucfirst(strtolower($request->first_name)));
            $input['last_name']  = encryptString(ucfirst(strtolower($request->last_name)));
            $input['phone']      = encryptString($request->phone);
            $input['email']      = encryptString($request->email);
            $input['address']    = encryptString($request->address);
            $input['bio']        = encryptString($request->bio);
            
            $input['updated_at']=convertcurrenttimetoUtc(date('Y-m-d H:i:s'));
            
            if($request->has('image')) {
                $imageName = rand(10000, 9999999) . '-' . time() . '.' . request()->image->getClientOriginalExtension();
                $directoryName = "providerProfileImages";
                uploadFile(request()->image,$directoryName,$imageName);
                $input['image'] =  $imageName;
            }
            $save  = $provider->update($input);
            // update specialization Ids.
            if(!empty($request->specialization)) {
                $provider->specializations()->sync($request->specialization);
            }
            $oauth_user = $this->oauth_user::where(['user_fk'=>$id])
            ->update([
                'username' => encryptString($request->email),
                'email' => encryptString($request->email),
                'mobile' => encryptString($request->phone),
            ]);
            DB::commit();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROFILE_UPDATED'), 'records' =>$save]);
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function updateVerification(Request $request, $id)
    {
        has_scope('provider.write');
        $this->validate($request, [
            'fbi_background_clearance_date'     => 'required',
         //   'child_abuse_clearance_date'        => 'required',
            'liability_insurance_issued_date'   => 'required',
            'liability_insurance_expiration_date'  => 'required',
        ]);
        try {
            //dd($request->all());
            DB::beginTransaction();
            $input = $request->all();

            /*   fbi background */
            //$delete = ProviderFbiBackground::where('provider_id', $id)->delete();
            $fbiBackground = ProviderFbiBackground::where('provider_id', $id)->first();
            $fbiBackground->provider_id = $id;
            $fbiBackground->clearance_date = dateFormat($input['fbi_background_clearance_date']);
            $fbiBackground->save();
            if(!empty($input['fbi_background_files']))
            {
                foreach($input['fbi_background_files'] as $file)
                {
                    $imageName = rand(10000, 9999999) . '-' . time() . '.' .$file->getClientOriginalExtension();
                    $directoryName = "providerFbiBackgroundFiles";
                    uploadFile($file,$directoryName,$imageName);
                    $fileExtension = substr(strrchr($imageName,'.'),1);
                    $imageExtension = array("jpg", "jpeg", "png");
                    if(in_array($fileExtension, $imageExtension)){
                        $fileType = "image";
                    }
                    $pdfExtension = array("pdf");
                    if(in_array($fileExtension, $pdfExtension)){
                        $fileType = "pdf";
                    }
                    $documentFile = new ProviderDocumentFile();
                    $documentFile->provider_fbi_background_id = $fbiBackground->id;
                    $documentFile->file = $imageName;
                    $documentFile->type = $fileType;
                    $documentFile->document_type = "fbi_background";
                    $documentFile->save();
                }
            }
            /*   fbi background */
            /*  child abuse files */
            //$delete = ProviderChildAbuseHistory::where('provider_id', $id)->delete();
            // $childAbuseHistory = ProviderChildAbuseHistory::where('provider_id', $id)->first();
            // $childAbuseHistory->provider_id = $id;
            // $childAbuseHistory->clearance_date = dateFormat($input['child_abuse_clearance_date']);
            // $childAbuseHistory->save();
            // if(!empty($input['child_abuse_files']))
            // {
            //     foreach($input['child_abuse_files'] as $file)
            //     {
            //         $imageName = rand(10000, 9999999) . '-' . time() . '.' .$file->getClientOriginalExtension();
            //         $directoryName = "providerChildAbuseFiles";
            //         uploadFile($file,$directoryName,$imageName);
            //         $fileExtension = substr(strrchr($imageName,'.'),1);
            //         $imageExtension = array("jpg", "jpeg", "png");
            //         if(in_array($fileExtension, $imageExtension)){
            //             $fileType = "image";
            //         }
            //         $pdfExtension = array("pdf");
            //         if(in_array($fileExtension, $pdfExtension)){
            //             $fileType = "pdf";
            //         }
            //         $documentFile = new ProviderDocumentFile();
            //         $documentFile->provider_child_abuse_history_id = $childAbuseHistory->id;
            //         $documentFile->file = $imageName;
            //         $documentFile->type = $fileType;
            //         $documentFile->document_type = "child_abuse";
            //         $documentFile->save();
            //     }
            // }
            /*  child abuse files */
            //$delete = ProviderLiabilityInsurance::where('provider_id', $id)->delete();
            $liabilityInsurance = ProviderLiabilityInsurance::where('provider_id', $id)->first();
            $liabilityInsurance->provider_id = $id;
            $liabilityInsurance->issued_date = dateFormat($input['liability_insurance_issued_date']);
            $liabilityInsurance->expiration_date = dateFormat($input['liability_insurance_expiration_date']);
            $liabilityInsurance->save();
            if(!empty($input['liability_insurance_files']))
            {
                foreach($input['liability_insurance_files'] as $file)
                {
                    $imageName = rand(10000, 9999999) . '-' . time() . '.' .$file->getClientOriginalExtension();
                    $directoryName = "providerLiabilityInsuranceFiles";
                    uploadFile($file,$directoryName,$imageName);
                    $fileExtension = substr(strrchr($imageName,'.'),1);
                    $imageExtension = array("jpg", "jpeg", "png");
                    if(in_array($fileExtension, $imageExtension)){
                        $fileType = "image";
                    }
                    $pdfExtension = array("pdf");
                    if(in_array($fileExtension, $pdfExtension)){
                        $fileType = "pdf";
                    }
                    $documentFile = new ProviderDocumentFile();
                    $documentFile->provider_liability_insurance_id = $liabilityInsurance->id;
                    $documentFile->file = $imageName;
                    $documentFile->type = $fileType;
                    $documentFile->document_type = "liability_insurance";
                    $documentFile->save();
                }
            }

            DB::commit();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROFILE_UPDATED')]);
            }
            catch (Exception $e) {

                DB::rollBack();
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
            }
            catch (Throwable $e) {

                DB::rollBack();
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
            }
    }


    public function updateProfessionalReference(Request $request, $id)
    {
        try {
            $input = $request->all();
            //return response()->json(['data' => $input]);
            $professionalReference = ProviderProfessionalReference::updateOrCreate([
                'professional_reference'   => $input['professional_reference'],
                'provider_id'   => $id,
            ],[
                'provider_id'   => $id,
                'first_name'    => encryptString($input['first_name']),
                'last_name'     => encryptString($input['last_name']),
                'phone'         => encryptString($input['phone']),
                'email'         => encryptString($input['email']),
                'relation'      => encryptString($input['relation']),
                'professional_reference'  => $input['professional_reference']
            ]);
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROFILE_UPDATED')]);
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function updateAdditionalCertificate(Request $request, $id)
    {
        has_scope('provider.write');
        try {
            //dd($request->all());
            DB::beginTransaction();
            $input = $request->all();
            if(!empty($input['certificate_data']))
            {
                $certificationId = [];
                foreach($input['certificate_data'] as $key => $certification){
                    $certificationId[] = $certification['certification_id'];
                }
                if(!empty($certificationId)){  
                    ProviderCertification::where('provider_id', $id)->whereNotIn('id', $certificationId)->delete();  
                }
                foreach($input['certificate_data'] as $key => $certification)
                {
                    if(empty($certification['certification_name'])){
                        $errors[$key] = "This field is required";
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $errors]);
                    }
                    /* update the certification */
                    if(!empty($certification['certification_id'])){
                        $providerCertification = ProviderCertification::where('id', $certification['certification_id'])->first();
                        $providerCertification->provider_id = $id;
                        $providerCertification->certification_name = encryptString($certification['certification_name']);
                        $providerCertification->save();
                        // In case if provider choose new file
                        if(!empty($certification['certification_files']))
                        {
                            foreach($certification['certification_files'] as $file)
                            {
                                $imageName = rand(10000, 9999999) . '-' . time() . '.' .$file->getClientOriginalExtension();
                                $directoryName = "providerAdditionalCertifications";
                                uploadFile($file,$directoryName,$imageName);
                                $fileExtension = substr(strrchr($imageName,'.'),1);
                                $imageExtension = array("jpg", "jpeg", "png");
                                if(in_array($fileExtension, $imageExtension)){
                                    $fileType = "image";
                                }
                                $pdfExtension = array("pdf");
                                if(in_array($fileExtension, $pdfExtension)){
                                    $fileType = "pdf";
                                }
                                $providerDocumentFile = new ProviderDocumentFile();
                                $providerDocumentFile->provider_certification_id = $providerCertification->id;
                                $providerDocumentFile->file = $imageName;
                                $providerDocumentFile->type = $fileType;
                                $providerDocumentFile->document_type = "certification";
                                $providerDocumentFile->save();
                            }
                        }
                    }
                    /* end update the license */
                    else
                    {
                        $providerCertification = new ProviderCertification();
                        $providerCertification->provider_id = $id;
                        $providerCertification->certification_name = encryptString($certification['certification_name']);
                        $providerCertification->save();

                        if(!empty($certification['certification_files']))
                        {
                            foreach($certification['certification_files'] as $file)
                            {
                                $imageName = rand(10000, 9999999) . '-' . time() . '.' .$file->getClientOriginalExtension();
                                $directoryName = "providerAdditionalCertifications";
                                uploadFile($file,$directoryName,$imageName);
                                $fileExtension = substr(strrchr($imageName,'.'),1);
                                $imageExtension = array("jpg", "jpeg", "png");
                                if(in_array($fileExtension, $imageExtension)){
                                    $fileType = "image";
                                }
                                $pdfExtension = array("pdf");
                                if(in_array($fileExtension, $pdfExtension)){
                                    $fileType = "pdf";
                                }
                                $providerDocumentFile = new ProviderDocumentFile();
                                $providerDocumentFile->provider_certification_id = $providerCertification->id;
                                $providerDocumentFile->file = $imageName;
                                $providerDocumentFile->type = $fileType;
                                $providerDocumentFile->document_type = "certification";
                                $providerDocumentFile->save();
                            }
                        }
                    }
                }
                DB::commit();
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROFILE_UPDATED')]);
            }
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function updateLicense(Request $request, $id)
    {
        has_scope('provider.write');
        try {
            //dd($request->all());
            DB::beginTransaction();
            $input = $request->all();
            if(!empty($input['license_data']))
            {
                $licenseId = [];
                foreach($input['license_data'] as $key => $license){
                    $licenseId[] = $license['license_id'];
                }
                if(!empty($licenseId)){    
                    ProviderLicense::where('provider_id', $id)->whereNotIn('id', $licenseId)->delete();
                }
                foreach($input['license_data'] as $key => $license)
                {
                    if(empty($license['license_title']))
                    {
                        $errors[$key] = "This field is required.";
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $errors,'input'=>'license_title']);
                    }
                    if(empty($license['license_number']))
                    {
                        $errors[$key] = "This field is required.";
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $errors,'input'=>'license_number']);
                    }
                    if(empty($license['license_board']))
                    {
                        $errors[$key] = "This field is required.";
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $errors,'input'=>'license_board']);
                    }
                    if(empty($license['issued_date']))
                    {
                        $errors[$key] = "This field is required.";
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $errors,'input'=>'issued_date']);
                    }
                    if(empty($license['expiration_date']))
                    {
                        $errors[$key] = "This field is required.";
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $errors,'input'=>'expiration_date']);
                    }
                    if(dateFormat($license['issued_date']) > date("Y-m-d"))
                    {
                        $errors[$key] = "Issue date should be less than or equal to current date.";
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $errors,'input'=>'issued_date']);
                    }
                    if(dateFormat($license['expiration_date']) < date("Y-m-d"))
                    {
                        $errors[$key] = "Expiration date should be greater than current date.";
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $errors,'input'=>'expiration_date']);
                    }
                    if(empty($license['state']))
                    {
                        $errors[$key] = "This field is required.";
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $errors,'input'=>'state']);
                    }

                    /* update the license */
                    if(!empty($license['license_id'])){
                        $providerLicense = ProviderLicense::where('id', $license['license_id'])->first();
                        $providerLicense->provider_id    = $id;
                        $providerLicense->license_title  = encryptString($license['license_title']);
                        $providerLicense->license_number = encryptString($license['license_number']);
                        $providerLicense->license_board  = encryptString($license['license_board']);
                        $providerLicense->issued_date    = dateFormat($license['issued_date']);
                        $providerLicense->expiration_date = dateFormat($license['expiration_date']);
                        $providerLicense->state = $license['state'];
                        $providerLicense->save();
                        // In case if provider choose new file
                        if(!empty($license['license_files']))
                        {
                            foreach($license['license_files'] as $file)
                            {
                                $imageName = rand(10000, 9999999) . '-' . time() . '.' .$file->getClientOriginalExtension();
                                $directoryName = "providerLicenseFiles";
                                uploadFile($file,$directoryName,$imageName);
                                $fileExtension = substr(strrchr($imageName,'.'),1);
                                $imageExtension = array("jpg", "jpeg", "png");
                                if(in_array($fileExtension, $imageExtension)){
                                    $fileType = "image";
                                }
                                $pdfExtension = array("pdf");
                                if(in_array($fileExtension, $pdfExtension)){
                                    $fileType = "pdf";
                                }
                                $providerDocumentFile = new ProviderDocumentFile();
                                $providerDocumentFile->provider_license_id = $providerLicense->id;
                                $providerDocumentFile->file = $imageName;
                                $providerDocumentFile->type = $fileType;
                                $providerDocumentFile->document_type = "license";
                                $providerDocumentFile->save();
                            }
                        }
                    }
                    /* end update the license */
                    else{
                        $providerLicense = new ProviderLicense();
                        $providerLicense->provider_id    = $id;
                        $providerLicense->license_title  = encryptString($license['license_title']);
                        $providerLicense->license_number = encryptString($license['license_number']);
                        $providerLicense->license_board  = encryptString($license['license_board']);
                        $providerLicense->issued_date    = dateFormat($license['issued_date']);
                        $providerLicense->expiration_date = dateFormat($license['expiration_date']);
                        $providerLicense->state = $license['state'];
                        $providerLicense->save();

                        if(!empty($license['license_files']))
                        {
                            foreach($license['license_files'] as $file)
                            {
                                $imageName = rand(10000, 9999999) . '-' . time() . '.' .$file->getClientOriginalExtension();
                                $directoryName = "providerLicenseFiles";
                                uploadFile($file,$directoryName,$imageName);
                                $fileExtension = substr(strrchr($imageName,'.'),1);
                                $imageExtension = array("jpg", "jpeg", "png");
                                if(in_array($fileExtension, $imageExtension)){
                                    $fileType = "image";
                                }
                                $pdfExtension = array("pdf");
                                if(in_array($fileExtension, $pdfExtension)){
                                    $fileType = "pdf";
                                }
                                $providerDocumentFile = new ProviderDocumentFile();
                                $providerDocumentFile->provider_license_id = $providerLicense->id;
                                $providerDocumentFile->file = $imageName;
                                $providerDocumentFile->type = $fileType;
                                $providerDocumentFile->document_type = "license";
                                $providerDocumentFile->save();
                            }
                        }
                    }

                }
            }
            DB::commit();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROFILE_UPDATED')]);
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    // dashboard
    public function providerCount()
    {
        try {

            $totalProvider = Provider::where([['register_status','=','completed'],['register_step','=',5],['application_status','=','Approved']])->count();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_COUNT'), 'totalProvider' => $totalProvider]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function verifyTwoFactor(Request $request)
    {
        $this->validate($request, [
            'email' => 'required',
        ]);

        try{
            $input    = $request->all();
            $twoFactorCode = $input['two_factor_code'];
            $email  = $input['email'];
            if(!empty($email) && !empty($twoFactorCode))
            {
                $provider = Provider::where(['email' => $email,'two_factor_code' => $twoFactorCode])
                ->first();
                if($provider)
                {
                    $codeTime = $provider->two_factor_expires_at;
                    $validityTime = strtotime(date("Y-m-d H:i:s"));
                    if(($validityTime - $codeTime) > config('service.PROVIDER_API.PROVIDER_OTP_EXPIRE_TIME'))
                    {
                        $provider->resetTwoFactorCode();
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.OTP_EXPIRE')]);
                    }
                    else
                    {
                        $provider->is_blocked = "no";
                        $provider->login_attempt = null;
                        $provider->login_attempt_time = null;
                        $provider->save();
                        $provider->resetTwoFactorCode();

                        $response =  (new OauthServerController)->handleAccessTokenBruteForce();
                        unset($response['scope']);

                        $response['first_name']   =  $provider->first_name;
                        $response['last_name']    =  $provider->last_name;
                        $response['birth_date']   =  $provider->birth_date;
                        $response['gender']   =  $provider->gender;
                        $response['phone']   =  $provider->phone;
                        $response['email']   =  $provider->email;
                        $response['experience']   =  $provider->experience;
                        $response['bio']   =  $provider->bio;
                        $response['register_status'] =  $provider->register_status;
                        $response['register_step']   =  $provider->register_step;
                        $response['email_confirmed']   =  $provider->email_confirmed;
                        $response['application_status']   =  $provider->application_status;
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_LOGIN'), 'records' => $response]);
                    }
                }
                else
                {
                    return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.WRONG_OTP')]);

                }
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function providerRejectionReason(Request $request) 
    {

        try {
            $provider = Provider::select('rejection_reason','rejection_date')->where(['id' => $this->user_id,'application_status'=>'Rejected'])->first();
            if($provider){
                return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.REJECTION_REASON'), 'records' => $provider]);
            }
            else{
                return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.REASON_NOT_FOUND')]);
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    /**
    * @ Register user api.
    * @return \Illuminate\Http\Response
    */
    public function providerRegisterStep1(Request $request)
    {
        // print_r(date('Y-m-d h:i:s'));
        // exit;
        //has_scope('customers.write');
        $message = array(
            'password.regex' => config('service.PROVIDER_API.MESSAGE.PASSWORD_REGEX'),
            'birth_date.before_or_equal' => config('service.PROVIDER_API.MESSAGE.BIRTH_DATE')
        );
        $validator = Validator::make($request->all(), [
            'first_name' => 'required|max:1000',
            'last_name' => 'required|max:1000',
            'birth_date' => 'required|date_format:m/d/Y|before_or_equal:'.\Carbon\Carbon::now()->subYears(18)->format('m/d/Y'),
            'gender' => 'required',
            'experience' => 'required|max:100',
            'bio' => 'required|max:500',
            //'email' => 'required|email|max:100|unique:oauth_users',
            'email' => 'required|max:100|unique:oauth_users',
            'phone_code' => 'required|max:5',
            //'phone' => 'required|regex:/^([0-9\s\-\+\(\)]*)$/|min:10|max:10|unique:oauth_users,mobile',
            'phone' => 'required|unique:oauth_users,mobile',
            'address' => 'required|max:2000',
            'password' => [
                'required',
            'min:8', // must be at least 8 characters in length
            'regex:/[a-z]/', // must contain at least one lowercase letter
            'regex:/[A-Z]/', // must contain at least one uppercase letter
            'regex:/[0-9]/', // must contain at least one digit
            'regex:/[@$!%*#?&]/', // must contain a special character
            ],
            'confirm_password' => 'required|same:password',
            ], $message);

            if($validator->fails()){
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $validator->errors()->first()],200);
            }

        try {

            DB::beginTransaction();
            $input = $request->all();
            /*if(!empty($input['image']))
            {
            $fileExtension = substr(strrchr($input['image'],'.'),1);
            $allExtension = array("jpg", "jpeg", "png");
            if(!in_array($fileExtension, $allExtension))
            {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROFILE_FILE_EXTENSION')]);
            }
            }*/
            $input['register_step'] = '1';
            $input['register_status'] = 'in_progress';
            $input['email_confirmed'] = 'no';
            $input['created_at'] = date('Y-m-d h:i:s');
            $input['updated_at'] = date('Y-m-d h:i:s');
            $input['password'] = $this->hashPassword($request->password);
            $provider =  Provider::create($input);
            $provider->created_at=convertcurrenttimetoUtc(date('Y-m-d H:i:s'));
           
            if($provider)
            {
                $oauth = new $this->oauth_user;
                $oauth->user_fk  = $provider->id;
                $oauth->email    = $request->email;
                $oauth->username = $request->email;
                $oauth->mobile   = $request->phone;
                $oauth->password = $input['password'];
                $oauth->created_at=convertcurrenttimetoUtc(date('Y-m-d H:i:s'));
                $oauth->save();

                /* user password history */
                   $userPasswordHistory = new UserPasswordHistory();
                   $userPasswordHistory->user_id = $provider->id;
                   $userPasswordHistory->email = $provider->email;
                   $userPasswordHistory->password = $this->hashPassword($request->password);
                   $userPasswordHistory->save();
                /* user password history  */
                DB::commit();
                $response =  (new OauthServerController)->handleAccessTokenBruteForce();
                unset($response['scope']);
                $response['image']   =  $provider->image;
                $response['first_name']   =  $provider->first_name;
                $response['last_name']    =  $provider->last_name;
                $response['birth_date']   =  $provider->birth_date;
                $response['gender']   =  $provider->gender;
                $response['other_gender']   =  $provider->other_gender;
                $response['phone']   =  $provider->phone;
                $response['email']   =  $provider->email;
                $response['experience']   =  $provider->experience;
                $response['bio']   =  $provider->bio;
                $response['register_status'] =  $provider->register_status;
                $response['register_step']   =  $provider->register_step;
                $response['email_confirmed']   =  $provider->email_confirmed;
                $response['application_status']   =  $provider->application_status;
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_REGISTER_STEP_1'), 'records' => $response]);
            }

        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    private function hashPassword($password)
    {
        return sha1($password);
    }


    public function providerRegisterStep2(Request $request)
    {
        //has_scope('customers.write');
        $validator = Validator::make($request->all(), [
            'licenses'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $validator->errors()->first()],200);
        }

        try {
            DB::beginTransaction();
            $input = $request->all();
            if(!empty($input['licenses']))
            {
            //$delete = ProviderLicense::where('provider_id', $this->user_id)->delete();
                foreach($input['licenses'] as $license)
                {
                    if(strtotime($license['issued_date']) > strtotime(currentDate()))
                    {
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.ISSUED_DATE')]);
                    }
                    if(strtotime($license['expiration_date']) < strtotime(currentDate()))
                    {
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.EXPIRATION_DATE')]);
                    }
                    if(!empty($license['files'])){
                        foreach($license['files'] as $file){
                            $fileExtension = substr(strrchr($file,'.'),1);
                            $allExtension = array("jpg", "jpeg", "png", "pdf");
                            if(!in_array($fileExtension, $allExtension)){
                                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.FILE_EXTENSION')]);
                            }
                        }
                    }
                    /* update the license */
                    if(!empty($license['id'])){
                        $providerLicense = ProviderLicense::where('id', $license['id'])->first();
                        $providerLicense->provider_id    = $this->user_id;
                        $providerLicense->license_title  = $license['license_title'];
                        $providerLicense->license_number = $license['license_number'];
                        $providerLicense->license_board  = $license['license_board'];
                        $providerLicense->state  = $license['state'];
                        $providerLicense->issued_date    = dateFormat($license['issued_date']);
                        $providerLicense->expiration_date = dateFormat($license['expiration_date']);
                        $providerLicense->save();
                        // In case if provider choose new file
                        if(!empty($license['files']))
                        {
                            foreach($license['files'] as $file)
                            {
                                $fileExtension = substr(strrchr($file,'.'),1);
                                $imageExtension = array("jpg", "jpeg", "png");
                                if(in_array($fileExtension, $imageExtension)){
                                    $fileType = "image";
                                }
                                $pdfExtension = array("pdf");
                                if(in_array($fileExtension, $pdfExtension)){
                                    $fileType = "pdf";
                                }
                                $providerDocumentFile = new ProviderDocumentFile();
                                $providerDocumentFile->provider_license_id = $providerLicense->id;
                                $providerDocumentFile->file = $file;
                                $providerDocumentFile->type = $fileType;
                                $providerDocumentFile->document_type = "license";
                                $providerDocumentFile->save();
                            }
                        }
                    }
                    /* end update the license */
                    else{
                        $providerLicense = new ProviderLicense();
                        $providerLicense->provider_id    = $this->user_id;
                        $providerLicense->license_title  = $license['license_title'];
                        $providerLicense->license_number = $license['license_number'];
                        $providerLicense->license_board  = $license['license_board'];
                        $providerLicense->state  = $license['state'];
                        $providerLicense->issued_date    = dateFormat($license['issued_date']);
                        $providerLicense->expiration_date = dateFormat($license['expiration_date']);
                        $providerLicense->save();

                        if(!empty($license['files']))
                        {
                            foreach($license['files'] as $file)
                            {
                                $fileExtension = substr(strrchr($file,'.'),1);
                                $imageExtension = array("jpg", "jpeg", "png");
                                if(in_array($fileExtension, $imageExtension)){
                                    $fileType = "image";
                                }
                                $pdfExtension = array("pdf");
                                if(in_array($fileExtension, $pdfExtension)){
                                    $fileType = "pdf";
                                }
                                $providerDocumentFile = new ProviderDocumentFile();
                                $providerDocumentFile->provider_license_id = $providerLicense->id;
                                $providerDocumentFile->file = $file;
                                $providerDocumentFile->type = $fileType;
                                $providerDocumentFile->document_type = "license";
                                $providerDocumentFile->save();
                            }
                        }
                    }
                }
            }
            if($input['register_action'] == 'create'){
                $provider = Provider::find($this->user_id);
                $provider->register_step = '2';
                $provider->save();
            }
            DB::commit();
            $data = ProviderLicense::with(['licenseFiles'=>function($query) {
                $query->select('id','provider_license_id','file','type');
            }])
            ->where(['provider_id'=>$this->user_id])
            ->get();
            if($input['register_action'] == 'create'){
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_REGISTER_STEP_2'), 'records' => $data]);
            }
            else{
                /* Push and db notification */
                $notificationSetting = NotificationSetting::where('provider_id',$this->user_id)->first();
                if($notificationSetting->show_notification == 'active'){
                    (new ProviderNotificationController)->profileUpdate($this->user_id);
                }
                /* End push and db notification */
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_REGISTER_STEP_2_UPDATE'), 'records' => $data]);
            }
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function providerDeleteLicense(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'type'  => 'required',
            'id'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $validator->errors()->first()],200);
        }
        try {
            if($request->type=="license"){
                $providerLicense = ProviderLicense::findOrFail($request->id);
                $providerLicense->delete();
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_LICENSE_DELETED')]);
            }
            if($request->type=="license_file"){
                $providerDocumentFile = ProviderDocumentFile::findOrFail($request->id);
                $providerDocumentFile->delete();
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_LICENSE_FILE_DELETED')]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function providerRegisterStep3(Request $request)
    {
        /*$validator = Validator::make($request->all(), [
            'certifications'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }*/
        try {
            DB::beginTransaction();
            $input = $request->all();
            if(!empty($input['certifications']))
            {
            //$delete = ProviderCertification::where('provider_id', $this->user_id)->delete();
                foreach($input['certifications'] as $certification)
                {
                    if(!empty($certification['files'])){
                        foreach($certification['files'] as $file){
                            $fileExtension = substr(strrchr($file,'.'),1);
                            $allExtension = array("jpg", "jpeg", "png", "pdf");
                            if(!in_array($fileExtension, $allExtension)){
                                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.FILE_EXTENSION')]);
                            }
                        }
                    }
                    /* update the certification */
                    if(!empty($certification['id'])){
                        $providerCertification = ProviderCertification::where('id', $certification['id'])->first();
                        $providerCertification->provider_id = $this->user_id;
                        $providerCertification->certification_name = $certification['certification_name'];
                        $providerCertification->save();
                        // In case if provider choose new file
                        if(!empty($certification['files']))
                        {
                            foreach($certification['files'] as $file)
                            {
                                $fileExtension = substr(strrchr($file,'.'),1);
                                $imageExtension = array("jpg", "jpeg", "png");
                                if(in_array($fileExtension, $imageExtension)){
                                    $fileType = "image";
                                }
                                $pdfExtension = array("pdf");
                                if(in_array($fileExtension, $pdfExtension)){
                                    $fileType = "pdf";
                                }
                                $providerDocumentFile = new ProviderDocumentFile();
                                $providerDocumentFile->provider_certification_id = $providerCertification->id;
                                $providerDocumentFile->file = $file;
                                $providerDocumentFile->type = $fileType;
                                $providerDocumentFile->document_type = "certification";
                                $providerDocumentFile->save();
                            }
                        }
                    }
                    /* end update the license */
                    else{
                        $providerCertification = new ProviderCertification();
                        $providerCertification->provider_id = $this->user_id;
                        $providerCertification->certification_name = $certification['certification_name'];
                        $providerCertification->save();

                        if(!empty($certification['files']))
                        {
                            foreach($certification['files'] as $file)
                            {
                                $fileExtension = substr(strrchr($file,'.'),1);
                                $imageExtension = array("jpg", "jpeg", "png");
                                if(in_array($fileExtension, $imageExtension)){
                                    $fileType = "image";
                                }
                                $pdfExtension = array("pdf");
                                if(in_array($fileExtension, $pdfExtension)){
                                    $fileType = "pdf";
                                }
                                $providerDocumentFile = new ProviderDocumentFile();
                                $providerDocumentFile->provider_certification_id = $providerCertification->id;
                                $providerDocumentFile->file = $file;
                                $providerDocumentFile->type = $fileType;
                                $providerDocumentFile->document_type = "certification";
                                $providerDocumentFile->save();
                            }
                        }
                    }
                }
                if($input['register_action'] == 'create'){
                    $provider = Provider::find($this->user_id);
                    $provider->register_step = '3';
                    $provider->save();
                }
                DB::commit();
                $data = ProviderCertification::with(['certificationFiles'=>function($query) {
                    $query->select('provider_certification_id','file','type');
                }])
                ->where(['provider_id'=>$this->user_id])
                ->get();
                if($input['register_action'] == 'create'){
                    return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_REGISTER_STEP_3'),'records' => $data]);
                }else{
                    /* Push and db notification */
                    $notificationSetting = NotificationSetting::where('provider_id',$this->user_id)->first();
                    if($notificationSetting->show_notification == 'active'){
                        (new ProviderNotificationController)->profileUpdate($this->user_id);
                    }
                    /* End push and db notification */
                    return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_REGISTER_STEP_3_UPDATE'),'records' => $data]);
                }
            }
            else{
                $provider = Provider::find($this->user_id);
                $provider->register_step = '3';
                $provider->save();
            }

        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function providerDeleteCertificate(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'type'  => 'required',
            'id'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $validator->errors()->first()],200);
        }
        try {
            if($request->type=="certificate"){
                $providerCertification = ProviderCertification::findOrFail($request->id);
                $providerCertification->delete();
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_CERTIFICATE_DELETED')]);
            }
            if($request->type=="certificate_file"){
                $providerDocumentFile = ProviderDocumentFile::findOrFail($request->id);
                $providerDocumentFile->delete();
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_CERTIFICATE_FILE_DELETED')]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }


    public function providerRegisterStep4(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'fbi_background_clearance_date'     => 'required',
           // 'child_abuse_clearance_date'        => 'required',
            'liability_insurance_issued_date'   => 'required',
            'liability_insurance_expiration_date'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try {
            DB::beginTransaction();
            $input = $request->all();
            /* Fbi background check  */
            if(strtotime($input['fbi_background_clearance_date']) > strtotime(currentDate())){
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.FBI_BACKGROUND_CLEARANCE_DATE')]);
            }
            if(!empty($input['fbi_background_files'])){
                foreach($input['fbi_background_files'] as $file){
                    $fileExtension = substr(strrchr($file,'.'),1);
                    $allExtension = array("jpg", "jpeg", "png", "pdf");
                    if(!in_array($fileExtension, $allExtension)){
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.FBI_BACKGROUND_FILE_EXTENSION')]);
                    }
                }
            }
            $this->providerFbiBackground($request);
            /* End fbi background check */

            /* Child abuse history */
            // if(strtotime($input['child_abuse_clearance_date']) > strtotime(currentDate())){
            //     return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.CHILD_ABUSE_CLEARANCE_DATE')]);
            // }
            // if(!empty($input['child_abuse_files'])){
            //     foreach($input['child_abuse_files'] as $file){
            //         $fileExtension = substr(strrchr($file,'.'),1);
            //         $allExtension = array("jpg", "jpeg", "png", "pdf");
            //         if(!in_array($fileExtension, $allExtension)){
            //             return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.CHILD_ABUSE_FILE_EXTENSION')]);
            //         }
            //     }
            // }
            // $this->providerChildAbuseHistory($request);
            /* End child abuse history */

            /* Liability insurance */
            if(strtotime($input['liability_insurance_issued_date']) > strtotime(currentDate())){
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.LIABILITY_INSURANCE_ISSUED_DATE')]);
            }
            if(strtotime($input['liability_insurance_expiration_date']) < strtotime(currentDate())){
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.LIABILITY_INSURANCE_EXPIRATION_DATE')]);
            }
            if(!empty($input['liability_insurance_files'])){
                foreach($input['liability_insurance_files'] as $file){
                    $fileExtension = substr(strrchr($file,'.'),1);
                    $allExtension = array("jpg", "jpeg", "png", "pdf");
                    if(!in_array($fileExtension, $allExtension)){
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.LIABILITY_INSURANCE_FILE_EXTENSION')]);
                    }
                }
            }
            $this->providerLiabilityInsurance($request);
            /* End liability insurance */

            if($input['register_action'] == 'create'){
                $provider = Provider::find($this->user_id);
                $provider->register_step = '4';
                $provider->save();
            }
            DB::commit();
            $data['fbi_background'] = ProviderFbiBackground::with(['fbiBackgroundFiles'=>function($query) {
                $query->select('provider_fbi_background_id','file','type');
            }])->where(['provider_id'=>$this->user_id])->get();
            $data['child_abuse_history'] = ProviderChildAbuseHistory::with(['childAbuseFiles'=>function($query) {
                $query->select('provider_child_abuse_history_id','file','type');
            }])->where(['provider_id'=>$this->user_id])->get();
            $data['liability_insurance'] = ProviderLiabilityInsurance::with(['liabilityInsuranceFiles'=>function($query) {
                $query->select('provider_liability_insurance_id','file','type');
            }])->where(['provider_id'=>$this->user_id])->get();

            if($input['register_action'] == 'create'){
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_REGISTER_STEP_4'),'records' => $data]);
            }else{
                /* Push and db notification */
                $notificationSetting = NotificationSetting::where('provider_id',$this->user_id)->first();
                if($notificationSetting->show_notification == 'active'){
                    (new ProviderNotificationController)->profileUpdate($this->user_id);
                }
                /* End push and db notification */
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_REGISTER_STEP_4_UPDATE'),'records' => $data]);
            }
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function providerFbiBackground($request)
    {
        $input = $request->all();
        //$delete = ProviderFbiBackground::where('provider_id', $this->user_id)->delete();
        if(!empty($input['fbi_background_id']))
        {
            $fbiBackground = ProviderFbiBackground::where('id', $input['fbi_background_id'])->first();
            $fbiBackground->provider_id = $this->user_id;
            $fbiBackground->clearance_date = dateFormat($input['fbi_background_clearance_date']);
            $fbiBackground->save();
            if(!empty($input['fbi_background_files']))
            {
                foreach($input['fbi_background_files'] as $file)
                {
                    $fileExtension = substr(strrchr($file,'.'),1);
                    $imageExtension = array("jpg", "jpeg", "png");
                    if(in_array($fileExtension, $imageExtension)){
                        $fileType = "image";
                    }
                    $pdfExtension = array("pdf");
                    if(in_array($fileExtension, $pdfExtension)){
                        $fileType = "pdf";
                    }
                    $documentFile = new ProviderDocumentFile();
                    $documentFile->provider_fbi_background_id = $fbiBackground->id;
                    $documentFile->file = $file;
                    $documentFile->type = $fileType;
                    $documentFile->document_type = "fbi_background";
                    $documentFile->save();
                }
            }
        }
        else
        {
            $fbiBackground = new ProviderFbiBackground();
            $fbiBackground->provider_id = $this->user_id;
            $fbiBackground->clearance_date = dateFormat($input['fbi_background_clearance_date']);
            $fbiBackground->save();

            if(!empty($input['fbi_background_files']))
            {
                foreach($input['fbi_background_files'] as $file)
                {
                    $fileExtension = substr(strrchr($file,'.'),1);
                    $imageExtension = array("jpg", "jpeg", "png");
                    if(in_array($fileExtension, $imageExtension)){
                        $fileType = "image";
                    }
                    $pdfExtension = array("pdf");
                    if(in_array($fileExtension, $pdfExtension)){
                        $fileType = "pdf";
                    }
                    $documentFile = new ProviderDocumentFile();
                    $documentFile->provider_fbi_background_id = $fbiBackground->id;
                    $documentFile->file = $file;
                    $documentFile->type = $fileType;
                    $documentFile->document_type = "fbi_background";
                    $documentFile->save();
                }
            }
        }
    }


    public function providerChildAbuseHistory($request)
    {
        $input = $request->all();
        //$delete = ProviderChildAbuseHistory::where('provider_id', $this->user_id)->delete();
        if(!empty($input['child_abuse_history_id']))
        {
            $childAbuseHistory = ProviderChildAbuseHistory::where('id', $input['child_abuse_history_id'])->first();
            $childAbuseHistory->provider_id = $this->user_id;
            $childAbuseHistory->clearance_date = dateFormat($input['child_abuse_clearance_date']);
            $childAbuseHistory->save();
            if(!empty($input['child_abuse_files']))
            {
                foreach($input['child_abuse_files'] as $file)
                {
                    $fileExtension = substr(strrchr($file,'.'),1);
                    $imageExtension = array("jpg", "jpeg", "png");
                    if(in_array($fileExtension, $imageExtension)){
                        $fileType = "image";
                    }
                    $pdfExtension = array("pdf");
                    if(in_array($fileExtension, $pdfExtension)){
                        $fileType = "pdf";
                    }
                    $documentFile = new ProviderDocumentFile();
                    $documentFile->provider_child_abuse_history_id = $childAbuseHistory->id;
                    $documentFile->file = $file;
                    $documentFile->type = $fileType;
                    $documentFile->document_type = "child_abuse";
                    $documentFile->save();
                }
            }
        }
        else
        {
            $childAbuseHistory = new ProviderChildAbuseHistory();
            $childAbuseHistory->provider_id = $this->user_id;
            $childAbuseHistory->clearance_date = dateFormat($input['child_abuse_clearance_date']);
            $childAbuseHistory->save();

            if(!empty($input['child_abuse_files']))
            {
                foreach($input['child_abuse_files'] as $file)
                {
                    $fileExtension = substr(strrchr($file,'.'),1);
                    $imageExtension = array("jpg", "jpeg", "png");
                    if(in_array($fileExtension, $imageExtension)){
                        $fileType = "image";
                    }
                    $pdfExtension = array("pdf");
                    if(in_array($fileExtension, $pdfExtension)){
                        $fileType = "pdf";
                    }
                    $documentFile = new ProviderDocumentFile();
                    $documentFile->provider_child_abuse_history_id = $childAbuseHistory->id;
                    $documentFile->file = $file;
                    $documentFile->type = $fileType;
                    $documentFile->document_type = "child_abuse";
                    $documentFile->save();
                }
            }
        }
    }


    public function providerLiabilityInsurance($request)
    {
        $input = $request->all();
        //$delete = ProviderLiabilityInsurance::where('provider_id', $this->user_id)->delete();
        if(!empty($input['liability_insurance_id']))
        {
            $liabilityInsurance = ProviderLiabilityInsurance::where('id', $input['liability_insurance_id'])->first();
            $liabilityInsurance->provider_id = $this->user_id;
            $liabilityInsurance->issued_date = dateFormat($input['liability_insurance_issued_date']);
            $liabilityInsurance->expiration_date = dateFormat($input['liability_insurance_expiration_date']);
            $liabilityInsurance->save();

            if(!empty($input['liability_insurance_files']))
            {
                foreach($input['liability_insurance_files'] as $file)
                {
                    $fileExtension = substr(strrchr($file,'.'),1);
                    $imageExtension = array("jpg", "jpeg", "png");
                    if(in_array($fileExtension, $imageExtension)){
                        $fileType = "image";
                    }
                    $pdfExtension = array("pdf");
                    if(in_array($fileExtension, $pdfExtension)){
                        $fileType = "pdf";
                    }
                    $documentFile = new ProviderDocumentFile();
                    $documentFile->provider_liability_insurance_id = $liabilityInsurance->id;
                    $documentFile->file = $file;
                    $documentFile->type = $fileType;
                    $documentFile->document_type = "liability_insurance";
                    $documentFile->save();
                }
            }
        }
        else
        {
            $liabilityInsurance = new ProviderLiabilityInsurance();
            $liabilityInsurance->provider_id = $this->user_id;
            $liabilityInsurance->issued_date = dateFormat($input['liability_insurance_issued_date']);
            $liabilityInsurance->expiration_date = dateFormat($input['liability_insurance_expiration_date']);
            $liabilityInsurance->save();

            if(!empty($input['liability_insurance_files']))
            {
                foreach($input['liability_insurance_files'] as $file)
                {
                    $fileExtension = substr(strrchr($file,'.'),1);
                    $imageExtension = array("jpg", "jpeg", "png");
                    if(in_array($fileExtension, $imageExtension)){
                        $fileType = "image";
                    }
                    $pdfExtension = array("pdf");
                    if(in_array($fileExtension, $pdfExtension)){
                        $fileType = "pdf";
                    }
                    $documentFile = new ProviderDocumentFile();
                    $documentFile->provider_liability_insurance_id = $liabilityInsurance->id;
                    $documentFile->file = $file;
                    $documentFile->type = $fileType;
                    $documentFile->document_type = "liability_insurance";
                    $documentFile->save();
                }
            }
        }
    }


    public function providerDeleteVerificationDoc(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'type'  => 'required',
            'id'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $validator->errors()->first()],200);
        }
        try {
            if($request->type=="fbi_background_file"){
                $providerDocumentFile = ProviderDocumentFile::findOrFail($request->id);
                $providerDocumentFile->delete();
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_FBI_BACKGROUND_FILE_DELETED')]);
            }
            if($request->type=="child_abuse_file"){
                $providerDocumentFile = ProviderDocumentFile::findOrFail($request->id);
                $providerDocumentFile->delete();
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_CHILD_ABUSE_FILE_DELETED')]);
            }
            if($request->type=="liability_insurance_file"){
                $providerDocumentFile = ProviderDocumentFile::findOrFail($request->id);
                $providerDocumentFile->delete();
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_LIABILITY_INSURANCE_FILE_DELETED')]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }


    public function providerProfessionalReferences(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'first_name'  => 'required|max:191',
            'last_name'  => 'required|max:191',
            'phone_code'  => 'required',
            //'phone'  => 'required|regex:/^([0-9\s\-\+\(\)]*)$/|min:10|max:10',
            'phone'  => 'required',
            //'email'  => 'required|email|max:191',
            'email'  => 'required|max:500',
            'relation'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try {
            $input = $request->all();
            if(!empty($input['professional_reference']) && $input['professional_reference'])
            {
                $professionalReference = ProviderProfessionalReference::updateOrCreate([
                    'professional_reference'   => $input['professional_reference'],
                    'provider_id'   => $this->user_id,
                ],[
                    'provider_id'   => $this->user_id,
                    'first_name'    => $input['first_name'],
                    'last_name'     => $input['last_name'],
                    'phone_code'    => $input['phone_code'],
                    'phone'         => $input['phone'],
                    'email'         => $input['email'],
                    'relation'      => $input['relation'],
                    'professional_reference'        => $input['professional_reference']
                ]);
            }
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_PROFESSIONAL_REFERENCE'),'records' => $professionalReference]);
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function specializationList()
    {
        try {
            $locale = request()->header('locale');
            App::setLocale($locale);
            $specializations = Specialization::all();
            $data = [];
            foreach ($specializations as $key => $value) {
                $data[] = [
                    'id' => $value->id,
                    'title' => $value->title
                ];
            }
            if (empty($data)) {
                throw new Exception(config('service.SPECIALIZATION_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.SUCCESS_STATUS'), 'message' => config('service.SPECIALIZATION_LIST'), 'records' => $data]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function languageList()
    {
        try {
            $locale = request()->header('locale');
            App::setLocale($locale);
            $languages = Language::all();
            $data = [];
            foreach ($languages as $key => $value) {
                $data[] = [
                    'id' => $value->id,
                    'name' => $value->name
                ];
            }
            if (empty($data)) {
                throw new Exception(config('service.LANGUAGE_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.SUCCESS_STATUS'), 'message' => config('service.LANGUAGE_LIST'), 'records' => $data]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    
    public function StateList(Request $request)
    {
        try {

            if($request->country_id){
                $countryID = $request->country_id;
            }else{
                $countryID = 1;
            }


            $States = State::where('country_id', $countryID)->orderBy('name', 'ASC')
            ->get();
            // $States = State::all();
            // $States = State::all();
            $data = [];
            foreach ($States as $key => $value) {
                $data[] = [
                    'id' => $value->id,
                    'name' => $value->name,
                    'code' => $value->code
                ];
            }
            if (empty($data)) {
                throw new Exception(config('service.STATE_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.SUCCESS_STATUS'), 'message' => config('service.STATE_LIST'), 'records' => $data]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function providerRegisterStep5(Request $request)
    {
        $message = array(
            'specialization_id.required' => "Please select atleast one specialization",
            'language_id.required' => "Please select atleast one language"
        );
        $validator = Validator::make($request->all(), [
            'specialization_id'  => 'required',
            'language_id'  => 'required',
        ], $message);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try {
            DB::beginTransaction();
            $input = $request->all();

            if(!empty($input['specialization_id']))
            {
                $delete = ProviderSpecialization::where('provider_id', $this->user_id)->delete();
                foreach($input['specialization_id'] as $specializationId)
                {
                    $specialization = new ProviderSpecialization();
                    $specialization->provider_id = $this->user_id;
                    $specialization->specialization_id = $specializationId;
                    $specialization->save();
                }
            }
            if(!empty($input['language_id']))
            {
                $delete = ProviderLanguage::where('provider_id', $this->user_id)->delete();
                foreach($input['language_id'] as $languageId)
                {
                    $language = new ProviderLanguage();
                    $language->provider_id = $this->user_id;
                    $language->language_id = $languageId;
                    $language->save();
                }
            }

            if($input['register_action'] == 'create'){
                $token   = Str::random(60);
                $provider = Provider::find($this->user_id);
                $provider->other_specialization = !empty($input['other_specialization']) ? $input['other_specialization']:'';
                $provider->other_language = !empty($input['other_language']) ? $input['other_language']:'';
                $provider->register_step = '5';
                $provider->register_status = 'completed';
                $provider->email_confirm_token = $token;
                $provider->save();
                DB::commit();

                /* Notification setting */
                   $this->notificationSetting($provider->id);
                /* Notification setting  */

                $templateName = "Provider welcome email";
                $templateData = EmailTemplate::where('name',$templateName)->first();
                if($templateData)
                {
                    /* Welcome email */
                        $greeting = 'Hi ' . decryptString($provider->first_name)." ".decryptString($provider->last_name) . ',';
                        $body     = $templateData->text;
                        $type     = $templateData->title;
                        $sendMail = new ProviderWelcomeMailJob(decryptString($provider->email), $greeting, $body, $type);
                        $this->dispatchNow($sendMail);
                    /* End welcome email  */
                }

                $templateName = "Provider account verification";
                $templateData = EmailTemplate::where('name',$templateName)->first();
                if($templateData)
                {
                    /* Confirm email */
                    $greeting = 'Hi ' . decryptString($provider->first_name)." ".decryptString($provider->last_name);
                    $body     = $templateData->text;
                    $type     = $templateData->title;
                    $link     = env('FRONT_END') . '/' . 'admin/provider-confirm-email/'. $token;
                    $sendMail = new ProviderConfirmEmailJob(decryptString($provider->email), $greeting, $body, $link, $type);
                    $this->dispatchNow($sendMail);
                    /* End confirm email  */
                }

                $templateName = "Provider Request";
                $templateData = EmailTemplate::where('name',$templateName)->first();
                if($templateData)
                {
                    $admin = User::where(['id'=>1])->first();
                    /* New provider email to admin */
                    $greeting = 'Hi ' . $admin->first_name." ".$admin->last_name . ',';
                    $body     = $templateData->text;
                    $type     = $templateData->title;
                    $link     = env('FRONT_END') . '/' . 'admin/pending/providers/';
                    $sendMail = new ProviderRegisteredEmailJob($admin->email, $greeting, $body, $link, $type);
                    $this->dispatchNow($sendMail);
                    /* End email  */
                }

                /* Push and db notification */
                $notificationSetting = NotificationSetting::where('provider_id',$this->user_id)->first();
                if($notificationSetting->show_notification == 'active'){
                    (new ProviderNotificationController)->register($this->user_id);
                }
                /* End push and db notification */

            }
            if($input['register_action'] == 'update'){
                $provider = Provider::find($this->user_id);
                $provider->other_specialization = !empty($input['other_specialization']) ? $input['other_specialization']:'';
                $provider->other_language = !empty($input['other_language']) ? $input['other_language']:'';
                $provider->save();
                DB::commit();
            }
            $data['provider_specialization'] = ProviderSpecialization::with(['specialization'=>function($query) {
                $query->select('*');
            }])
            ->where(['provider_id'=>$this->user_id])
            ->get();
            $data['provider_language'] = ProviderLanguage::with(['language'=>function($query) {
                $query->select('*');
            }])
            ->where(['provider_id'=>$this->user_id])
            ->get();
            if($input['register_action'] == 'create'){
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_REGISTER_STEP_5'),'records' => $data]);
            }else{
                /* Push and db notification */
                $notificationSetting = NotificationSetting::where('provider_id',$this->user_id)->first();
                if($notificationSetting->show_notification == 'active'){
                    (new ProviderNotificationController)->profileUpdate($this->user_id);
                }
                /* End push and db notification */
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_REGISTER_STEP_5_UPDATE'),'records' => $data]);
            }
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    private function notificationSetting($providerId)
    {
        $updateSetting = NotificationSetting::updateOrCreate([
            'provider_id'   => $providerId,
        ],[
            'provider_id'  => $providerId,
            'show_notification' => 'active',
            'appointment_reminder' => 'active',
            'call_notification' => 'active',
            'new_message_notification' => 'active',
        ]);
        if($updateSetting){
            return true;
        }
    }

    public function providerConfirmEmail(Request $request)
    {
        try {
            $token = $request->token;
            $provider = $this->providers::where(['email_confirm_token'=>$token])->first();
            if($provider)
            {
                $provider->email_confirmed = "yes";
                $provider->email_confirm_token = null;
                $provider->save();

                /* Push and db notification */
                $notificationSetting = NotificationSetting::where('provider_id',$this->user_id)->first();
                if($notificationSetting->show_notification == 'active'){
                    (new ProviderNotificationController)->providerConfirmEmail($this->user_id);
                }
                /* End push and db notification */

                return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_EMAIL_VERIFIED')]);
            }
            else
            {
                return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.TOKEN_EXPIRED')]);
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function profile()
    {
        try {
            $provider = Provider::with([
                'licenses.licenseFiles'=>function($query){
                    $query->select('id','provider_license_id','file','type','document_type');
                },
                'certifications.certificationFiles'=>function($query){
                    $query->select('id','provider_certification_id','file','type','document_type');
                },
                'fbiBackgrounds.fbiBackgroundFiles'=>function($query){
                    $query->select('id','provider_fbi_background_id','file','type','document_type');
                },
                'childAbuseHistories.childAbuseFiles'=>function($query){
                    $query->select('id','provider_child_abuse_history_id','file','type','document_type');
                },
                'liabilityInsurances.liabilityInsuranceFiles'=>function($query){
                    $query->select('id','provider_liability_insurance_id','file','type','document_type');
                },
                'professionalReferences'=>function($query){
                    $query->select('*');
                },
                'specializations'=>function($query){
                    $query->select('*');
                },
                'languages'=>function($query){
                    $query->select('*');
                }
            ])
            ->select('id','first_name','last_name','email','birth_date','gender','other_gender','phone_code','phone','address','apartment','swift_number','experience','bio','image','rating','reviews','email_confirmed','other_specialization','other_language','application_status','timezone')
            ->where(['id'=>$this->user_id])
            ->first();
            if (empty($provider)) {
                throw new Exception(config('service.PROVIDER_API.MESSAGE.PROVIDER_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_DETAILS'), 'records' => $provider]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function updateProviderProfileStep1(Request $request)
    {
        $message = array(
            'birth_date.before_or_equal' => config('service.PROVIDER_API.MESSAGE.BIRTH_DATE')
        );
        $validator = Validator::make($request->all(), [
            'first_name' => 'required|max:100',
            'last_name' => 'required|max:100',
            'birth_date' => 'required|date_format:m/d/Y|before_or_equal:'.\Carbon\Carbon::now()->subYears(18)->format('m/d/Y'),
            'gender' => 'required',
            'experience' => 'required|max:100',
            'bio' => 'required|max:500',
            'address' => 'required|max:2000',
            'phone_code' => 'required|max:5',
            //'phone' => "required|regex:/^([0-9\s\-\+\(\)]*)$/|min:10|max:10|unique:providers,phone,$this->user_id",
            'phone' => "required|unique:providers,phone,$this->user_id",
        ],$message);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try {
            DB::beginTransaction();
            $input = $request->all();
            if(!empty($input['image']))
            {
                $fileExtension = substr(strrchr($input['image'],'.'),1);
                $allExtension = array("jpg", "jpeg", "png");
                if(!in_array($fileExtension, $allExtension))
                {
                    return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROFILE_FILE_EXTENSION')]);
                }
            }
            $updateProvider = Provider::find($this->user_id);
            if(!empty($input['image']))
            {
                $updateProvider->image = $input['image'];
            }
            $updateProvider->first_name = $input['first_name'];
            $updateProvider->last_name = $input['last_name'];
            $updateProvider->birth_date = $input['birth_date'];
            $updateProvider->gender = $input['gender'];
            $updateProvider->other_gender = !empty($input['other_gender']) ? $input['other_gender']:'';
            $updateProvider->experience = $input['experience'];
            $updateProvider->bio = $input['bio'];
            if(!empty($input['timezone'])){
                $updateProvider->timezone = $input['timezone'];
            }
            $updateProvider->address = $input['address'];
            $updateProvider->phone_code = $input['phone_code'];
            $updateProvider->phone = $input['phone'];
            $updateProvider->apartment = $input['apartment'];
            $updateProvider->swift_number = $input['swift_number'];
            $provider = $updateProvider->save();
            if($provider)
            {
                $oauth = $this->oauth_user::where(['user_fk'=>$this->user_id])
                ->update(['mobile'=>$request->phone]);
                DB::commit();
                $provider = Provider::find($this->user_id);
                $response['image']   =  $provider->image;
                $response['first_name']   =  $provider->first_name;
                $response['last_name']    =  $provider->last_name;
                $response['birth_date']   =  $provider->birth_date;
                $response['gender']   =  $provider->gender;
                $response['other_gender']   =  $provider->other_gender;
                $response['phone']   =  $provider->phone;
                $response['address']   =  $provider->address;
                $response['experience']   =  $provider->experience;
                $response['bio']   =  $provider->bio;

                /* Push and db notification */
                $notificationSetting = NotificationSetting::where('provider_id',$this->user_id)->first();
                if($notificationSetting->show_notification == 'active'){
                    (new ProviderNotificationController)->profileUpdate($this->user_id);
                }
                /* End push and db notification */

                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_UPDATE_STEP_1'), 'records' => $response]);
            }
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function forgotPasswordOtp(Request $request)
    {
        $message = array(
            'email.exists'   => config('service.PROVIDER_API.MESSAGE.NOTEXSIST_EMAIL'),
            'phone.exists'   => config('service.PROVIDER_API.MESSAGE.NOTEXSIST_PHONE'),
            'email.email'   => config('service.PROVIDER_API.MESSAGE.VALID_EMAIL')
        );
        $validator = Validator::make($request->all(), [
            'email' => 'required_without:phone|exists:providers',
            'phone' => 'required_without:email|exists:providers',
        ],$message);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try{
            $digits = 4;
            $otpCode = rand(pow(10, $digits-1), pow(10, $digits)-1);
            if(!empty($request->email)){
                $provider= Provider::where(['email'=>$request->email])->first();
            }
            if(!empty($request->phone)){
                $provider= Provider::where(['phone'=>$request->phone])->first();
            }
            if(!empty($provider))
            {

                $validityTime = config('service.PROVIDER_API.FORGOT_PASS_OTP_EXPIRE_TIME');
                $otpValidity  = Carbon::now()->addMinutes($validityTime)->toDateTImeString();
                $provider->otp = $otpCode;
                $provider->otp_validity = $otpValidity;
                $provider->save();

                //send otp
                $templateName = "Provider forgot password";
                $templateData = EmailTemplate::where('name',$templateName)->first();
                if($templateData)
                {
                    //dd($templateData->title);
                    $greeting = config('service.PROVIDER_API.MESSAGE.DEAR').decryptString($provider->first_name) . ',';
                    //$body     = config('service.PROVIDER_API.MESSAGE.EMAIL_OTP')." ".$otpCode;
                    $body     = $templateData->text." ".$otpCode;
                    $type     = $templateData->title;

                    //customer mail
                    $sendMail = new ProviderForgotPasswordMailJob(decryptString($provider->email), $greeting, $body, $type);
                    $this->dispatchNow($sendMail);
                }
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.EMAIL_OTP_SENT')]);
            } else{
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.SOMETHING_WENT_WRONG')]);
            }
        } catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }
    public function verifyForgotPasswordOtp(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'otp' => 'required',
            'email' => 'required_without:phone',
            'phone' => 'required_without:email',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try {

                if(!empty($request->email) && !empty($request->otp))
                {
                    $provider = Provider::where(['email'=>$request->email,'otp'=>$request->otp])->first();
                }
                if(!empty($request->phone) && !empty($request->otp))
                {
                    $provider = Provider::where(['phone'=>$request->phone,'otp'=>$request->otp])->first();
                }
                if($provider)
                {
                    $otpValidity = strtotime($provider->otp_validity);
                    $currentDateTime = strtotime(date('Y-m-d h:i:s'));

                    if($currentDateTime < $otpValidity){
                        $provider->otp = null;
                        $provider->otp_validity = null;
                        $provider->save();
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.OTP_VERIFY')]);
                    }else{
                        $provider->otp = null;
                        $provider->otp_validity = null;
                        $provider->save();
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' =>config('service.PROVIDER_API.MESSAGE.OTP_EXPIRED')]);
                    }
                }
                else
                {
                    return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' =>config('service.PROVIDER_API.MESSAGE.INVALID_OTP')]);
                }

        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function forgotPassword(Request $request)
    {
        $message = array(
            'password.regex' => config('service.PROVIDER_API.MESSAGE.PASSWORD_REGEX'),
        );
        $validator = Validator::make($request->all(), [
            'email' => 'required_without:phone',
            'phone' => 'required_without:email',
            'password' => [
            'required',
            'min:8', // must be at least 8 characters in length
            'max:30', // must be max 30 characters in length
            'regex:/[a-z]/', // must contain at least one lowercase letter
            'regex:/[A-Z]/', // must contain at least one uppercase letter
            'regex:/[0-9]/', // must contain at least one digit
            'regex:/[@$!%*#?&]/', // must contain a special character
            ],
            'password_confirmation' => 'required|same:password',
            ],$message);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try
        {
            if(!empty($request->email)){     
                $provider = Provider::where('email', $request->email)->first();
            }
            if(!empty($request->phone)){
                $provider = Provider::where('phone', $request->phone)->first();
            }
            if(!$provider) {
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS') , 'message' => config('service.PROVIDER_API.MESSAGE.INVALID_USER') ]);
            }
            $oldPassword = UserPasswordHistory::where(
                                [
                                    'user_id' => $provider->id,
                                    'email' => $provider->email
                                ]
                           )
                           ->orderBy('id', 'desc')
                           ->take(config('service.LAST_PASSWORD_COUNT'))
                           ->pluck('password')
                           ->toArray();
            if(count($oldPassword)){
                foreach ($oldPassword as $key => $password) {
                    if (sha1($request->password) == $password) {
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.OLD_PASSWORD_CHECK')]);
                    }
                }
            }
            DB::beginTransaction();  
            $provider->password = sha1($request->password);
            $provider->save();
            $save_data['password'] = sha1($request->password);
            $result = $this->oauth_user::where('user_fk', $provider->id)
            ->update($save_data);

            /* user password history */
               $userPasswordHistory = new UserPasswordHistory();
               $userPasswordHistory->user_id = $provider->id;
               $userPasswordHistory->email = $provider->email;
               $userPasswordHistory->password = sha1($request->password);
               $userPasswordHistory->save();
            /* user password history  */

            DB::commit();
            $templateName = "Provider password reset";
            $templateData = EmailTemplate::where('name',$templateName)->first();
            if($templateData)
            {
                $greeting = 'Hi ' .decryptString($provider->first_name).' '.decryptString($provider->last_name).',';
                //$body     = 'Your password to login as Provider on PM Now Provider APP has been recently changed';
                $body     = $templateData->text;
                $type     = $templateData->title;

                $sendMail = new ProviderMailJob(decryptString($provider->email), $greeting, $body, $type);
                $this->dispatchNow($sendMail);
            }
            
            /* Push and db notification */
            $providerId = $provider->id;
            $notificationSetting = NotificationSetting::where('provider_id',$providerId)->first();
            if($notificationSetting->show_notification == 'active'){
                (new ProviderNotificationController)->forgotPassword($providerId);
            }
            /* End push and db notification */

            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS') , 'message' => config('service.PROVIDER_API.MESSAGE.PASSWORD_UPDATED') ]);
        }
        catch(Exception $e)
        {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS') , 'message' => $e->getMessage()]);
        }
        catch(Throwable $e)
        {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS') , 'message' => $e->getMessage()]);
        }
    }
    public function changePassword(Request $request)
    {
        $message = array(
            'new_password.regex' => config('service.PROVIDER_API.MESSAGE.PASSWORD_REGEX')
        );

        $validator = Validator::make($request->all(), [
            'current_password' => 'required',
            'new_password' => [
                'required',
                'min:8', // must be at least 8 characters in length
                'regex:/[a-z]/', // must contain at least one lowercase letter
                'regex:/[A-Z]/', // must contain at least one uppercase letter
                'regex:/[0-9]/', // must contain at least one digit
                'regex:/[@$!%*#?&]/', // must contain a special character
                ],
                'confirm_password' => 'required|same:new_password',
            ], $message);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }

        try {
            $currentPassword = $request->current_password;
            $currentPasswordEncrypted = $this->hashPassword($currentPassword);
            $provider = Provider::where('password', $currentPasswordEncrypted)
            ->where('id', $this->user_id)
            ->first();
            if (!isset($provider)) {

                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => config('service.PROVIDER_API.MESSAGE.WRONG_CURRENT_PASSWORD')],200);
            }
            $newPassword = $this->hashPassword($request->new_password);
            if ($newPassword == $provider->password) {
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.CURRENT_PASSWORD')]);
            }
            $oldPassword = UserPasswordHistory::where(
                                [
                                    'user_id' => $this->user_id,
                                    'email' => $provider->email
                                ]
                           )
                           ->orderBy('id', 'desc')
                           ->take(config('service.LAST_PASSWORD_COUNT'))
                           ->pluck('password')
                           ->toArray();
            if(count($oldPassword)){
                foreach ($oldPassword as $key => $password) {
                    if ($newPassword == $password) {
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.OLD_PASSWORD_CHECK')]);
                    }
                }
            }               
            DB::beginTransaction();
            $provider = Provider::find($this->user_id);
            $provider->password = $newPassword;
            $provider->save();
            $save_data['password'] = $newPassword;
            $result = DB::table('oauth_users')
            ->where('user_fk', $this->user_id)
            ->update($save_data);

            /* user password history */
                $userPasswordHistory = new UserPasswordHistory();
                $userPasswordHistory->user_id = $provider->id;
                $userPasswordHistory->email = $provider->email;
                $userPasswordHistory->password = $newPassword;
                $userPasswordHistory->save();
            /* user password history  */
            DB::commit();
            /*$greeting = 'Hi ' . $provider->first_name . '!';
            $body     = ' Your password has been changed successfully!';
            $type     =  'Password Updated';
            $sendMail = new SendMailJob($provider->email, $greeting, $body, $type);
            dispatch($sendMail);*/

            /* Push and db notification */
            $notificationSetting = NotificationSetting::where('provider_id',$this->user_id)->first();
            if($notificationSetting->show_notification == 'active'){
                (new ProviderNotificationController)->changePassword($this->user_id);
            }
            /* End push and db notification */

        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PASSWORD_UPDATED')]);
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function providerAvailabilityCreate(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'day'  => 'required|array',
            "day.*"  => "required|string",
            /*'first_half'  => 'required|array',
            "first_half.*"  => "required|string",
            'second_half'  => 'required|array',
            "second_half.*"  => "required|string", */
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try {
            DB::beginTransaction();
            $input = $request->all();

            if(!empty($input['day']))
            {
                if(!empty($input['first_half']['start_time']) && !empty($input['first_half']['end_time']))
                {
                    if(strtotime($input['first_half']['start_time']) > strtotime($input['first_half']['end_time'])){
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.FIRST_HALF_TIME_CHECK')]);
                    }
                    if((strtotime($input['first_half']['end_time']) - strtotime($input['first_half']['start_time'])) < config('service.PROVIDER_API.AVAILABILITY_TIME_INTERVEL')){
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.FIRST_HALF_TIME_INTERVAL')]);
                    }
                }
                if(!empty($input['first_half']['start_time']) && !empty($input['first_half']['end_time']) && !empty($input['second_half']['start_time']))
                {
                    if(strtotime($input['second_half']['start_time']) < strtotime($input['first_half']['start_time'])){
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.SECOND_HALF_START_TIME_F_S')]);
                    }
                    if(strtotime($input['second_half']['start_time']) < strtotime($input['first_half']['end_time'])){
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.SECOND_HALF_START_TIME_F_E')]);
                    }
                }

                if(!empty($input['second_half']['start_time']) && !empty($input['second_half']['end_time']))
                {
                    if(strtotime($input['second_half']['start_time']) > strtotime($input['second_half']['end_time'])){
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.SECOND_HALF_TIME_CHECK')]);
                    }
                    if((strtotime($input['second_half']['end_time']) - strtotime($input['second_half']['start_time'])) < config('service.PROVIDER_API.AVAILABILITY_TIME_INTERVEL')){
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.SECOND_HALF_TIME_INTERVAL')]);
                    }
                }
                
                foreach ($input['day'] as $key => $day) {
                    if(!empty($input['first_half']['start_time']) && !empty($input['first_half']['end_time']))
                    {
                        $checkDay = ProviderAvailability::where(['provider_id'=>$this->user_id,'day'=>$day,'type'=>'first_half'])
                        ->first();
                        if($checkDay){
                            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.CHECK_AVAILABILITY_EXIST').$day]);
                        }

                        $availability = new ProviderAvailability();
                        $availability->provider_id = $this->user_id;
                        $availability->day = $day;
                        // $availability->start_time = convertTimeLocalToUtc($input['first_half']['start_time'],request()->time_zone);
                        // $availability->end_time = convertTimeLocalToUtc($input['first_half']['end_time'],request()->time_zone);
                        $availability->start_time = changeTimeFormat24hrs($input['first_half']['start_time']);
                        $availability->end_time = changeTimeFormat24hrs($input['first_half']['end_time']);
                        $availability->type = 'first_half';
                        $availability->save();

                        /* create add slots */
                            $this->createSlots('first_half',$day,$input['first_half']['start_time'],$input['first_half']['end_time']);
                            DB::commit();
                        /* create add slots */
                    }
                    if(!empty($input['second_half']['start_time']) && !empty($input['second_half']['end_time']))
                    {
                        $checkDay = ProviderAvailability::where(['provider_id'=>$this->user_id,'day'=>$day,'type'=>'second_half'])
                        ->first();
                        if($checkDay){
                            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.CHECK_AVAILABILITY_EXIST').$day]);
                        }
                        $availability = new ProviderAvailability();
                        $availability->provider_id = $this->user_id;
                        $availability->day = $day;
                        // $availability->start_time = convertTimeLocalToUtc($input['second_half']['start_time'],request()->time_zone);
                        // $availability->end_time = convertTimeLocalToUtc($input['second_half']['end_time'],request()->time_zone);
                        $availability->start_time = changeTimeFormat24hrs($input['second_half']['start_time']);
                        $availability->end_time = changeTimeFormat24hrs($input['second_half']['end_time']);
                        $availability->type = 'second_half';
                        $availability->save();

                        /* create add slots */
                            $this->createSlots('second_half',$day,$input['second_half']['start_time'],$input['second_half']['end_time']);
                           DB::commit(); 
                        /* create add slots */
                    }
                }

                $templateName = "Provider availability set";
                $templateData = EmailTemplate::where('name',$templateName)->first();
                if($templateData)
                {
                    /* Availability set email */
                        $provider = Provider::findOrFail($this->user_id);
                        $greeting = 'Hi ' . decryptString($provider->first_name)." ".decryptString($provider->last_name) . ',';
                        $body     = $templateData->text;
                        $type     = $templateData->title;
                        $sendMail = new ProviderCommonMailJob(decryptString($provider->email), $greeting, $body, $type);
                        $this->dispatchNow($sendMail);
                    /* End email  */
                }

                /* Push and db notification */
                $notificationSetting = NotificationSetting::where('provider_id',$this->user_id)->first();
                if(!empty($notificationSetting) && $notificationSetting->show_notification == 'active'){
                    (new ProviderNotificationController)->providerAvailabilityCreate($this->user_id);
                }
                /* End push and db notification */
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_AVAILABILITY')]);
            }
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function providerAvailabilityUpdate(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'day'           => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }

        try {
            $input = $request->all();
            $availability = ProviderAvailability::where(['day'=>$input['day'], 'provider_id'=>$this->user_id])->first();
            if($availability)
            {
                if(!empty($input['first_half']['start_time']) && !empty($input['first_half']['end_time']))
                {
                    if(strtotime($input['first_half']['start_time']) > strtotime($input['first_half']['end_time'])){
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.FIRST_HALF_TIME_CHECK')]);
                    }
                    if((strtotime($input['first_half']['end_time']) - strtotime($input['first_half']['start_time'])) < config('service.PROVIDER_API.AVAILABILITY_TIME_INTERVEL')){
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.FIRST_HALF_TIME_INTERVAL')]);
                    }
                }
                if(!empty($input['second_half']['start_time']) && !empty($input['second_half']['end_time']))
                {
                    if(strtotime($input['second_half']['start_time']) > strtotime($input['second_half']['end_time'])){
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.SECOND_HALF_TIME_CHECK')]);
                    }
                    if((strtotime($input['second_half']['end_time']) - strtotime($input['second_half']['start_time'])) < config('service.PROVIDER_API.AVAILABILITY_TIME_INTERVEL')){
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.SECOND_HALF_TIME_INTERVAL')]);
                    }
                }
            if(!empty($input['first_half']['start_time']) && !empty($input['first_half']['end_time']))
            {
                if(!empty($input['not_available']) && $input['not_available']=="first_half"){
                    $availability = ProviderAvailability::where(['provider_id' => $this->user_id, 'day'=>$input['day'],'type'=>'first_half'])->first();
                    if($availability){

                        ProviderAvailability::where(['provider_id' => $this->user_id, 'day' => $input['day'],'type'=>'first_half'])->delete();
                        ProviderSlot::where(['provider_id' => $this->user_id, 'day' => $input['day'],'type'=>'first_half'])->delete();

                    }
                }else{
                    $checkAvailability = ProviderAvailability::where([
                        'day'=>$input['day'],
                        'provider_id'=> $this->user_id,
                        'type'=>'first_half'
                    ])->first();
                    if($checkAvailability){
                        $availability = ProviderAvailability::where([
                            'day'=>$input['day'],
                            'provider_id'=> $this->user_id,
                            'type'=>'first_half'
                        ])
                        ->update([
                            'start_time'=>$input['first_half']['start_time'],
                            'end_time'=>$input['first_half']['end_time'],
                        ]);
                        /* create add slots */
                            $this->updateSlots('first_half',$input['day'],$input['first_half']['start_time'],$input['first_half']['end_time']);
                        /* create add slots */
                    }else{
                        $availability = new ProviderAvailability();
                        $availability->provider_id = $this->user_id;
                        $availability->day = $input['day'];
                        $availability->start_time = $input['first_half']['start_time'];
                        $availability->end_time = $input['first_half']['end_time'];
                        $availability->type = 'first_half';
                        $availability->save();

                         /* create add slots */
                            $this->updateSlots('first_half',$input['day'],$input['first_half']['start_time'],$input['first_half']['end_time']);
                        /* create add slots */
                    }
                }
            }
            if(!empty($input['second_half']['start_time']) && !empty($input['second_half']['end_time']))
            {
                if(!empty($input['not_available']) && $input['not_available']=="second_half"){
                    $availability = ProviderAvailability::where(['provider_id' => $this->user_id, 'day'=>$input['day'],'type'=>'second_half'])->first();
                    if($availability){
                        ProviderAvailability::where(['provider_id' => $this->user_id, 'day' => $input['day'],'type'=>'second_half'])->delete();
                        ProviderSlot::where(['provider_id' => $this->user_id, 'day' => $input['day'],'type'=>'second_half'])->delete();
                    }
                }else{
                    $checkAvailability = ProviderAvailability::where([
                        'day'=>$input['day'],
                        'provider_id'=> $this->user_id,
                        'type'=>'second_half'
                    ])->first();
                    if($checkAvailability){
                        $availability = ProviderAvailability::where([
                            'day'=>$input['day'],
                            'provider_id'=> $this->user_id,
                            'type'=>'second_half'
                        ])
                        ->update([
                            'start_time'=>changeTimeFormat24hrs($input['second_half']['start_time']),
                            'end_time'=>changeTimeFormat24hrs($input['second_half']['end_time']),
                        ]);
                        /* create add slots */
                            $this->updateSlots('second_half',$input['day'],$input['second_half']['start_time'],$input['second_half']['end_time']);
                        /* create add slots */
                    }else{
                        $availability = new ProviderAvailability();
                        $availability->provider_id = $this->user_id;
                        $availability->day = $input['day'];
                        $availability->start_time = changeTimeFormat24hrs($input['second_half']['start_time']);
                        $availability->end_time = changeTimeFormat24hrs($input['second_half']['end_time']);
                        $availability->type = 'second_half';
                        $availability->save();

                        /* create add slots */
                            $this->updateSlots('second_half',$input['day'],$input['second_half']['start_time'],$input['second_half']['end_time']);
                        /* create add slots */
                    }
                }
            }

            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_AVAILABILITY_UPDATED')]);
            }
            else
            {
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.SOMETHING_WENT_WRONG')]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function createSlots($half_type,$day,$start_time,$end_time)
    {
        $startDateTime = strtotime(date("m/d/Y")." ".$start_time);
        $endDateTime = strtotime(date("m/d/Y")." ".$end_time);
        /*$startTime = strtotime(convertTimeLocalToUtc($start_time,request()->time_zone));
        $endTime    = strtotime(convertTimeLocalToUtc($end_time,request()->time_zone));*/

        for ($i = $startDateTime; $i < $endDateTime; $i+=2700) {     // 3600 second => 1 hour 
             if(($endDateTime - $i) >= 2700){
                 $providerSlot = new ProviderSlot();
                 $providerSlot->provider_id = $this->user_id;
                 $providerSlot->day = $day;
                 $providerSlot->slot = date("H:i", $i);
                 $providerSlot->type = $half_type;
                 $providerSlot->save();
             }
        }
    }

    public function updateSlots($half_type,$day,$start_time,$end_time)
    {
        ProviderSlot::where(['provider_id' => $this->user_id,'day' => $day,'type' => $half_type])->delete();
        $startDateTime = strtotime(date("m/d/Y")." ".$start_time);
        $endDateTime = strtotime(date("m/d/Y")." ".$end_time);

        /*$startTime = strtotime(convertTimeLocalToUtc($start_time,request()->time_zone));
        $endTime    = strtotime(convertTimeLocalToUtc($end_time,request()->time_zone));*/

        for ($i = $startDateTime; $i < $endDateTime; $i+=2700) {     // 3600 second => 1 hour 
            if(($endDateTime - $i) >= 2700){
             $providerSlot = new ProviderSlot();
             $providerSlot->provider_id = $this->user_id;
             $providerSlot->day = $day;
             $providerSlot->slot = date("H:i", $i);
             $providerSlot->type = $half_type;
             $providerSlot->save();
            }
        }
    }

    public function providerTestAvailabilityCreate(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'day'  => 'required|array',
            "day.*"  => "required|string",
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
    
        try {
            $input = $request->all();

            if(!empty($input['day']))
            {
                if(!empty($input['first_half']['start_time']) && !empty($input['first_half']['end_time']))
                {
                    if(strtotime($input['first_half']['start_time']) > strtotime($input['first_half']['end_time'])){
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.FIRST_HALF_TIME_CHECK')]);
                    }
                    //dd(strtotime($input['first_half']['end_time']) - strtotime($input['first_half']['start_time']));
                    if((strtotime($input['first_half']['end_time']) - strtotime($input['first_half']['start_time'])) < config('service.PROVIDER_API.AVAILABILITY_TIME_INTERVEL')){
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.FIRST_HALF_TIME_INTERVAL')]);
                    }
                }
                if(!empty($input['first_half']['start_time']) && !empty($input['first_half']['end_time']) && !empty($input['second_half']['start_time']))
                {
                    if(strtotime($input['second_half']['start_time']) < strtotime($input['first_half']['start_time'])){
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.SECOND_HALF_START_TIME_F_S')]);
                    }
                    if(strtotime($input['second_half']['start_time']) < strtotime($input['first_half']['end_time'])){
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.SECOND_HALF_START_TIME_F_E')]);
                    }
                }
                if(!empty($input['second_half']['start_time']) && !empty($input['second_half']['end_time']))
                {
                    if(strtotime($input['second_half']['start_time']) > strtotime($input['second_half']['end_time'])){
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.SECOND_HALF_TIME_CHECK')]);
                    }
                    if((strtotime($input['second_half']['end_time']) - strtotime($input['second_half']['start_time'])) < config('service.PROVIDER_API.AVAILABILITY_TIME_INTERVEL')){
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.SECOND_HALF_TIME_INTERVAL')]);
                    }
                }
                foreach ($input['day'] as $key => $day) {
                    if(!empty($input['first_half']['start_time']) && !empty($input['first_half']['end_time']))
                    {
                        $checkDay = ProviderAvailability::where(['provider_id'=>$this->user_id,'day'=>$day,'type'=>'first_half'])
                        //->whereDate('created_at', date('Y-m-d'))
                        ->first();
                        if($checkDay){
                            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.CHECK_AVAILABILITY_EXIST').$day]);
                        }

                        $availability = new ProviderAvailability();
                        $availability->provider_id = $this->user_id;
                        $availability->day = $day;
                        $availability->start_time = convertTimeLocalToUtc($input['first_half']['start_time'],request()->time_zone);
                        $availability->end_time = convertTimeLocalToUtc($input['first_half']['end_time'],request()->time_zone);
                        $availability->type = 'first_half';
                        $availability->save();

                        /* create add slots */
                            $this->createSlots($day,$input['first_half']['start_time'],$input['first_half']['end_time']);
                        /* create add slots */


                    }
                    if(!empty($input['second_half']['start_time']) && !empty($input['second_half']['end_time']))
                    {
                        $checkDay = ProviderAvailability::where(['provider_id'=>$this->user_id,'day'=>$day,'type'=>'second_half'])
                        //->whereDate('created_at', date('Y-m-d'))
                        ->first();
                        if($checkDay){
                            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.CHECK_AVAILABILITY_EXIST').$day]);
                        }
                        $availability = new ProviderAvailability();
                        $availability->provider_id = $this->user_id;
                        $availability->day = $day;
                        $availability->start_time = convertTimeLocalToUtc($input['second_half']['start_time'],request()->time_zone);
                        $availability->end_time = convertTimeLocalToUtc($input['second_half']['end_time'],request()->time_zone);
                        $availability->type = 'second_half';
                        $availability->save();

                        /* create add slots */
                            $this->createSlots($day,$input['second_half']['start_time'],$input['second_half']['end_time']);
                        /* create add slots */
                    }
                }

                $templateName = "Provider availability set  ";
                $templateData = EmailTemplate::where('name',$templateName)->first();
                if($templateData)
                {
                    /* Availability set email */
                        $provider = Provider::findOrFail($this->user_id);
                        $greeting = 'Hi ' . decryptString($provider->first_name)." ".decryptString($provider->last_name) . ',';
                        $body     = $templateData->text;
                        $type     = $templateData->title;
                        $sendMail = new ProviderCommonMailJob(decryptString($provider->email), $greeting, $body, $type);
                        $this->dispatchNow($sendMail);
                    /* End email  */
                }

                /* Push and db notification */
                $notificationSetting = NotificationSetting::where('provider_id',$this->user_id)->first();
                if($notificationSetting->show_notification == 'active'){
                    (new ProviderNotificationController)->providerAvailabilityCreate($this->user_id);
                }
                /* End push and db notification */
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_AVAILABILITY')]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function providerAvailabilityList()
    {
        try {
            $date = date('Y-m-d');
            $availabilityList = ProviderAvailability::where(['provider_id'=>$this->user_id])->get();
            $data = [];
            $days = ['Monday'=>0, 'Tuesday'=>1, 'Wednesday'=>2, 'Thursday'=>3,'Friday'=>4,'Saturday'=>5,'Sunday'=>6];
            foreach($availabilityList as $key => $value){

                if($value->type=="first_half"){
                    $data[$days[$value->day]]["first_half"] = [
                        'day'=> $value->day,
                        'start_time'=> date('h:i A',strtotime($date.' '.$value->start_time)),
                        'end_time'=> date('h:i A',strtotime($date.' '.$value->end_time)),
                    ];
                }
                if($value->type=="second_half"){
                    $S_time = $date.' '.$value->start_time;
                    $data[$days[$value->day]]["second_half"] = [
                        'day'=> $value->day,
                        'start_time'=> date('h:i A',strtotime($date.' '.$value->start_time)),
                        'end_time'=> date('h:i A',strtotime($date.' '.$value->end_time)),
                    ];
                }
               
            }
            $data = array_values($data);
        //dd($data);
            if (empty($availabilityList)) {
                throw new Exception(config('service.PROVIDER_API.MESSAGE.AVAILABILITY_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.AVAILABILITY_LIST'), 'records' => $data]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function providerAvailabilityDelete(Request $request, $day, $datetime = null) 
    {
        try {
            $availability = ProviderAvailability::where(['day'=>$day])->first();
            if($availability){
                // $currentDateTime = date('Y-m-d H:i');
                $currentDateTime = urldecode($datetime);
                if($datetime){
                    $currentDateTime = date('Y-m-d H:i');
                }
                
                $appointmentIds = AppointmentCancelled::where('provider_id', $this->user_id)->pluck('appointment_id')->toArray();
                // DB::enableQueryLog();
                $appointment = Appointment::where('provider_id',$this->user_id)
                ->where(DB::raw('CONCAT(date," ",time)'), '>=', $currentDateTime)
                ->where('day',$day)
                ->whereNotIn('id', $appointmentIds)
                ->first();
                // $query = DB::getQueryLog();
                // dd($query);
                if($appointment){
                    return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' =>config('service.PROVIDER_API.MESSAGE.APPOINTMENT_EXIST')]);
                }
                ProviderAvailability::where(['provider_id' => $this->user_id, 'day'=>$day])->delete();
                ProviderSlot::where(['provider_id' => $this->user_id, 'day' => $day])->delete();
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_AVAILABILITY_DELETED')]);
            }
            else{
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.SOMETHING_WENT_WRONG')]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }
    public function providerManageTimeOffCreate(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'reason'       => 'required|max:200',
            'start_date'   => 'required|date',
            'start_time'   => 'required',
            'end_date'     => 'required|date',
            'end_time'     => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $validator->errors()->first()],200);
        }

        try {
            $input = $request->all();

            if(strtotime($input['start_date']) < strtotime(currentDate()))
            {
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.START_DATE')]);
            }
            if(strtotime($input['end_date']) < strtotime($input['start_date']))
            {
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.END_DATE')]);
            }
            if(strtotime($input['start_date']) == strtotime($input['end_date']))
            {
                if(strtotime($input['end_time']) < strtotime($input['start_time']))
                {
                    return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.END_TIME_VALIDATION')]);
                }
            }
            $startDate = dateFormat($input['start_date']);
            $endDate = dateFormat($input['end_date']);
            $period = CarbonPeriod::create($startDate, $endDate);
            // Iterate over the period
            foreach ($period as $date) {
                $date = $date->format('Y-m-d');
                $startTime = strtotime(convertTimeLocalToUtc($input['start_time'],request()->time_zone));
                $endTime    = strtotime(convertTimeLocalToUtc($input['end_time'],request()->time_zone));
                for ($i = $startTime; $i <= $endTime; $i+=60) {     // 3600 second => 1 hour
                    $utcTime = gmdate("H:i", $i);


                    $appointmentIds = AppointmentCancelled::where('provider_id', $this->user_id)->pluck('appointment_id')->toArray();
                    $appointment = Appointment::where(['provider_id'=>$this->user_id,'date'=> $date,'time'=> $utcTime])->whereNotIn('id', $appointmentIds)->first();
                    if($appointment){
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.NOT_APPLY_PROVIDER_TIMEOFF')]);
                    }
                }
            }
            $timeOff = new ProviderTimeOff();
            $timeOff->provider_id = $this->user_id;
            $timeOff->reason     = $input['reason'];
            $timeOff->start_date = $startDate;
            $timeOff->start_time = convertTimeLocalToUtc($input['start_time'],request()->time_zone);
            $timeOff->end_date   = $endDate;
            $timeOff->end_time   = convertTimeLocalToUtc($input['end_time'],request()->time_zone);
            $timeOff->save();

            /* Time off mail */
            $provider = Provider::findOrFail($this->user_id);
            $greeting = 'Hi ' .decryptString($provider->first_name)." ".decryptString($provider->last_name). ',';
            $body     = "You have added ".$input['start_date']."(".$input['start_time'].")". " to ". $input['end_date'] ."(".$input['end_time'].")". " as days or time off. If you wish to change this, please login to your app to manage day-offs now.";
            $type     = 'PM Now | Days Off Added';
            $sendMail = new ProviderTimeOffMailJob(decryptString($provider->email), $greeting, $body, $type);
            $this->dispatchNow($sendMail);
            /* End time off mail */

            /* Push and db notification */
            $notificationSetting = NotificationSetting::where('provider_id',$this->user_id)->first();
                if($notificationSetting->show_notification == 'active'){
                (new ProviderNotificationController)->providerManageTimeOffCreate($this->user_id);
            }
            /* End push and db notification */

            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_TIMEOFF')]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function providerManageTimeOffUpdate(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'id'           => 'required',
            'reason'       => 'required|max:200',
            'start_date'   => 'required|date',
            'start_time'   => 'required',
            'end_date'     => 'required|date',
            'end_time'     => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }

        try {
            $input = $request->all();
            $timeOff = ProviderTimeOff::where(['id'=>$input['id'], 'provider_id'=>$this->user_id])->first();
            if($timeOff)
            {
                if(strtotime($input['start_date']) < strtotime(currentDate()))
                {
                    return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.START_DATE')]);
                }
                if($input['end_date'] < $input['start_date'])
                {
                    return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.END_DATE')]);
                }
                $timeOff->reason     = $input['reason'];
                $timeOff->start_date = dateFormat($input['start_date']);
                $timeOff->start_time = changeTimeFormat24hrs($input['start_time']);
                $timeOff->end_date   = dateFormat($input['end_date']);
                $timeOff->end_time   = changeTimeFormat24hrs($input['end_time']);
                $timeOff->save();

                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_TIMEOFF_UPDATED')]);
            }
            else
            {
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.SOMETHING_WENT_WRONG')]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function providerManageTimeOffList()
    {
        try {
            $timeOff = ProviderTimeOff::where(['provider_id'=>$this->user_id])->orderBy('id', 'desc')->get();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_TIME_OFF_LIST'), 'records' => $timeOff]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function providerManageTimeOffDelete(Request $request, $id) {
        $timeOff = ProviderTimeOff::findOrFail($id);
        try {
            $timeOff  = $timeOff->delete();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_TIME_OFF_DELETED')]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }

    public function providerMyAppointments(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'date'         => 'required|date',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try {
            $input = $request->all();
            $appDate = dateFormat($input['date']);
            //$appointmentIds = AppointmentCancelled::where('provider_id', $this->user_id)->pluck('appointment_id')->toArray();
            $result['appointments'] = Appointment::with(['customer'=>function($query){
                $query->select('id','first_name','last_name','email','address','image');
            }])
            ->where(['provider_id'=>$this->user_id,'date'=>$appDate])
            //->whereNotIn('id', $appointmentIds)
            ->orderBy('id','desc')
            ->get();

            $prev_day  = date('Y-m-d', strtotime($input['date']." -1 day"));
            $curr_day  = date('Y-m-d', strtotime($input['date']));
            $next_day  = date('Y-m-d', strtotime($input['date']." +1 day"));
            $dates = [$prev_day, $curr_day, $next_day];

            $availableSots = [];
            foreach ($dates as $date) {
                $day  = date('l', strtotime($date));
                $slots = ProviderSlot::where(['provider_id' => $this->user_id, 'day' => $day])->get();
                if($slots->count()){
                    foreach ($slots as $key => $value)
                    {     
                        $datetime = $date." ".$value->slot;
                        /* convert data time utc to local  */
                        $slotDateTime = strtotime(convertDateTimeUtcToLocal($datetime, request()->time_zone));
                        /* convert data time utc to local  */

                        $appointment = Appointment::with(['cancell_appointment'])->where(['provider_id' => $this->user_id,'date' => $date,'time'=> $value->slot])->orderBy('id','desc')->first();
                        if($appointment && empty($appointment->cancell_appointment)){
                            $availableSots[date("Y-m-d", $slotDateTime)][] =  [
                                'from_time' => date("h:i A", $slotDateTime),
                                'available' => 'No',
                            ];
                        }else{
                            $availableSots[date("Y-m-d", $slotDateTime)][] =  [
                                'from_time' => date("h:i A", $slotDateTime),
                                'available' => 'Yes',
                            ];
                        }
                    }
                }
            }

            $allSlots = !empty($availableSots[$curr_day]) ? $availableSots[$curr_day] : [];

            /* provider applied day off  */
            $leaves = [];
            foreach ($allSlots as $key => $slot) {
                $providerTimeOff = ProviderTimeOff::where(['provider_id' => $this->user_id])->get();
                foreach ($providerTimeOff as $key => $value) {
                    $startDate = dateFormat($value->start_date);
                    $endDate = dateFormat($value->end_date);

                    if(($curr_day >= $startDate)  && ($curr_day <= $endDate))
                    {
                        if((strtotime($curr_day." ".$slot['from_time']) >= strtotime($curr_day." ".$value->start_time) && strtotime($curr_day." ".$slot['from_time']) < strtotime($curr_day." ".$value->end_time)))
                        {
                            $leaves[] =  [
                                'from_time' => $slot['from_time'],
                                'available' => $slot['available']
                            ];
                        }
                    }
                }
            }
            /* provider applied day off  */

            $finalSlots = checkLeave($allSlots, $leaves);

            $result['first_half']= [];
            $result['second_half']= [];
            foreach($finalSlots as $key => $value){
                if(strtotime($value['from_time']) <= strtotime(date("12:00 PM"))){
                    $result['first_half'][] =  [
                        'from_time' => $value['from_time'],
                        'to_time' => gmdate("h:i A", strtotime($value['from_time'])+2700),
                        'available' => $value['available']
                    ];
                }else{
                     $result['second_half'][] =  [
                        'from_time' => $value['from_time'],
                        'to_time' => gmdate("h:i A", strtotime($value['from_time'])+2700),
                        'available' => $value['available']
                    ];
                }
            }


            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_APPOINTMENT_LIST'), 'records' => $result]);

        } catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function providerMyAppointmentsV2(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'date'         => 'required|date',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try {
            $input = $request->all();
            $appDate = dateFormat($input['date']);
            //$appointmentIds = AppointmentCancelled::where('provider_id', $this->user_id)->pluck('appointment_id')->toArray();
            $result['appointments'] = Appointment::with(['customer'=>function($query){
                $query->select('id','first_name','last_name','email','address','image');
            }])
            ->where(['provider_id'=>$this->user_id,'date'=>$appDate])
            //->whereNotIn('id', $appointmentIds)
            ->orderBy('id','desc')
            ->get();

            // print_r($result);
            // exit;
            $prev_day  = date('Y-m-d', strtotime($input['date']." -1 day"));
            $curr_day  = date('Y-m-d', strtotime($input['date']));
            $next_day  = date('Y-m-d', strtotime($input['date']." +1 day"));
            $dates = [$prev_day, $curr_day, $next_day];

            $availableSots = [];
            foreach ($dates as $date) {
                $day  = date('l', strtotime($date));
                $slots = ProviderSlot::where(['provider_id' => $this->user_id, 'day' => $day])->get();

                if($slots->count()){
                    foreach ($slots as $key => $value)
                    {     
                        if(!empty($slots[$key+1])){
                            if(strtotime($value->slot) > strtotime($slots[$key+1]->slot)){
                                /* convert data time utc to local  */  
                                // $slotDateTime = strtotime($date." ".convertTimeUtcToLocal($value->slot,request()->time_zone));
                                $slotDateTime = strtotime($date." ".$value->slot);
                                /* convert data time utc to local  */
                            }else{
                                $datetime = $date." ".$value->slot;
                                /* convert data time utc to local  */
                                $slotDateTime = strtotime($datetime);
                                // $slotDateTime = strtotime(convertDateTimeUtcToLocal($datetime, request()->time_zone));
                                /* convert data time utc to local  */
                            }
                        }else{
                            $datetime = $date." ".$value->slot;
                                /* convert data time utc to local  */
                            // $slotDateTime = strtotime(convertDateTimeUtcToLocal($datetime, request()->time_zone));
                            $slotDateTime = strtotime($datetime);
                        }
                        
                        
                        $appointment = Appointment::with(['cancell_appointment'])->where(['provider_id' => $this->user_id,'date' => $date,'time'=> $value->slot])->orderBy('id','desc')->first();
                        if($appointment && empty($appointment->cancell_appointment)){
                            $availableSots[date("Y-m-d", $slotDateTime)][] =  [
                                'from_time' => date("h:i A", $slotDateTime),
                                'available' => 'No',
                            ];
                        }else{
                            $availableSots[date("Y-m-d", $slotDateTime)][] =  [
                                'from_time' => date("h:i A", $slotDateTime),
                                'available' => 'Yes',
                            ];
                        }
                    }
                }
            }
           

            $allSlots = !empty($availableSots[$curr_day]) ? $availableSots[$curr_day] : [];
            
            /* provider applied day off  */
            $leaves = [];
            foreach ($allSlots as $key => $slot) {
                $providerTimeOff = ProviderTimeOff::where(['provider_id' => $this->user_id])->get();
                foreach ($providerTimeOff as $key => $value) {
                    $startDate = dateFormat($value->start_date);
                    $endDate = dateFormat($value->end_date);

                    if(($curr_day >= $startDate)  && ($curr_day <= $endDate))
                    {
                        if((strtotime($curr_day." ".$slot['from_time']) >= strtotime($curr_day." ".$value->start_time) && strtotime($curr_day." ".$slot['from_time']) < strtotime($curr_day." ".$value->end_time)))
                        {
                            $leaves[] =  [
                                'from_time' => $slot['from_time'],
                                'available' => $slot['available']
                            ];
                        }
                    }
                }
            }
         
            /* provider applied day off  */

            $finalSlots = checkLeave($allSlots, $leaves);
            
            $result['first_half']= [];
            $result['second_half']= [];
            foreach($finalSlots as $key => $value){
                if(strtotime($value['from_time']) < strtotime(date("12:00 PM"))){
                    $result['first_half'][] =  [
                        'from_time' => $value['from_time'],
                        // 'to_time' => convertTimeUtcToLocal(gmdate("h:i A", strtotime($value['from_time'])+2700)),
                        'to_time' => date("h:i A", strtotime($value['from_time'])+2700),
                        'available' => $value['available']
                    ];
                }else{
                     $result['second_half'][] =  [
                        'from_time' => $value['from_time'],
                        // 'to_time' => convertTimeUtcToLocal(gmdate("h:i A", strtotime($value['from_time'])+2700)),
                        'to_time' => date("h:i A", strtotime($value['from_time'])+2700),
                        'available' => $value['available']
                    ];
                }
            }

            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_APPOINTMENT_LIST'), 'records' => $result]);

        } catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function old_providerMyAppointments(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'date'         => 'required|date',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try {
            $input = $request->all();
            $appDate = dateFormat($input['date']);
            //$appointmentIds = AppointmentCancelled::where('provider_id', $this->user_id)->pluck('appointment_id')->toArray();
            $result['appointments'] = Appointment::with(['customer'=>function($query){
                $query->select('id','first_name','last_name','email','address','image');
            }])
            ->where(['provider_id'=>$this->user_id,'date'=>$appDate])
            //->whereNotIn('id', $appointmentIds)
            ->orderBy('id','desc')
            ->get();

            $day  = date('l', strtotime($input['date']));
            $availabilityLists = ProviderAvailability::where(['provider_id'=>$this->user_id,'day'=>$day])->get();
            $fData['f_half']= [];
            $sData['s_half']= [];
            if($availabilityLists->count()){
                foreach ($availabilityLists as $key => $value)
                {
                    if($value->type=="first_half")
                    {
                        $startTime = strtotime(convertTimeUtcToLocal($value->start_time,request()->time_zone));
                        $endTime    = strtotime(convertTimeUtcToLocal($value->end_time,request()->time_zone));
                        $fData['f_half'] = [];
                        for ($i = $startTime; $i < $endTime; $i+=2700) {     // 3600 second => 1 hour
                            $time = gmdate("h:i A", $i);
                        //dd($time);
                            $utcTime = convertTimeLocalToUtc($time,request()->time_zone);
                        //dd($utcTime);
                            $appointment = Appointment::with(['cancell_appointment'])->where(['provider_id'=>$this->user_id,'date'=>$appDate,'time'=>$utcTime])->first();
                            if($appointment && empty($appointment->cancell_appointment)){
                                $fData['f_half'][] =  [
                                    'from_time' => gmdate("h:i A", $i),
                                    'to_time' => gmdate("h:i A", $i+2700),
                                    'available' => 'No',
                                ];
                            }else{
                                $fData['f_half'][] =  [
                                    'from_time' => gmdate("h:i A", $i),
                                    'to_time' => gmdate("h:i A", $i+2700),
                                    'available' => 'Yes',
                                ];
                            }
                        }
                    }
                    if($value->type=="second_half")
                    {
                        $startTime = strtotime(convertTimeUtcToLocal($value->start_time,request()->time_zone));
                        $endTime    = strtotime(convertTimeUtcToLocal($value->end_time,request()->time_zone));
                        $sData['s_half'] = [];
                        for ($i = $startTime; $i < $endTime; $i+=2700) {     // 3600 second => 1 hour
                            $time = gmdate("h:i A", $i);
                            $utcTime = convertTimeLocalToUtc($time,request()->time_zone);
                            $appointment = Appointment::with(['cancell_appointment'])->where(['provider_id'=>$this->user_id,'date'=>$appDate,'time'=>$utcTime])->first();
                            if($appointment && empty($appointment->cancell_appointment)){
                                $sData['s_half'][] =  [
                                    'from_time' => gmdate("h:i A", $i),
                                    'to_time' => gmdate("h:i A", $i+2700),
                                    'available' => 'No',
                                ];
                            }else{
                                $sData['s_half'][] =  [
                                    'from_time' => gmdate("h:i A", $i),
                                    'to_time' => gmdate("h:i A", $i+2700),
                                    'available' => 'Yes',
                                ];
                            }
                        }
                    }
                }
            }
            $data['first_half_leave'] = [];
            foreach ($fData['f_half'] as $key => $first_half) {
                $providerTimeOff = ProviderTimeOff::where(['provider_id' => $this->user_id])->get();
                foreach ($providerTimeOff as $key => $value) {
                    $startDate = dateFormat($value->start_date);
                    $endDate = dateFormat($value->end_date);

                    if(($appDate >= $startDate)  && ($appDate <= $endDate))
                    {
                        if((strtotime($appDate." ".$first_half['from_time']) >= strtotime($appDate." ".$value->start_time) && strtotime($appDate." ".$first_half['from_time']) < strtotime($appDate." ".$value->end_time)))
                        {
                            $data['first_half_leave'][] =  [
                                'from_time' => $first_half['from_time'],
                                'to_time' => $first_half['to_time'],
                                'available' => $first_half['available']
                            ];
                        }
                    }
                }
            }
            $data['second_half_leave'] = [];
            foreach ($sData['s_half'] as $key => $second_half) {
                $providerTimeOff = ProviderTimeOff::where(['provider_id' => $this->user_id])->get();
                foreach ($providerTimeOff as $key => $value) {
                    $startDate = dateFormat($value->start_date);
                    $endDate = dateFormat($value->end_date);

                    if(($appDate >= $startDate)  && ($appDate <= $endDate))
                    {
                        if((strtotime($appDate." ".$second_half['from_time']) >= strtotime($appDate." ".$value->start_time) && strtotime($appDate." ".$second_half['from_time']) < strtotime($appDate." ".$value->end_time)))
                        {
                            $data['second_half_leave'][] =  [
                                'from_time' => $second_half['from_time'],
                                'to_time' => $second_half['to_time'],
                                'available' => $second_half['available']
                            ];
                        }
                    }
                }
            }
            $arrayFirstHalf = $fData['f_half'];
            $arrayFirstHalfLeave = $data['first_half_leave'];
            $arraySecondHalf = $sData['s_half'];
            $arraySecondHalfLeave = $data['second_half_leave'];
            $result['first_half'] = checkFirstHalfLeave($arrayFirstHalf, $arrayFirstHalfLeave);
            $result['second_half'] = checkSecondHalfLeave($arraySecondHalf, $arraySecondHalfLeave);
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_APPOINTMENT_LIST'), 'records' => $result]);

        } catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function providerAddAppointmentNote(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'id'  => 'required',
            'provider_note'  => 'required|max:2000',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }

        try {
            $input = $request->all();
            $appointment = Appointment::where(['id' => $input['id'],'provider_id' => $this->user_id])->first();

            $appointmentDateTime = $appointment->date_time;
            $appointmentDateTime = date('Y-m-d H:i', $appointmentDateTime);
            $appointmentDateTime = convertDateTimeUtcToLocal($appointmentDateTime,request()->time_zone);
            $currentUtcDateTime = date('Y-m-d H:i');
            $currentDateTime = convertDateTimeUtcToLocal($currentUtcDateTime,request()->time_zone);

            if(strtotime($currentDateTime) <= strtotime($appointmentDateTime))
            {
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.CAN_NOT_CREATE_APPOINTMENT_NOTE')]);
            }

            $audioVideoCallHistory = AudioVideoCallHistory::where(['appointment_id' => $input['id'], 'provider_id' => $this->user_id])->first();
            if(empty($audioVideoCallHistory)){
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_DID_NOT_JOIN_SESSION')]);
            }

            $appointment->provider_note = $input['provider_note'];
            if(!empty($input['appointment_status']) && $input['appointment_status'] == "Completed"){
               $noShow  = NoShow::where(['appointment_id' => $input['id']])->first();
               if($noShow){
                    $noShow->delete();
               }
                $appointment->appointment_status = $input['appointment_status'];
            }else{
                $appointment->appointment_status = "Pending";
            }   
            if($appointment->save()) {

                /* Push and db notification */
                /*$customerId = $appointment->customer_id;
                $notificationSetting = NotificationSetting::where('customer_id',$customerId)->first();
                if($notificationSetting->show_notification == 'active' && $notificationSetting->appointment_reminder == 'active'){
                    (new ProviderNotificationController)->providerAddAppointmentNote($customerId);
                }*/
                /* End push and db notification */

                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.APPOINTMENT_NOTE'), 'records' =>$appointment]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function providerViewAppointment($id)
    {
        try {
            $data['appointment'] = Appointment::with(['customer'=>function($query){
                $query->select('id','first_name','last_name','email','address','image');
            },'provider'=>function($query){
                $query->select('id','first_name','last_name','email','address','image');
            }])
            ->where(['id'=>$id,'provider_id'=>$this->user_id])
            ->first();

            $customer = Customer::where(['id' => $data['appointment']->customer_id,'current_provider_id'=>$this->user_id])->first();
            if($customer)
            {

                $data['total_journal'] = Journal::where(['customer_id'=>$data['appointment']->customer_id])->count();
                $data['journals'] = Journal::where(['customer_id'=>$data['appointment']->customer_id])->get();
            }
            $data['intake_questions_options'] = CustomerIntakeQuestion::with(['question','intakeQuestionAnswer.answer'])->where(['customer_id'=>$data['appointment']->customer_id])->get();
            if (empty($data['appointment'])) {
                throw new Exception(config('service.PROVIDER_API.MESSAGE.APPOINTMENT_NOT_FOUND'));
            }
            $uniq_sender_reciever_id = $data['appointment']->customer_id . '-'. $this->user_id;
            $uniqueRoomId = $this->generateUniqueRoomID($uniq_sender_reciever_id);
            $roomExist = MessageRoom::where(['room_id'=>$uniqueRoomId])->first();
            if($roomExist){
                $data['room_id'] = $uniqueRoomId;
            }else{
                $data['room_id'] = "";
            }
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.APPOINTMENT_DETAILS'), 'records' => $data]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function providerRatingReviews(Request $request)
    {
        try {
            $ratingReviews['rating_review'] = ProviderRatingReview::with(['customer'=>function($query){
                $query->select('id','first_name','last_name','email','address','image');
            }])
            ->where(['provider_id'=>$this->user_id,'published'=>'active'])
            ->orderBy('id', 'desc')
            ->get();

            // total rating review
            $ratingReviews['total_rating_review'] = $ratingReviews['rating_review']->count();

            // average rating
            $ratingReviews['avg_rating_review'] = number_format((float)$ratingReviews['rating_review']->avg('session_rating'), 1, '.', '');

            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_RATING_REVIEWS'), 'records' => $ratingReviews]);

        } catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function providerSetPin(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'pin' => 'required|max:4',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try {
            $provider = Provider::find($this->user_id);
            $provider->pin = $request->pin;
            $provider->save();

            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_SET_PIN')]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function providerChangePin(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'current_pin' => 'required|max:4',
            'new_pin' => 'required|max:4',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try {
            $provider = Provider::where(['pin'=>$request->current_pin,'id'=>$this->user_id])->first();
            if($provider){
                $provider->pin = $request->new_pin;
                $provider->save();
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_CHANGE_PIN')]);
            }else{
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.CURRENT_PIN_WRONG')]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function providerResetPin(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'email' => 'required|exists:providers',
            'new_pin' => 'required|max:4',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try {
            $provider = Provider::where(['email'=>$request->email,'id'=>$this->user_id])->first();
            if($provider){
                $provider->pin = $request->new_pin;
                $provider->save();
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_RESET_PIN')]);
            }else{
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.WRONG_EMAIL')]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function providerFollowUpMessage(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'appointment_id'  => 'required',
            'customer_id'  => 'required',
            'follow_up_messages'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try {
            DB::beginTransaction();
            $input = $request->all();
            if(!empty($input['follow_up_messages']))
            {
                foreach($input['follow_up_messages'] as $follow_up_message)
                {
                    $followUpMessage = new FollowUpMessage();
                    $followUpMessage->appointment_id = $request->appointment_id;
                    $followUpMessage->customer_id =  $request->customer_id;
                    $followUpMessage->provider_id = $this->user_id;
                    $followUpMessage->time = $follow_up_message['time'];
                    $followUpMessage->message = $follow_up_message['message'];
                    if($followUpMessage->save()){
                        $currentDateTime = Carbon::now();
                        $newDateTime = Carbon::now()->addHours($follow_up_message['time']);
                        $scheduledDateTime = $newDateTime->format('Y-m-d H:i');
                        $scheduledNotification = new ScheduledNotification();
                        $scheduledNotification->title = 'Follow up Message';
                        $scheduledNotification->message = $follow_up_message['message'];
                        $scheduledNotification->sender_id = $this->user_id;
                        $scheduledNotification->receiver_id = $request->customer_id;
                        $scheduledNotification->send_to = 'customer';
                        $scheduledNotification->scheduled_date_time = $scheduledDateTime;
                        $scheduledNotification->save();
                    }

                }
                DB::commit();
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.FOLLOW_UP_MESSAGE')]);
            }
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function providerChatRoom(Request $request)
    {
        try
        {
            $all_room = MessageRoom::has('customer')
            ->with(['customer' => function($q) {
                $q->select(['id', 'first_name', 'last_name', 'image']);
            },'provider' => function($q) {
                $q->select(['id', 'first_name', 'last_name', 'image']);
            },
            'unread_message' => function($q) {
                $q->select('room_id', DB::raw('COUNT(id) as unread_message_count'))
                ->where('status', '=', 0)
                ->groupBy('room_id');
            }]);

            $all_room = $all_room->where([['provider_id', '=', $this->user_id],['deleted_by_provider','=',null]])->orderBy('last_message_date_time', 'desc')->paginate(50);
            return response()->json(['status'=> config('service.SUCCESS_STATUS'), 'message' => config('service.ROOM_LIST'), 'records' => $all_room]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function getMessageById(Request $request, $id)
    {
        try
        {
            $roomId = $id;
            $room = MessageRoom::where(['room_id'=>$roomId])->first();
            if(!empty($room))
            {
                $order = "desc";
                $limit = isset($request->limit) ? $request->limit : 25;
                if($limit > 500) {
                    $limit = 25;
                }
                if(isset($request->order) && strtolower($request->order)=="asc") {
                    $order = $request->order;
                }
                $order_by_column_name = "id";
                if(isset($request->order_by_column_name)) {
                    if(in_array($request->order_by_column_name, $this->columnName())) {
                        $order_by_column_name = $request->order_by_column_name;
                    }
                }
                /*$all_message = MessageRoomChat::with(['customer' => function($q) {
                $q->select(['id', 'first_name', 'last_name', 'image']);
                }])*/
                $all_message = MessageRoomChat::where(function($q) use($room,$request) {
                    $q->where([
                        ['room_id', '=', $room->room_id],
                //['contractor_delete_chat_status', '=', 'ACTIVE'],
                    ]);
                })
                ->where(function($q) use($request) {
                // Global searching...
                    if(isset($request->search) && !empty($request->search) && $request->search!="") {
                        if(!empty($this->columnName())) {
                            foreach ($this->columnName() as $key => $col) {
                                if($key==0) {
                                    $q->where($col, 'like', '%' .$request->search. '%');
                                } else {
                                    $q->orWhere($col, 'like', '%' .$request->search. '%');
                                }
                            }
                        }
                    }
                })
                ->where(function($q) use($request) {
                // Column wise searching filter ...
                    $allowableColumn = $this->columnName();
                    if(isset($request->column_name) && !empty($request->column_name) && $request->column_name!="" && in_array($request->column_name, $allowableColumn)) {
                        $q->where($request->column_name,  'like', '%' .$request->column_value. '%');
                    }
                })
                ->orderBy($order_by_column_name, $order)
                ->paginate(50);

                // find current provider
                $customer = Customer::select('current_provider_id','plan_id','appointment_session')->where(['id' => $room->customer_id, 'current_provider_id' => $room->provider_id])->first();
                if(!empty($customer->current_provider_id)){
                    $isCurrentProvider = "yes";
                }else{
                    $isCurrentProvider = "no";   
                }
                if(!empty($customer->plan_id) && $customer->appointment_session > 0){
                    $isSessionActive = "yes"; 
                }else{
                    $isSessionActive = "no"; 
                }


                return response()->json(['status'=> config('service.SUCCESS_STATUS'), 'message' => config('service.CHAT_MESSAGES_LIST'),'is_current_provider' => $isCurrentProvider, 'is_session_active' => $isSessionActive, 'records' => $all_message]);
            }
            else
            {
                throw new Exception(config('service.ROOM_NOT_EXISTS'));
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    private function columnName() {
        $col_name = [];
        $col = MessageRoomChat::first();
        if(!empty($col) && $col!="") {
            return array_keys($col->toArray());
        }
        return ['id'];
    }
    public function sendMessageById(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'message' => isset($request->message) ? 'required|max:256' : "",
            'room_id' => !empty($request->room_id) ? 'required|max:256' : "",
            'appointment_id' => 'required',
            //'firebase_sender_id' => 'required',
            //'firebase_peer_id' => 'required',
            'image' => $request->has('image') ? 'required|mimes:jpeg,png,jpg,svg,gif' : "",
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try
        {

            /* appointment cancel request */
                if(!empty($request->provider_cancel_request)){
                    $appointment = Appointment::find($request->appointment_id);
                    if(strtotime(date("Y-m-d H:i")) > $appointment->date_time){
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.APPOINTMENT_CANCEL_REQUEST_PROVIDER')]);
                    }
                }
            /*  appointment cancel request  */

            $uniq_sender_reciever_id = $request->id . '-'. $this->user_id;
            $uniqueRoomId = $this->generateUniqueRoomID($uniq_sender_reciever_id);
            $roomId = !empty($request->room_id) ? $request->room_id : $uniqueRoomId;
            $room = MessageRoom::where('room_id',$roomId)->first();
            if(!empty($room))
            {
                // Update room with latest message
                $room = MessageRoom::find($room->id);
                $input['last_message'] = $request->message;
                $input['appointment_id'] = $request->appointment_id;
                $input['last_message_date_time'] = date('Y-m-d H:i:s');
                $room->update($input);
                $reciever_id = !empty($room->customer_id) ? $request->id : $room->customer_id;

                // save user message
                $input['room_id_db'] = $this->user_id . '-' .$reciever_id;
                $input['sender_id']   = $this->user_id;
                $input['sender_type']   = "provider";
                $input['receiver_type']   = "customer";
                $input['receiver_id'] = $reciever_id;
                $input['room_id'] = $roomId;
                $input['message'] = $request->message;
                $input['msg_date_time'] = date('Y-m-d H:i:s');
                $input['firebase_sender_id'] = !empty($request->firebase_sender_id) ? $request->firebase_sender_id:'';
                $input['firebase_peer_id'] = !empty($request->firebase_peer_id) ? $request->firebase_peer_id:'';
                if($request->has('image')) {
                /*$image = $request->file('image');
                $imagename = time().'.'.$image->getClientOriginalName();
                $image->move(public_path('assets/images/chats'),$imagename);
                $input['image'] = $imagename;*/
                }
                $message = MessageRoomChat::create($input);

                /* appointment cancel request */
                    if(!empty($request->provider_cancel_request)){
                        $appointment = Appointment::find($request->appointment_id);
                        $appointment->provider_cancel_request = $request->provider_cancel_request;
                        $appointment->save();
                    }
                /*  appointment cancel request  */


                $resp = MessageRoomChat::with(['customer' => function($q) {
                    $q->select(['id', 'first_name', 'last_name', 'image']);
                }])->with('chat_room')->find($message->id);

                /* Push and db notification */
                $notificationSetting = NotificationSetting::where('customer_id',$reciever_id)->first();
                if($notificationSetting->show_notification == 'active' && $notificationSetting->new_message_notification == 'active'){
                    (new ProviderNotificationController)->providerSendMessage($reciever_id);
                }
                /* End push and db notification */

                return response()->json(['status'=> config('service.SUCCESS_STATUS'), 'message' => config('service.CHAT_MESSAGE_SENT'), 'records' => $resp]);
            }
            else
            {
            // check sender & reciever pair is already exists or not
                $checkRoomExists = MessageRoom::where('room_id', '=', $roomId)->exists();
                if($checkRoomExists) {
                    throw new Exception("Sorry you have already exists room_id pair with this customer. please use room_id to send message to this customer");
                }
                $input['customer_id'] = $request->id;
                $input['provider_id'] = $this->user_id;
                $input['room_id'] = $roomId;
                $input['last_message'] = $request->message;
                $input['appointment_id'] = $request->appointment_id;
                $input['last_message_date_time'] = date('Y-m-d H:i:s');
                MessageRoom::create($input);
                // save user message
                $input['room_id_db'] = $this->user_id . '-' . $request->id;
                $input['sender_id']   = $this->user_id;
                $input['sender_type']   = "provider";
                $input['receiver_type']   = "customer";
                $input['receiver_id'] = $request->id;
                $input['message'] = $request->message;
                $input['msg_date_time'] = date('Y-m-d H:i:s');
                $input['firebase_sender_id'] = !empty($request->firebase_sender_id) ? $request->firebase_sender_id:'';
                $input['firebase_peer_id'] = !empty($request->firebase_peer_id) ? $request->firebase_peer_id:'';
                if($request->has('image')) {
                /*$image = $request->file('image');
                $imagename = time().'.'.$image->getClientOriginalName();
                $image->move(public_path('assets/images/chats'),$imagename);
                $input['image'] = $imagename;*/
                }
                $message = MessageRoomChat::create($input);

                /* appointment cancel request */
                    if(!empty($request->provider_cancel_request)){
                        $appointment = Appointment::find($request->appointment_id);
                        $appointment->provider_cancel_request = $request->provider_cancel_request;
                        $appointment->save();
                    }
                /*  appointment cancel request  */

                $resp = MessageRoomChat::with(['customer' => function($q) {
                    $q->select(['id', 'first_name', 'last_name', 'image']);
                }])->with('chat_room')->find($message->id);


                /* Push and db notification */
                $notificationSetting = NotificationSetting::where('customer_id',$request->id)->first();
                if($notificationSetting->show_notification == 'active' && $notificationSetting->new_message_notification == 'active'){
                    (new ProviderNotificationController)->providerSendMessage($request->id);
                }
                /* End push and db notification */

                return response()->json(['status'=> config('service.SUCCESS_STATUS'), 'message' => config('service.CHAT_MESSAGE_SENT'), 'records' => $resp]);
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    private function generateUniqueRoomID($uniq_sender_reciever_id) {
        return md5($uniq_sender_reciever_id);
    }
    public function deleteChatRoom(Request $request, $id)
    {
        try {

            $room   = MessageRoom::findorFail($id);
            $room->deleted_by_provider = date("Y-m-d");
            if($room->save()){
                return response()->json(['status'=> config('service.SUCCESS_STATUS'), 'message' => config('service.ROOM_DELETE')]);
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function providerAudioVideoSessionList(Request $request)
    {
        try
        {
            $audioVideoCallHistory = AudioVideoCallHistory::with(['provider'=>function($query){
                $query->select('id','first_name','last_name','image');
            },'customer'=> function($q) {
                $q->select(['id', 'first_name', 'last_name', 'image']);
            }, 'appointment'=> function($q) {
                $q->select(['id', 'date', 'time', 'end_time']);
            }])
            ->where(['provider_id'=>$this->user_id])
            ->whereNotNull('customer_id')
            ->orderBy('id','desc')
            ->paginate(10);
            if($audioVideoCallHistory->count()){
                return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.AUDIO_VIDEO_SESSION_LIST'), 'records' => $audioVideoCallHistory]);
            }else{
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.AUDIO_VIDEO_SESSION_NOT_FOUND')]);
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function providerTwoFactorEnableDisable(Request $request) 
    {
        $validator = Validator::make($request->all(), [
            'two_factor' =>  'required|max:60',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try {
            $provider = $this->providers::findOrFail($this->user_id);
            $provider->two_factor = $request->two_factor;
            $provider->save();
            if($request->two_factor=="enable"){
                return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.TWO_FACTOR_ENABLE')]);
            }
            if($request->two_factor=="disable"){
                return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.TWO_FACTOR_DISABLE')]);
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function sessionToken(Request $request)
    {
        // date_default_timezone_set('EST');
        $validator = Validator::make($request->all(), [
            'appointment_id'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try {
                $input = $request->all();
                $checkAppointment = Appointment::where(['id' => $input['appointment_id'],'provider_id' => $this->user_id])->where('appointment_status', '!=' , 'Completed')->first();
                if($checkAppointment)
                {   
                    $appointmentDateTime = $checkAppointment->date_time;
                    $appointmentDateTime = date('Y-m-d H:i', $appointmentDateTime);
                    $appointmentDateTime = $appointmentDateTime;
                    $currentUtcDateTime = date('Y-m-d H:i');
                    $currentDateTime = $currentUtcDateTime;

                    $currentDateTime = $input['current_date_time'];
                    
                    $appointmentDateTime = $checkAppointment['date'].' '.$checkAppointment['time'];
                     if(strtotime($currentDateTime) >= strtotime($appointmentDateTime) && strtotime($currentDateTime) <= strtotime("+45 minutes", strtotime($appointmentDateTime)))
                    {
                        $checkToken = DB::table('zoom_session_tokens')->select('token','session_name')->where(['appointment_id' => $input['appointment_id']])->first();
                        if($checkToken)
                        {
                            $data = [
                                    'token' => $checkToken->token,
                                    'session_name' => $checkToken->session_name,
                                    'call_end_time' => (strtotime($checkAppointment->end_time_utc) - strtotime(convertTimeUtcToLocalv3(date('H:i'),'EDT'))) / 60,
                                ];
                            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_SESSION_TOKEN'),'records' => $data]);
                        }
                        else
                        { 
                            /*  generate unique session name */
                            $sessionId = DB::table('zoom_session_tokens')->orderBy('id','desc')->value('id');
                            $sessionId = ($sessionId)?$sessionId+1:1;
                            $sessionName = 'SESSION-'.str_pad($sessionId,2,"0",STR_PAD_LEFT);
                            /*  end session name */

                            $headers = ['alg'=>'HS256','typ'=>'JWT'];
                            $headers_encoded = $this->base64url_encode(json_encode($headers));

                            $payload = [
                                'app_key' => env('ZOOM_API_KEY', ''),
                                'version' => 1,
                                'role_type' => 1,
                                'iat' => strtotime(date('Y-m-d', strtotime('+1 days'))),
                                'exp' => strtotime("+45 minutes", strtotime(date('Y-m-d', strtotime('+1 days')))),
                                'tpc' => $sessionName
                                ];
                            $payload_encoded = $this->base64url_encode(json_encode($payload));

                            //build the signature
                            $secretKey = env('ZOOM_API_SECRET', '');
                            $signature = hash_hmac('SHA256',"$headers_encoded.$payload_encoded",$secretKey,true);
                            $signature_encoded = $this->base64url_encode($signature);

                            //build and return the token
                            $token = "$headers_encoded.$payload_encoded.$signature_encoded";
                            

                            $data = DB::table('zoom_session_tokens')->insert(
                                ['appointment_id' => $input['appointment_id'] , 'session_name' => $sessionName,'token' => $token]
                            );

                            $noShowTime = Setting::where('name','noshow')->value('value');

                            if($data){
                                $data = [
                                    'token' => $token,
                                    'session_name' => $sessionName,
                                    'noshow_time' => $noShowTime,
                                    'call_end_time' => (strtotime($checkAppointment->end_time_utc) - strtotime(convertTimeUtcToLocalv3(date('H:i'),'EDT'))) / 60,
                                ];
                                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_SESSION_TOKEN'),'records' => $data]);
                            }
                        }
                    }
                    else
                    {
                        $data = [
                                    'token' => '',
                                    'session_name' => '',
                                    'noshow_time' => ''
                                ];
                        return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.APPOINTMENT_TIME'),'records' => $data]);
                    }  

                }
                else
                {
                    return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.SESSION_TOKEN_NOT_CREATED')]);
                }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function base64url_encode($data) 
    {
        return rtrim(strtr(base64_encode($data), '+/', '-_'), '=');
    }

    public function providerJoinSession(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'appointment_id'  => 'required',
            'customer_id'  => 'required',
            'call_started_time'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }

        try {
            $input = $request->all();
            
            /* No show data */
            $noShow = NoShow::updateOrCreate([
                'appointment_id'   => $input['appointment_id'],
            ],[
                'appointment_id'   => $input['appointment_id'],
                'provider_id'   => $this->user_id,
            ]);
            /* No show data */
            $audioVideoCallHistory = AudioVideoCallHistory::where(['appointment_id'=>$input['appointment_id']])->first();
            if($audioVideoCallHistory) {
                if(empty($audioVideoCallHistory->provider_id)){
                    /* Deduct the appointment session of customer */
                        $customer = Customer::findOrFail($input['customer_id']);
                        $customer->appointment_session = (int)$customer->appointment_session-1;
                        $customer->save();
                    /* End deduct the appointment session of customer */
                }
                $audioVideoCallHistory->provider_id = $this->user_id;
                $audioVideoCallHistory->save();

                /* Push and db notification */
                /*$notificationSetting = NotificationSetting::where('customer_id',$input['customer_id'])->first();
                if(!empty($notificationSetting) && $notificationSetting->show_notification == 'active' && $notificationSetting->call_notification == 'active'){*/
                    (new ProviderNotificationController)->providerJoinSessionAgain($input['customer_id']);
              //  }
                /* End push and db notification */
            }else{
                $audioVideoCallHistory = new AudioVideoCallHistory();
                $audioVideoCallHistory->appointment_id = $input['appointment_id'];
                $audioVideoCallHistory->provider_id = $this->user_id;
                $audioVideoCallHistory->call_started_at = convertTimeLocalToUtc($input['call_started_time'],request()->time_zone);
                $audioVideoCallHistory->save();

                /* Deduct the appointment session of customer */
                    $customer = Customer::findOrFail($input['customer_id']);
                    $customer->appointment_session = (int)$customer->appointment_session-1;
                    $customer->save();
                /* End deduct the appointment session of customer */

                $templateName = "Provider call joined";
                $templateData = EmailTemplate::where('name',$templateName)->first();
                if($templateData)
                {
                    $appointment = Appointment::with(['customer'])->where(['id'=>$input['appointment_id']])->first();
                    if(!empty($appointment->customer)){
                        /* Join session email */
                            $greeting = 'Hi ' . decryptString($appointment->customer->first_name)." ".decryptString($appointment->customer->last_name) . ',';
                            $body     = $templateData->text;
                            $type     = $templateData->title;
                            $sendMail = new ProviderCallJoinMailJob(decryptString($appointment->customer->email), $greeting, $body, $type);
                            $this->dispatchNow($sendMail);
                        /* End Join session email  */
                    }
                }

                /* Push and db notification */
                /*$notificationSetting = NotificationSetting::where('customer_id',$input['customer_id'])->first();
                if(!empty($notificationSetting) && $notificationSetting->show_notification == 'active' && $notificationSetting->call_notification == 'active'){*/
                    (new ProviderNotificationController)->providerJoinSession($input['customer_id']);
                //}
                /* End push and db notification */
            }
            
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_JOIN_SESSION')]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function providerSaveVideoSessionDetail(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'appointment_id'  => 'required',
            //'duration'  => 'required',
            'call_end_time'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }

        try {
            $input = $request->all();
            $audioVideoCallHistory = AudioVideoCallHistory::where(['appointment_id'=>$input['appointment_id'],'provider_id' => $this->user_id])->first();
            if($audioVideoCallHistory){
                if(empty($audioVideoCallHistory->duration)){
                    //$callStartTime = strtotime($audioVideoCallHistory->call_started_at);
                    $callStartTime = strtotime($audioVideoCallHistory->call_started_at_utc);
                    $callEndTime = strtotime(convertTimeLocalToUtc($input['call_end_time'],request()->time_zone));
                    $durationCall = ($callEndTime - $callStartTime) / 60; 
                    $audioVideoCallHistory->duration = $durationCall;
                    $audioVideoCallHistory->call_end_at = convertTimeLocalToUtc($input['call_end_time'],request()->time_zone);
                    $audioVideoCallHistory->save();


                    /* Push and db notification */
                    $notificationSetting = NotificationSetting::where('provider_id',$this->user_id)->first();
                    if($notificationSetting->show_notification == 'active' && $notificationSetting->call_notification == 'active'){
                        (new ProviderNotificationController)->providerSessionCompleted($this->user_id);
                    }
                    /* End push and db notification */
                }
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_SAVE_SESSION_DETAIL')]);
            }

        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function providerNoShowTime(Request $request) 
    {

        try {
            $noShowTime = Setting::where('name','noshow')->value('value');
            if($noShowTime){
                return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.NO_SHOW_TIME'), 'records' => $noShowTime]);
            }
            else{
                return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.NO_SHOW_TIME_NOT_FOUND')]);
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function noShowNotificationToCustomer(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'customer_id'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }

        try {
            $input = $request->all();
            
            $customer = Customer::where(['id'=>$input['customer_id']])->first();
            if($customer){
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Appointment";
                $message = "Provider has joined the session, and waiting for you.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_JOIN_SESSION')]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function sessionDidNotJoinNotificationToCustomer(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'customer_id'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }

        try {
            $input = $request->all();
            
            /* Push and db notification */
                $notificationSetting = NotificationSetting::where('customer_id',$input['customer_id'])->first();
                if($notificationSetting->show_notification == 'active' && $notificationSetting->call_notification == 'active'){
                    (new ProviderNotificationController)->customerDidNotJoinSession($input['customer_id']);
                }
            /* End push and db notification */

            /*  Consumer no show email to admin  */
            $templateName = "Consumer No Show";
            $templateData = EmailTemplate::where('name',$templateName)->first();
            if($templateData)
            {
                $admin = User::where(['id'=>1])->first();
                $greeting = 'Hi ' . $admin->first_name." ".$admin->last_name . ',';
                $body     = $templateData->text;
                $type     = $templateData->title;
                $link     = env('FRONT_END') . '/' . 'admin/noshow/appointments/';
                $sendMail = new CustomerNoShowEmailJob($admin->email, $greeting, $body, $link, $type);
                $this->dispatchNow($sendMail);
            }
            /* End Consumer no show email to admin   */
                
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_JOIN_SESSION')]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function providerNotifications(Request $request)
    {
        try
        {
            $provider = Provider::where(['id'=>$this->user_id])->first();
            $allNotifications = $provider->notifications()->paginate(10);
            if($allNotifications->count()){
                $data = [];
                foreach ($allNotifications as $key => $allNotification) {
                    $data[] = [
                        'id'=> $allNotification->id,
                        'notifiable_id'=> $allNotification->notifiable_id,
                        'data'=> $allNotification->data,
                        'read_at'=> $allNotification->read_at,
                        'created_at'=> date("Y-m-d H:i",strtotime($allNotification->created_at)),
                    ];
                }
                return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.NOTIFICATION_LIST'), 'records' => $data]);
            }else{
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.NOTIFICATION_NOT_FOUND')]);
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function providerNotificationSetting(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'setting_name'  => 'required|max:191',
            'status'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try {
            $input = $request->all();
            if(!empty($input['setting_name']))
            {
                if($input['setting_name']=="show_notification"){
                    $this->updateNotificationSetting($input['setting_name'],$input['status']);
                }
                elseif($input['setting_name']=="appointment_reminder"){
                    $this->updateNotificationSetting($input['setting_name'],$input['status']);
                }
                elseif($input['setting_name']=="call_notification"){
                    $this->updateNotificationSetting($input['setting_name'],$input['status']);
                }
                elseif($input['setting_name']=="new_message_notification"){
                    $this->updateNotificationSetting($input['setting_name'],$input['status']);
                }
                else{
                    return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.NOTIFICATION_SETTING_NAME')]);
                }
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.NOTIFICATION_SETTING')]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    private function updateNotificationSetting($settingName,$status){
        $updateSetting = NotificationSetting::updateOrCreate([
            'provider_id'   => $this->user_id,
        ],[
            'provider_id'  => $this->user_id,
            $settingName   => $status,
        ]);
        if($updateSetting){
            return true;
        }
    }

    public function providerNotificationSettingList(Request $request)
    {
        try
        {
            $settings = NotificationSetting::select('id','provider_id','show_notification','appointment_reminder','call_notification','new_message_notification')->where(['provider_id'=>$this->user_id])->first();
            if($settings){
                return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.NOTIFICATION_SETTING_LIST'), 'records' => $settings]);
            }else{
                $settings = [
                                'id' => null,
                                'provider_id' => null,
                                'show_notification' => null,
                                'appointment_reminder' => null,
                                'call_notification' => null,
                                'new_message_notification' => null,
                            ];
                return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.NOTIFICATION_SETTING_LIST'), 'records' => $settings]);
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function deleteProviderAccount() 
    {
        try {
            $provider = Provider::find($this->user_id);
            DB::beginTransaction();
            $email = $provider->email;
            $id = $provider->id;
            $provider->delete();
            $this->oauth_user::where(['user_fk' => $id,'email'=>$email])->delete();
            DB::commit();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_DELETED')]);
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }
    


    public function download($file)
    {   
        $file = base64_decode($file);
        
        $imageURL=$file;
        
        
        //get extension of image
        
            
        header("Content-Type: application/octet-stream"); //
        // set it as an attachment and give a file name
        header('Content-Disposition: attachment; filename='.'abc.jpg');            
        //echo $output;

            
        return response()->json(['status'=> config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.NOTIFICATION_SETTING_LIST'), 'records' => $imageURL]);

    }

    
    public function providerMatchPin(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'pin' => 'required|max:4',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try {
            $Provider = Provider::where(['pin'=>$request->pin,'id'=>$this->user_id])->first();
            if($Provider){
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PROVIDER_PIN_MATCH'), 'match_status' => true]);
            }else{
                return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.PROVIDER_API.MESSAGE.PIN_NOT_MATCH'), 'match_status' => false]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.PROVIDER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    

}
?>